# Sulk - missions - MISH_space_hulk_4

#*****************************************************************************
#
#                                   Sulk
#
#       Copyright (c) 2002 Toby Woodwark <zirtix@softhome.net>
#
# This program is free software.  See the file COPYING for details.
#
#******************************************************************************

"""
This module contains constants and functions for a Sulk mission.
"""

from constants.mission_locals import *


####### Strings
NAME = "Cleanse and Burn"

NUM = 4

FAMILY = "Space Hulk"

AUTHOR = "Games Workshop"

INFO = "The Space Marines must cleanse each of two Gene Banks with a flamer hit. \
The Genestealers must prevent this."

STORY = "Techmarine squad 3 has been captured by the Stealers and cryogenically \
stored in Gene Banks.  The Blood Angels' gene seed could not be allowed to fall into enemy \
hands, but there was not enough time to rescue the Techmarines.  Squads Pilgrim \
and Stone were sent to put them out of their misery."

####### Board array

# Four parts:
#               rooms
#	    c o r r i d o r
#	    c o r r i d o r
#		rooms

BOARD = (

## Start of top room part
( #5
((13, 0),),
((13, 1),),
((13, 2),),
((13, 3),),
((13, 4), )
),
( #T
                                ((13,5),),
((12, 6), ),          ((13,6),),          ((14,6),)
),
( #2room
((8, 5),),                      ((9, 5),),          ((10, 5),),
((8, 6),),                      ((9, 6),{O:1,COMMENT:"Gene Bank"}),  ((10, 6),),              ((11, 6), {DOOR:RIGHT}),
((8, 7),),                      ((9, 7),),          ((10, 7),),
                                ((9, 8),{DOOR:UP})
),
( #3room
				((16, 5),),	    ((17, 5),),		((18, 5),),
((15, 6), {DOOR:RIGHT}),	((16, 6),),	    ((17, 6),),		((18, 6),),              ((19, 6), {DOOR:RIGHT}),
				((16, 7),),	    ((17, 7),),		((18, 7),),
						    ((17, 8), {DOOR:UP})
),
( #corner
				((21, 5),),
((20, 6),),           ((21, 6),)
),
( #2
((21, 3),{ENTRY:UP}),
((21, 4),)
),

## Start of upper corridor part
( #corner
((0, 10),),			((1, 10),),
((0, 11),)
),
( #T
                                ((3, 9),),
((2, 10), ),          ((3, 10),),	    ((4, 10),)
),
( #3
((3, 6),{ENTRY:UP}),
((3, 7),),
((3, 8),)
),
( #3
((5, 10), ),          ((6, 10),{DOOR:RIGHT}),((7, 10),)
),
( #X
                                ((9, 9),),     
((8, 10),),		((9, 10),),         ((10, 10),),
                                ((9, 11),)
),
( #2
((11, 10),),		((12, 10),)
),
( #3
((13, 10),),		((14, 10),),	    ((15, 10),)
),
( #X
                                ((17, 9),),     
((16, 10),{DOOR:RIGHT}),	((17, 10),),	    ((18, 10),{DOOR:RIGHT}),
                                ((17, 11),)
),
( #T
((19, 10), ),         ((20, 10),),	    ((21, 10),),
				((20, 11),)
),
( #4
((22, 10),{DOOR:RIGHT}),	((23, 10),),	    ((24, 10),),    ((25, 10),)
),
( #X
                                ((27, 9),),     
((26, 10),),		((27, 10),),	    ((28, 10),),
                                ((27, 11),)
),
( #4
((27, 5),),
((27, 6),),
((27, 7),),
((27, 8), )
),
( #T
((26, 3), {ENTRY:UP}),		((27, 3),),	    ((28, 3),{ENTRY:RIGHT}),
				((27, 4),)
),
( #5
((29, 10),{M:"Pilgrim"}),((30, 10),{M:"Pilgrim"}), ((31, 10),{M:"Pilgrim"}),    ((32, 10),{M:"Pilgrim"}),    ((33, 10),{M:"Pilgrim"}) 
),

## Start of lower corridor part
( #corner
((0, 12),),
((0, 13),),			((1, 13),)
),
( #T
((2, 13), ),          ((3, 13),),	    ((4, 13),),
				((3, 14),)
),
( #3
((3, 15),),
((3, 16),),
((3, 17),{ENTRY:DOWN}),
),
( #3
((5, 13), ),          ((6, 13),{DOOR:RIGHT}),((7, 13),)
),
( #X
                                ((9, 12),),
((8, 13),),		((9, 13),),         ((10, 13),),
                                ((9, 14),),  
),
( #2
((11, 13),),		((12, 13),)
),
( #3
((13, 13),),		((14, 13),),	    ((15, 13),)
),
( #X
                                ((17, 12),),     
((16, 13),{DOOR:RIGHT}),	((17, 13),),	    ((18, 13),{DOOR:RIGHT}),
                                ((17, 14),)
),
( #T
				((20, 12),),
((19, 13), ),         ((20, 13),),	    ((21, 13),),
),
( #4
((22, 13),{DOOR:RIGHT}),	((23, 13),),	    ((24, 13),),    ((25, 13),)
),
( #X
                                ((27, 12),),     
((26, 13),),		((27, 13),),	    ((28, 13),),
                                ((27, 14),)
),
( #4
((27, 15),),
((27, 16),),
((27, 17),),
((27, 18),)
),
( #T
				((27, 19),),
((26, 20), {ENTRY:DOWN}),		((27, 20),),	    ((28, 20),{ENTRY:RIGHT}),
),
( #5
((29, 13),{M:"Stone"}),((30, 13),{M:"Stone"}),((31, 13),{M:"Stone"}),((32, 13),{M:"Stone"}),((33, 13),{M:"Stone"}) 
),

## Start of bottom room part
( #5
((13, 19),),
((13, 20),),
((13, 21),),
((13, 22),),
((13, 23),)
),
( #T
((12, 17), ),		((13,17),),          ((14,17),),
				((13,18),)
),
( #2room
				((9, 15),{DOOR:UP}),
((8, 16),),                     ((9, 16),),          ((10, 16),),
((8, 17),),                     ((9, 17),{O:1,COMMENT:"Gene Bank"}),  ((10, 17),),      ((11, 17), {DOOR:RIGHT}),
((8, 18),),                     ((9, 18),),          ((10, 18),)
),
( #3room
						    ((17, 15), {DOOR:UP}),
				((16, 16),),	    ((17, 16),),	((18, 16),),
((15, 17), {DOOR:RIGHT}),	((16, 17),),	    ((17, 17),),	((18, 17),),        ((19, 17), {DOOR:RIGHT}),
				((16, 18),),	    ((17, 18),),	((18, 18),)
),
( #corner
((20, 17),),          ((21, 17),),
				((21, 18),),
),
( #2
((21, 19),),
((21, 20),{ENTRY:DOWN})
),


)	
	
####### Constants
BEGINPLACE = (12, 10)

FLAGS = ()

MARINES =   (
		(
		"Pilgrim", 
		    ( (TERMINATOR_MARINE_WITH_STORM_BOLTER,MARINETEAM), (TERMINATOR_MARINE_WITH_STORM_BOLTER,MARINETEAM), (TERMINATOR_MARINE_WITH_STORM_BOLTER,MARINETEAM), (TERMINATOR_MARINE_SERGEANT,MARINETEAM), (TERMINATOR_MARINE_WITH_HEAVY_FLAMER,MARINETEAM) )
		),
		(
		"Stone",
		    ( (TERMINATOR_MARINE_WITH_STORM_BOLTER,MARINETEAM), (TERMINATOR_MARINE_WITH_STORM_BOLTER,MARINETEAM), (TERMINATOR_MARINE_WITH_STORM_BOLTER,MARINETEAM), (TERMINATOR_MARINE_SERGEANT,MARINETEAM), (TERMINATOR_MARINE_WITH_HEAVY_FLAMER,MARINETEAM) )
		)
	    )

BLIPS = (0, 2) #begin game with 0 blips; 2 reinf per turn

BLIP_SET = BLIP_SET_BASIC


####### Functions
def pre_deploy_rule(aclass, square, squadname=None):    
    if getattr(aclass.team, 'type', None) == MARINETEAM and square.deploy_attribute != squadname: 
        return False
    return True
    

def victory_check(sulkgame):
    # N.B. In this mission, no draw is possible
    
    marines = sulkgame.marines
    
    # Find the objective squares (of which there are 2)
    if not hasattr(marines, "objsquares"):
	marines.objsquares = []
        for square in sulkgame.squares.sprites():
            if square.objectivenum == 1:
                marines.objsquares.append(square)
		square.cleansed = 0

    # Mark the currently flaming squares as cleansed
    for sq in marines.objsquares:
	if not getattr(sq, 'cleansed', 0) and sq.get_flaming():
	    sq.cleansed = 1
	    sq.comment += " (cleansed)" #?
            
    # Check for marine win
    for sq in marines.objsquares:
	if not sq.cleansed:
	    break
    else: return marines
    
    # Check for stealer win
    for m in marines.sprites():
        if m.type == TERMINATOR_MARINE_WITH_HEAVY_FLAMER and m.ammoremaining:
            break
    else: return sulkgame.stealers

