# Sulk - missions - MISH_space_hulk_6

#*****************************************************************************
#
#                                   Sulk
#
#       Copyright (c) 2002 Toby Woodwark <zirtix@softhome.net>
#
# This program is free software.  See the file COPYING for details.
#
#******************************************************************************

"""
This module contains constants and functions for a Sulk mission.
"""

from constants.mission_locals import *

####### Strings
NAME = "Defend"

NUM = 6

FAMILY = "Space Hulk"

AUTHOR = "Games Workshop"

INFO = "To win, the Space Marines must defend the ducts in the control room for \
16 complete turns.  The Stealers must attack and destroy the ducts. \
Flamer Marines may not fire into or \
out of the control room or adjoining corridor.  Also, they are limited to 4 rounds of \
ammo each."

STORY = "The last two Marine squads moved into position around the cryogenics control room. \
There they were to introduce a lethal toxin into the ducts to kill the dormant Genestealers. \
The Genestealers were aware of this and attempted to breach these fragile ducts before the full \
dose could be released."

####### Board array

# Five parts + a bit:
#		     top
#		 left   right
#		      -
#		      -
# 	 bottom left  -   bottom right

BOARD = (

## Top part
( #1room - control room
((13, 0),{O:2,DUCTING:RIGHT}),	    ((14, 0),{O:2,DUCTING:RIGHT}),  ((15, 0),{O:2,DUCTING:RIGHT}), #no squares here in the original game
((13, 1),{O:1,M:None}),		    ((14, 1),{O:1,M:None}),		    ((15, 1),{O:1,M:None}),
((13, 2),{O:1,M:None}),		    ((14, 2),{O:1,M:None,COMMENT:"Control room"}),((15, 2),{O:1,M:None}),
((13, 3),{O:1,M:None}),		    ((14, 3),{O:1,M:None}),		    ((15, 3),{O:1,M:None}),
				    ((14, 4),{O:1,M:None,DOOR:UP})
),
( #3 - adjoining corridor
((14, 5),{O:1,M:None}),
((14, 6),{O:1,M:None}),
((14, 7),{O:1,M:None})
),
( #T
				((14, 8),{M:None}),
((13, 9),{M:None}),	((14, 9),{M:None}),	    ((15, 9),{M:None})
),

## Central joining parts
( #3
((13,17),{M:None}),	((14,17),{M:None}),	    ((15,17),{M:None})
),
( #3
((13,23),{M:None}),	((14,23),{M:None}),	    ((15,23),{M:None})
),
( #T
((13,28),{M:None}),	((14,28),{M:None}),	    ((15,28),{M:None}),
				((14,29),{M:None,ENTRY:DOWN})
),

## Left part
( #T
((11, 8),{M:None}),
((11, 9),{M:None}),		((12, 9),{M:None,DOOR:RIGHT}),
((11,10),{M:None})
),
( #corner
((10, 6),{M:None}),	((11, 6),{M:None}),
				((11, 7),{M:None})
),
( #5
(( 5, 6),{M:None}),	(( 6, 6),{M:None}),	    (( 7, 6),{M:None}),	(( 8, 6),{M:None}), (( 9, 6),{M:None})
),
( #X
				(( 3, 5),{M:None}),
(( 2, 6),{M:None}),	(( 3, 6),{M:None}),	    (( 4, 6),{M:None}),
				(( 3, 7),{M:None})
),
( #X
				(( 3, 2),{M:None,ENTRY:UP}),
(( 2, 3),{M:None,ENTRY:LEFT}),	(( 3, 3),{M:None}),	    (( 4, 3),{M:None,ENTRY:RIGHT}),
				(( 3, 4),{M:None})
),
( #2
(( 0, 6),{M:None,ENTRY:LEFT}),	(( 1, 6),{M:None})
),
( #4
(( 3, 8),{M:None}),
(( 3, 9),{M:None}),
(( 3,10),{M:None}),
(( 3,11),{M:None})
),
( #T
				(( 3,12),{M:None}),
(( 2,13),{M:None,ENTRY:LEFT}),	(( 3,13),{M:None}),	    (( 4,13),{M:None})
),
( #4
(( 5,13),{M:None}),	(( 6,13),{M:None}),	    (( 7,13),{M:None}),	(( 8,13),{M:None})
),
( #3room
							    ((11,11),{M:None,DOOR:UP}),
					((10,12),{M:None}), ((11,12),{M:None}),	((12,12),{M:None}),
(( 9,13),{M:None,DOOR:RIGHT}),	((10,13),{M:None}), ((11,13),{M:None}),	((12,13),{M:None}),
					((10,14),{M:None}), ((11,14),{M:None}),	((12,14),{M:None}),
							    ((11,15),{M:None,DOOR:UP})
),

## Right part
( #T
				((17, 8),{M:None}),
((16, 9),{M:None,DOOR:RIGHT}),	((17, 9),{M:None}),
				((17,10),{M:None})
),
( #corner
((17, 6),{M:None}),		((18, 6),{M:None}),
((17, 7),{M:None})
),
( #5 mirrorwise
((23, 6),{M:None}),	((22, 6),{M:None}),	    ((21, 6),{M:None}),	((20, 6),{M:None}), ((19, 6),{M:None})
),
( #X mirrorwise
				((25, 5),{M:None}),
((26, 6),{M:None}),		((25, 6),{M:None}),	    ((24, 6),{M:None}),
				((25, 7),{M:None})
),
( #X
				((25, 2),{M:None,ENTRY:UP}),
((26, 3),{M:None,ENTRY:RIGHT}),	((25, 3),{M:None}),	    ((24, 3),{M:None,ENTRY:LEFT}),
				((25, 4),{M:None})
),
( #2 mirrorwise
((28, 6),{M:None,ENTRY:RIGHT}),	((27, 6),{M:None})
),
( #4
((25, 8),{M:None}),
((25, 9),{M:None}),
((25,10),{M:None}),
((25,11),{M:None})
),
( #T mirrorwise
				((25,12),{M:None}),
((26,13),{M:None,ENTRY:RIGHT}),	((25,13),{M:None}),	    ((24,13),{M:None})
),
( #4 mirrorwise
((23,13),{M:None}),		((22,13),{M:None}),	    ((21,13),{M:None}),	((20,13),{M:None})
),
( #3room mirrorwise
						    ((17,11),{M:None,DOOR:UP}),
				((18,12),{M:None}), ((17,12),{M:None}),	((16,12),{M:None}),
((19,13),{M:None,DOOR:RIGHT}),	((18,13),{M:None}), ((17,13),{M:None}),	((16,13),{M:None}),
				((18,14),{M:None}), ((17,14),{M:None}),	((16,14),{M:None}),
						    ((17,15),{M:None,DOOR:UP})
),

## Bottom left part

( #T
((11,16),{M:None}),
((11,17),{M:None}),		((12,17),{M:None}),
((11,18),{M:None})
),
( #3
((11,19),{M:None}),
((11,20),{M:None,DOOR:UP}),
((11,21),{M:None})
),
( #X
				((11,22),{M:None}),
((10,23),{M:None}),	((11,23),{M:None}),	    ((12,23),{M:None}),
				((11,24),{M:None})
),
( #2room
(( 6,22),{M:None}), (( 7,22),{M:None}),	(( 8,22),{M:None}),
(( 6,23),{M:None}), (( 7,23),{M:None}),	(( 8,23),{M:None}), (( 9,23),{M:None,DOOR:RIGHT}),
(( 6,24),{M:None}), (( 7,24),{M:None}),	(( 8,24),{M:None}),
		    (( 7,25),{M:None,DOOR:UP})
),
( #5
(( 7,26),{M:None}),
(( 7,27),{M:None}),
(( 7,28),{M:None}),
(( 7,29),{M:None}),
(( 7,30),{M:None,ENTRY:DOWN})
),
( #2 -- NB original layout map is ambiguous!
((11,25),{M:None}),
((11,26),{M:None})
),
( #corner
((11,27),{M:None}),
((11,28),{M:None}),		((12,28),{M:None})
),

## Bottom right part

( #T
				((17,16),{M:None}),
((16,17),{M:None}),	((17,17),{M:None}), 
				((17,18),{M:None})
),
( #3
((17,19),{M:None}),
((17,20),{M:None,DOOR:UP}),
((17,21),{M:None})
),
( #X
				((17,22),{M:None}),
((16,23),{M:None}),	((17,23),{M:None}),	    ((18,23),{M:None}),
				((17,24),{M:None})
),
( #2room
				((20,22),{M:None}), ((21,22),{M:None}),	((22,22),{M:None}),
((19,23),{M:None,DOOR:RIGHT}),	((20,23),{M:None}), ((21,23),{M:None}),	((22,23),{M:None}),
				((20,24),{M:None}), ((21,24),{M:None}),	((22,24),{M:None}),
						    ((21,25),{M:None,DOOR:UP})
),
( #5
((21,26),{M:None}),
((21,27),{M:None}),
((21,28),{M:None}),
((21,29),{M:None}),
((21,30),{M:None,ENTRY:DOWN}),
),
( #2 -- NB original layout map is ambiguous!
((17,25),{M:None}),
((17,26),{M:None})
),
( #corner
				((17,27),{M:None}),
((16,28),{M:None}),	((17,28),{M:None})
),

)
	
####### Constants
BEGINPLACE = (14, 6)

FLAGS = ()

MARINES =   (
		(
		"Luther", 
		    ( (TERMINATOR_MARINE_WITH_STORM_BOLTER,MARINETEAM), (TERMINATOR_MARINE_WITH_STORM_BOLTER,MARINETEAM), (TERMINATOR_MARINE_WITH_STORM_BOLTER,MARINETEAM), (TERMINATOR_MARINE_SERGEANT,MARINETEAM), (TERMINATOR_MARINE_WITH_HEAVY_FLAMER,MARINETEAM) )
		),
		(
		"Snow",
		    ( (TERMINATOR_MARINE_WITH_STORM_BOLTER,MARINETEAM), (TERMINATOR_MARINE_WITH_STORM_BOLTER,MARINETEAM), (TERMINATOR_MARINE_WITH_STORM_BOLTER,MARINETEAM), (TERMINATOR_MARINE_SERGEANT,MARINETEAM), (TERMINATOR_MARINE_WITH_HEAVY_FLAMER,MARINETEAM) )
		)
	    )

BLIPS = (2, 2) #begin game with 2 blips; 2 reinf per turn

BLIP_SET = BLIP_SET_BASIC


####### Functions
def post_deploy_script(piece):
    if piece.type == TERMINATOR_MARINE_WITH_HEAVY_FLAMER: 
	piece.ammoremaining = 4
    

def post_action_script(piece, actiontype):
    if piece.square.objectivenum == 1 and actiontype == SHOOTING \
    and piece.type == TERMINATOR_MARINE_WITH_HEAVY_FLAMER:
	# kludge -- randomly destroy some ducting!
	# this ensures the marines will lose this turn.
	
	# First find ducting
	print >> DEBUG_STREAM, "Space Hulk mission 6 --> post_action_script, killing ducting" #DEBUG
	for s in piece.team.game.squares.sprites():
	    if s.objectivenum == 2:
		for f in s.features.sprites():
		    if f.type == DUCTING:
			# Destroy it
			f.kill()
			return
   
   
def victory_check(sulkgame):
    # N.B. In this mission, no draw is possible

    # Find ducting and the room squares (the control room plus corridor)
    roomsquares = []
    ductingsquares = []
    for s in sulkgame.squares.sprites():
	if s.objectivenum == 1: roomsquares.append(s)
	elif s.objectivenum == 2: ductingsquares.append(s)
    
    # Check for stealer win by destroying ducting
    for sq in ductingsquares:
	for f in sq.features.sprites():
	    if f.type == DUCTING: break
	else:
	    # No ducting found on a square!
	    return sulkgame.stealers

    # Check for stealer win by room on fire
    for sq in roomsquares:
	if sq.get_flaming(): return sulkgame.stealers


    #Check for stealer win by no marines
    if not sulkgame.marines:
        return sulkgame.stealers

    # Check for marine win KLUDGE
    # (endphase only)
    if sulkgame.phase.__class__.__name__ == "End_Phase" \
    and sulkgame.turnnumber == 16: return sulkgame.marines
