# Sulk - constants - layout

#*****************************************************************************
#
#                                   Sulk
#
#       Copyright (c) 2002 Toby Woodwark <zirtix@softhome.net>
#
# This program is free software.  See the file COPYING for details.
#
#******************************************************************************

"""
Main constants.
"""

##############################################################################
####### Import

import pygame
import sys, getopt, os, time
import ConfigParser
from pygame import Rect

import options


##############################################################################
####### Classes

class _BitBucket:
    """A class to replace sys.stdout if we want no console messages.
    """
    
    def write(self, message):
	pass
	
    def flush(self):
	pass


##############################################################################
####### Go

# How big is the screen area?  Let's get a rough idea.
if options.SCREEN_WIDTH < 480 or options.SCREEN_HEIGHT < 480: 
    raise SystemExit, \
    "Error: geometry %dx%d is too small (minimum: 480x480)."%\
    (options.SCREEN_WIDTH, options.SCREEN_HEIGHT)
_screen_area = options.SCREEN_WIDTH * options.SCREEN_HEIGHT
if _screen_area < 400000: _size = 0
elif _screen_area > 600000: _size = 2
else: _size = 1

del _screen_area

# Are we debugging?
if options.PRINT_DEBUG_INFO: DEBUG_STREAM = sys.stdout
else: DEBUG_STREAM = _BitBucket()

del _BitBucket

# Python
print >> DEBUG_STREAM, "Python version:", sys.version #DEBUG

# Print info now
if options.SULKRC_FILE_NAME:
    print >> DEBUG_STREAM, "Loading config from '%s'"%\
    os.path.abspath(options.SULKRC_FILE_NAME) #DEBUG
print >> DEBUG_STREAM, "Loading missions from directory '%s'"%\
os.path.abspath(options.MISSIONS_DIRECTORY) #DEBUG
print >> DEBUG_STREAM, "Loading images from directory '%s'"%\
os.path.abspath(options.IMAGE_DIRECTORY) #DEBUG
print >> DEBUG_STREAM, "Loading theme from directory '%s'"%\
os.path.abspath(options.THEME_DIRECTORY) #DEBUG
print >> DEBUG_STREAM, "Loading sounds from directory '%s'"%\
os.path.abspath(options.SOUND_DIRECTORY) #DEBUG
print >> DEBUG_STREAM, "Loading fonts from directory '%s'"%\
os.path.abspath(options.FONT_DIRECTORY) #DEBUG

# Speed
MAX_TICK_RATE = 50 #hmm, too fast?
MIN_TICK_LENGTH = float(1000) / MAX_TICK_RATE #float
# Tooltips should arrive after 0.3 sec, die after .1
TOOLTIP_TIME = 300
TOOLTIP_KILL_TIME = 100


##############################################################################
####### Fonts

pygame.font.init()

_font_name = os.path.join(options.FONT_DIRECTORY,'kimberley.ttf')
_mkfont = pygame.font.Font
if os.access(_font_name, os.R_OK):
    # Use nice font
    if _size: #i.e. if we have plenty of room:
	COMMENT_FONT		= _mkfont(_font_name, 25)
	SCROLLMESSAGE_FONT	= _mkfont(_font_name, 22)
	DIALOG_FONT		= _mkfont(_font_name, 16)
	CLICKBOX_FONT		= _mkfont(_font_name, 18)
	AMMOCLOCK_FONT = DISPLAYDIE_FONT = CPCLOCK_FONT \
	= APCLOCK_FONT		= _mkfont(_font_name, 32)
	TURNTIMER_FONT		= _mkfont(_font_name, 32)
	DEPLOYCLOCK_FONT	= _mkfont(_font_name, 20)
    else: #i.e. if we are rather squeezed:
	COMMENT_FONT = SCROLLMESSAGE_FONT = _mkfont(_font_name, 15)
	CLICKBOX_FONT = DIALOG_FONT = _mkfont(_font_name, 14)
	AMMOCLOCK_FONT = DISPLAYDIE_FONT = CPCLOCK_FONT = APCLOCK_FONT =\
	_mkfont(_font_name, 17)
	TURNTIMER_FONT = _mkfont(_font_name, 20)
	DEPLOYCLOCK_FONT =  _mkfont(_font_name, 15)
else:
    # Use the default font, 3px bigger (don't ask)
    print >> DEBUG_STREAM, \
    "Preferred font '%s' could not be accessed, \
falling back to Pygame default"%_font_name #DEBUG
    _font_name = None
    if _size:
        COMMENT_FONT = SCROLLMESSAGE_FONT = _mkfont(_font_name, 28)
        CLICKBOX_FONT = DIALOG_FONT = _mkfont(_font_name, 20)
        AMMOCLOCK_FONT = DISPLAYDIE_FONT = CPCLOCK_FONT = APCLOCK_FONT =\
	_mkfont(_font_name, 35)
        TURNTIMER_FONT = _mkfont(_font_name, 38)
        DEPLOYCLOCK_FONT =  _mkfont(_font_name, 23)
    else:
        COMMENT_FONT = SCROLLMESSAGE_FONT = _mkfont(_font_name, 18)
        CLICKBOX_FONT = DIALOG_FONT = _mkfont(_font_name, 15)
        AMMOCLOCK_FONT = DISPLAYDIE_FONT = CPCLOCK_FONT = APCLOCK_FONT =\
	_mkfont(_font_name, 21)
        TURNTIMER_FONT = _mkfont(_font_name, 21)
        DEPLOYCLOCK_FONT =  _mkfont(_font_name, 18)

del _font_name

_tt_font_name = os.path.join(options.FONT_DIRECTORY,'fudd.ttf')
if os.access(_tt_font_name, os.R_OK):
    if _size:
	TOOLTIP_FONT = _mkfont(_tt_font_name, 12)
    else:
	TOOLTIP_FONT = _mkfont(_tt_font_name, 10)
else:
    print >> DEBUG_STREAM, \
    "Preferred font '%s' could not be accessed, \
falling back to Pygame default"%_tt_font_name #DEBUG
    _tt_font_name = None
    if _size:
	TOOLTIP_FONT = _mkfont(_tt_font_name, 12)
    else:
	TOOLTIP_FONT = _mkfont(_tt_font_name, 10)
	
del _tt_font_name, _mkfont


##############################################################################
####### Visual constants (depending on image sizes etc)

# Find some example images in the options.THEME_DIRECTORY and set scale
# for sprites which are important to layout

try:
    _sqimage	 = pygame.image.load(
		os.path.join(options.THEME_DIRECTORY, 'square_corridor.png'))
    _minisqimage = pygame.image.load(
		os.path.join(options.THEME_DIRECTORY, 'mini_square.png'))
    _buttonimage = pygame.image.load(
		os.path.join(options.IMAGE_DIRECTORY, 'attack_enabled.png'))
    
    SQ_LENGTH = _sqimage.get_rect().width
    MINI_SQ_LENGTH = _minisqimage.get_rect().width
    MINIMAP_BOX_LENGTH = 50 * MINI_SQ_LENGTH #?
    
    # Check that the MINI_SCALE is going to be an integer
    if SQ_LENGTH % MINI_SQ_LENGTH: 
	raise UserWarning, \
	"mini_square.png does not fit precisely into square_corridor.png"

    # Check that the BUTTON_SIZE width is not larger than MINIMAP_BOX_LENGTH 
    BUTTON_SIZE = _buttonimage.get_rect().size
    if BUTTON_SIZE[1] > MINIMAP_BOX_LENGTH: 
	raise UserWarning, "mini_square.png is too small"
    
    #TODO -- better sanity checking
except UserWarning, message:
    print "Your theme is dodgy:", message
except Exception, message:
    print "Your theme is broken!"
    raise SystemExit, message

del _sqimage, _minisqimage, _buttonimage

# Pad screen layout
if _size: PAD = 10
else: PAD = 6
HALF = int(PAD / 2)
 
del _size

# Misc
MINI_SCALE	= int(SQ_LENGTH / MINI_SQ_LENGTH)
SCREEN_SIZE	= (options.SCREEN_WIDTH, options.SCREEN_HEIGHT)
PIECE_RECT	= Rect(0, 0, SQ_LENGTH, SQ_LENGTH)
MINI_PIECE_RECT	= Rect(0, 0, MINI_SQ_LENGTH, MINI_SQ_LENGTH)
MINIMAP_BOX_SIZE = (MINIMAP_BOX_LENGTH, MINIMAP_BOX_LENGTH)
SCROLLMESSAGE_BOX_HEIGHT = TURNTIMER_FONT.get_height() - PAD #hmm

####### Screen layout -- rects

# Layout rects
SCREEN_RECT	= Rect((0, 0), SCREEN_SIZE) #fixed pos
MINIMAP_RECT	= Rect(
(SCREEN_RECT.right - (MINIMAP_BOX_SIZE[0] + PAD), SCREEN_RECT.top + PAD),  #fixed pos
MINIMAP_BOX_SIZE)
SCROLLMESSAGE_RECT = Rect(
SCREEN_RECT.left + PAD, SCREEN_RECT.bottom - (PAD + SCROLLMESSAGE_BOX_HEIGHT),
SCREEN_RECT.width - MINIMAP_RECT.width - (3 * PAD), SCROLLMESSAGE_BOX_HEIGHT
)
MAP_RECT	= Rect(
SCROLLMESSAGE_RECT.left, MINIMAP_RECT.top,
SCROLLMESSAGE_RECT.width, SCREEN_RECT.height - SCROLLMESSAGE_RECT.height - (3 * PAD)
)
TURNTIMER_RECT	= Rect(
MINIMAP_RECT.left, SCROLLMESSAGE_RECT.top, 
MINIMAP_RECT.width, SCROLLMESSAGE_RECT.height
)

# Button layout rects
EXTRABUTTONS_RECT = Rect(MINIMAP_RECT.left, MINIMAP_RECT.bottom + PAD,
		    MINIMAP_RECT.width, 1.9 * BUTTON_SIZE[1]) #hmm
PIECEBUTTONS_RECT = Rect(0, EXTRABUTTONS_RECT.bottom + PAD, 
		    BUTTON_SIZE[0], 6 * BUTTON_SIZE[1]) #max of 6 buttons in there
PIECEBUTTONS_RECT.centerx = MINIMAP_RECT.centerx

####### Screen layout -- positions

# HUD
CPCLOCK_TOPRIGHT    = (MAP_RECT.right - HALF, MAP_RECT.top + HALF)
APCLOCK_TOPRIGHT    = (
CPCLOCK_TOPRIGHT[0] - CPCLOCK_FONT.size(" CP: 6 ")[0] - HALF, CPCLOCK_TOPRIGHT[1]
)
AMMOCLOCK_TOPLEFT   = (MAP_RECT.left + HALF, MAP_RECT.top + HALF)
FPSCLOCK_BOTTOMLEFT = (MAP_RECT.left + HALF, MAP_RECT.bottom - HALF)

# The first die's bottomright:
DICE_BOTTOMRIGHT    = (MAP_RECT.right - HALF, MAP_RECT.bottom - HALF)

# Dialog
DIALOG_CENTER	    = MAP_RECT.center#hmm SCREEN_RECT.center?
DIALOG_MAX_QUERY_WIDTH = MAX_DIALOG_WIDTH = int(4 * MAP_RECT.width / 5) #hmm, too many names

# ToolTip
TOOLTIP_MAX_WIDTH   = TOOLTIP_FONT.size(25 * "M")[0]

# Splash logo
SPLASH_MIDTOP	    = (SCREEN_RECT.centerx, MAP_RECT.top + PAD)


##############################################################################
####### Loading images

PERPIXEL_ALPHA, TOPLEFT_COLORKEY = range(2)

