# Sulk - constants - options

#*****************************************************************************
#
#                                   Sulk
#
#       Copyright (c) 2002 Toby Woodwark <zirtix@softhome.net>
#
# This program is free software.  See the file COPYING for details.
#
#******************************************************************************

"""
Sulk commandline options and sulkrc.
"""

##############################################################################
####### Import

import sys, getopt, os, time
import ConfigParser

from strings import APP_AND_VERSION_STRING, COPYRIGHT_STRING,\
MISSIONS_GLOB_STRING, DEFAULT_SOUND_DIRECTORY, DEFAULT_IMAGE_DIRECTORY,\
DEFAULT_THEME_DIRECTORY, DEFAULT_MISSIONS_DIRECTORY, DEFAULT_FONT_DIRECTORY
#AI_PLAYER, LOCAL_PLAYER, REMOTE_HOST, REMOTE_CLIENT -- no


##############################################################################
####### Defaults

# N.B. mostly, but not all, changeable on commandline / ingame / sulkrc

SOUND_DIRECTORY = DEFAULT_SOUND_DIRECTORY
IMAGE_DIRECTORY = DEFAULT_IMAGE_DIRECTORY
THEME_DIRECTORY	= DEFAULT_THEME_DIRECTORY
FONT_DIRECTORY = DEFAULT_FONT_DIRECTORY
MISSIONS_DIRECTORY = DEFAULT_MISSIONS_DIRECTORY
SOUND_ENABLED = True
SOUND_VOL_PERCENT = 80
SCREEN_WIDTH, SCREEN_HEIGHT = 800, 600
PRINT_DEBUG_INFO = False
SCREEN_FULLSCREEN = False
MARINE_BEGINNER = False
NEXTMISH = None #NOT USED YET
SULKRC_FILE_NAME = 'sulkrc'
#PLAYER1, PLAYER2 = LOCAL_PLAYER, LOCAL_PLAYER


##############################################################################
####### Functions

def _usage():
    """_usage() -> None
    print program info"""
    
    print APP_AND_VERSION_STRING
    print COPYRIGHT_STRING
    print
    print"""\
Command line options are:
  -b, --beginner	  give the marine player extra time
  -c, --config FILE	  use configuration options from FILE\
 (default: '$HOME/.sulkrc')
  -d, --debug		  show debugging output
  -f, --fullscreen	  use full screen graphics
  -g, --geometry WxH	  use graphics with width W and height H pixels\
 (default: 800x600)
  -h, --help		  see this message
  -m, --missionsdir DIR	  use '%s' files from directory DIR\
 (default: '%s')
  -s, --nosound		  suppress sounds
  -t, --themedir DIR	  use images from theme directory DIR\
 (default: '%s')
  -w, --windowed	  use windowed graphics\
 (default)
  -v, --volume VOL	  set sound volume to VOL%%\
 (default: 80%%)
"""%(MISSIONS_GLOB_STRING, DEFAULT_MISSIONS_DIRECTORY, DEFAULT_THEME_DIRECTORY)


def _tryexpand(path):
    """_tryexpand(path) -> path
    try to expand ~ to user

    Only done if we are on the right platform."""
    
    if os.name == "posix": 
	return os.path.expanduser(path)
    return path


def _retrieve_parser():
    """_retrieve_parser() -> ConfigParser object
    get a ConfigParser instance
    
    Suck globals out of the air and set up a parser
    with all appropriate values set"""
    
    configdict = {
	"global":   {"debug":str(PRINT_DEBUG_INFO)
		    },
	"locations":{"missions_dir":MISSIONS_DIRECTORY,
		    "theme_dir":THEME_DIRECTORY,
		    "sounds_dir":SOUND_DIRECTORY
		    },
	"rules":    {"marine_beginner":str(MARINE_BEGINNER)
		    },
	"sound":    {"enabled":str(SOUND_ENABLED),
		    "volume":str(SOUND_VOL_PERCENT)
		    },
	"graphics": {"screen_width":str(SCREEN_WIDTH), 
		    "screen_height":str(SCREEN_HEIGHT),
		    "fullscreen":str(SCREEN_FULLSCREEN)
		    }
	}
    cfp = ConfigParser.ConfigParser()
    for sectkey in configdict.keys():
	cfp.add_section(sectkey)
	for optkey in configdict[sectkey].keys():
	    cfp.set(sectkey, optkey, str(configdict[sectkey][optkey]))
    return cfp


def init():
    """init() -> None

    Process sulkrc and the command line options."""
    
    # Attributes which we may modify:
    global SCREEN_WIDTH, SCREEN_HEIGHT, PRINT_DEBUG_INFO,\
	SCREEN_FULLSCREEN, SULKRC_FILE_NAME, SOUND_ENABLED,\
	SOUND_VOL_PERCENT, MARINE_BEGINNER, NEXTMISH,\
	MISSIONS_DIRECTORY, THEME_DIRECTORY, SOUND_DIRECTORY
	#PLAYER1, PLAYER2
    
    # Find an existing ~/.sulkrc or ./sulkrc
    # add $CWD/sulkrc to the list
    checkfiles = ['sulkrc']
    # look for $HOME envariable
    user_homedir = os.getenv("HOME", 0)
    if user_homedir:
	# add $HOME/.sulkrc to the list
	checkfiles.append(os.path.join(user_homedir,'.sulkrc'))
    
    # See which candidate config files are accessible
    for filename in checkfiles:
	if os.access(filename, os.R_OK):
	    SULKRC_FILE_NAME = filename
    
    # Get the opts
    try:
	opts, args = getopt.getopt(
	sys.argv[1:], 
	"g:hdf1:2:t:wsc:bm:v:", 
	["geometry=", 
	"help", 
	"debug", 
	"fullscreen",
	"player1=", #remove -- pointless, really
	"player2=", #remove
	"themedir=",
	"windowed",
	"nosound",
	"config=",
	"beginner", 
	"missionsdir=",
	"volume="]
	)
    except getopt.GetoptError, message:
	_usage()
	raise SystemExit, "Error: %s"%message
	    
    # Try to get an alternate configfile
    # -- this will supersede and block the 'built-in' ~/.sulkrc and $CWD/sulkrc
    for o, a in opts:
	# Alterna-config
	if o in ("-c","--config"):
	    SULKRC_FILE_NAME = _tryexpand(a)
    	# Help
	elif o in ("-h", "--help"):
	    _usage()
	    raise SystemExit
	
    # Read the configfile, if any
    if SULKRC_FILE_NAME:
	# Make a parser with defaults set
	cfp = _retrieve_parser()
	#print "Loading config from '%s'"%SULKRC_FILE_NAME #?
	try:
	    # Read in the chosen config file
	    cfp.read(SULKRC_FILE_NAME)
	except IOError:
	    print "Warning: could not read config file '%s'"%SULKRC_FILE_NAME
	    # Unset the filename to prevent writing config back to it (????)
	    #SULKRC_FILE_NAME = None
	except Exception, message:
	    print "Warning: config file '%s' could not be processed:"%SULKRC_FILE_NAME, message
	    #SULKRC_FILE_NAME = None
	else:
	    try:
		# Set values
		PRINT_DEBUG_INFO    = cfp.getboolean("global", "debug")
		MISSION_DIRECTORY   = _tryexpand(cfp.get("locations", "missions_dir"))
		THEME_DIRECTORY	    = _tryexpand(cfp.get("locations", "theme_dir"))
		SOUND_DIRECTORY	    = _tryexpand(cfp.get("locations", "sounds_dir"))
		MARINE_BEGINNER	    = cfp.getboolean("rules", "marine_beginner")
		SOUND_ENABLED	    = cfp.getboolean("sound", "enabled")
		SOUND_VOL_PERCENT   = cfp.getint("sound", "volume")
		SCREEN_WIDTH	    = cfp.getint("graphics", "screen_width")
		SCREEN_HEIGHT	    = cfp.getint("graphics", "screen_height")
		SCREEN_FULLSCREEN   = cfp.getboolean("graphics", "fullscreen")
	    except Exception, message:
		print "Error parsing config file '%s':"%SULKRC_FILE_NAME, message
	    
    # Now check other getopts (thus taking precedence over sulkrc)
  #  playerstrings = AI_PLAYER, LOCAL_PLAYER, REMOTE_HOST, REMOTE_CLIENT
 #   playermsg = "Error: couldn't understand player '%s'"
    for o, a in opts:
	# Debug
	if o in ("--debug", "-d"):
	    PRINT_DEBUG_INFO = True
	# Beginner
	elif o in ("--beginner", "-b"):
	    MARINE_BEGINNER = True
	# Player -- EXPERIMENTAL
	#elif o in ("--player1", "-1"):
	#    al = a.lower()
	#    if al in playerstrings:
#		PLAYER1 = al
#	    else:
#		print playermsg%a
#	elif o in ("--player2", "-2"):
#	    al = a.lower()
#	    if al in playerstrings:
#		PLAYER2 = al
#	    else:
#		print playermsg%a
	# Missions
	elif o in ("--missionsdir", "-m"):
	    MISSIONS_DIRECTORY = a
	# Theme
	elif o in ("--themedir", "-t"):
	    THEME_DIRECTORY = a
	# Sound
	elif o in ("--nosound", "-s"):
	    SOUND_ENABLED = 0
	elif o in ("--volume", "-v"):
	    try:
		SOUND_VOL_PERCENT = int(a)
	    except:
		print "Error: couldn't understand volume '%s'"%a
	# Screen
	elif o in ('--fullscreen', '-f'):
	    SCREEN_FULLSCREEN = True
	elif o in ('--windowed', '-w'):
	    SCREEN_FULLSCREEN = False
	elif o in ('--geometry', '-g'):
	    # Find a geometry
	    for splitstring in ("x", "X"):
		try:
		    # Attempt to parse
		    widthstring, heightstring = a.split(splitstring,1)
		    width, height = int(widthstring), int(heightstring)
		except:
		    pass
		else: break
	    else: 
		# Found no parsable and reasonable sizes
		print "Error: couldn't understand geometry '%s'"%a
		continue
	    SCREEN_WIDTH, SCREEN_HEIGHT = width, height


def save_opts(filename):
    """save_opts(filename) -> bool

    Try to save the current settings to 'filename',
    returning true on success."""
   
    if filename:
	if os.access(filename, os.F_OK) and PRINT_DEBUG_INFO: 
	    print "Warning: file '%s' already exists"%filename #DEBUG
	try:
	    # Open chosen config file location
	    f = file(filename, 'w')
	except Exception, message:
	    if PRINT_DEBUG_INFO: print "Could not open config file '%s' for writing,"%filename, message #DEBUG
	    ret = 0
	else:
	    cfp = _retrieve_parser()
	    print >> f, "# Config generated by %s"%APP_AND_VERSION_STRING
	    print >> f, "# on %s"%time.asctime()
	    print >> f, ""
	    try:
		cfp.write(f)
	    except Exception, message:
		if PRINT_DEBUG_INFO: print "Could not write config,", message #DEBUG
		ret = False
	    else: 
		ret = True
	    f.close()
	return ret
    return False


##############################################################################
####### Go

init()
