# Sulk - constants - rules

#*****************************************************************************
#
#                                   Sulk
#
#       Copyright (c) 2002 Toby Woodwark <zirtix@softhome.net>
#
# This program is free software.  See the file COPYING for details.
#
#******************************************************************************

"""
Rules constants.
"""

##############################################################################
####### Import

from layout import SQ_LENGTH


##############################################################################
####### Game LOS etc

# Beginner / clock
TURNTIMER_BASE_TIME = 120 #in seconds
MARINE_BEGINNER_TURNTIMER_BONUS = 30

# Facing
UP, RIGHT, DOWN, LEFT = range(4) #this order is important
FACINGNAMES = {UP:"UP", RIGHT:"RIGHT", DOWN:"DOWN", LEFT:"LEFT"} #to debug

# How a piece moving 1 square in the given facing direction will move:
FACEMAP = {
UP:(0, -SQ_LENGTH), 
RIGHT:(SQ_LENGTH, 0), 
DOWN:(0, SQ_LENGTH), 
LEFT:(-SQ_LENGTH, 0) }
FACE_COORDMAP = {
UP:(0, -1), 
RIGHT:(1, 0), 
DOWN:(0, 1), 
LEFT:(-1, 0) }

# Angles (think 16-way joystick)
F, F_R_F, F_R, F_R_R, R, B_R_R, B_R, B_R_B,\
B, B_L_B, B_L, B_L_L, L, F_L_L, F_L, F_L_F = range(16)
ANGLENAMES = {	
F:"F", F_R_F:"F_R_F", F_R:"F_R", F_R_R:"F_R_R", 
R:"R", B_R_R:"B_R_R", B_R:"B_R", B_R_B:"B_R_B", B:"B", B_L_B:"B_L_B", 
B_L:"B_L", B_L_L:"B_L_L", L:"L", F_L_L:"F_L_L", F_L:"F_L", F_L_F:"F_L_F", 
None:"None!"} #to debug
NOARC	= (B_R_R, B_R, B_R_B, B, B_L_B, B_L, B_L_L) #back 180 deg exclusive
FIREARC = (F, F_R_F, F_R, F_L, F_L_F) #forward 90 deg
LOSARC	= (F, F_R_F, F_R, F_R_R, R, L, F_L_L, F_L, F_L_F) #forward 180 deg inclusive
FACINGANGLES = {UP:F, RIGHT:R, DOWN:B, LEFT:L} #hmm

# How far a piece moving 1 squarelength in the given angle will move 
# forwards:
FORWARDMAP = {	
F:SQ_LENGTH, F_R:SQ_LENGTH, R:0, B_R:-SQ_LENGTH,\
B:-SQ_LENGTH, B_L:-SQ_LENGTH, L:0,  F_L:SQ_LENGTH }

# How far a piece moving 1 squarelength in the given angle will move 
# rightwards:
RIGHTMAP =   {	
F:0, F_R:SQ_LENGTH, R:SQ_LENGTH, B_R:SQ_LENGTH,\
B:0,  B_L:-SQ_LENGTH, L:-SQ_LENGTH, F_L:-SQ_LENGTH }

# 'Low-level' line-intersection logic (David Clark code in sprites.functions)
DONT_INTERSECT, COLINEAR = 0, -1	  

##############################################################################
####### Fighting

# Weapons
GUN, FLAMEGUN, ASSAULT_CANNON = range(3)

# Ways for flames to spread
NOSPREAD, SPREAD = range(2)		   

# Ways to die
NODEATH, SILENT, SKEWERED, SHOT, FLAMBEED = range(5)


##############################################################################
####### UI

# Buttons
DISABLED, PRESSED, ENABLED = range(3)

# HiLite gfx
REDHILITE, GREENHILITE = range(2)   


##############################################################################
####### Board elements and objectives

# Misc flags, features and piece types 
# M - marine deploy, E - stealer entrypoint, O - objective
M, ENTRY, EXIT, O, COMMENT,\
USE_AMBUSH_COUNTERS, MAY_SEAL_ENTRYPOINTS = range(7)

# CAT deployment methods
WITHSQUAD, ONBOARD = range(2)

# Action types
#N.B. ACTING is just for generic rubbish (like unjamming a gun)
ACTING, MOVING, USING, CCING, OVERWATCHING, TURNING, SHOOTING,\
RELOADING, AUTOFIRING, CUTTING = range(10)


##############################################################################
####### Blip sets

# Standard Space Hulk 1st ed blip set:
BLIP_SET_BASIC = {1:8, 2:4, 3:9} # 8 1-blips, etc
