# Sulk - missions

#*****************************************************************************
#
#                                   Sulk
#
#       Copyright (c) 2002 Toby Woodwark <zirtix@softhome.net>
#
# This program is free software.  See the file COPYING for details.
#
#******************************************************************************

"""Provides mission modules to Sulk.

Imports all the missions in MISSIONS_DIRECTORY or directories below that.  

A mission should be called 'MISH<something>.py' and should conform to 
'MISSIONS_INFO'.  You can't import more than 1 mission with the same NUM 
and FAMILY attributes.
"""

# FIXME -- you can't import two missions called MISHfoo.py from different
# directories.  (This needs the imp module to fix, probably.)

####### Import

import fnmatch, os, sys

from constants import options, DEBUG_STREAM
from constants.options import MISSIONS_DIRECTORY, MISSIONS_GLOB_STRING


####### Function

def _process_dir(missions_dict, dirname, names):
    """Import all the missions in a directory which we can.
    
    For use in os.path.walk(..)"""
    
    try:
	sys.path.insert(0, dirname)
    except: 
	print >> DEBUG_STREAM,\
	"Error importing missions: could not open directory '%s'"%\
	os.path.abspath(dirname) #DEBUG
	return
     
    # Make a list of all the MISH*.py files
    correctnames = fnmatch.filter(names, MISSIONS_GLOB_STRING) #Python 2.2
    potential_mish_names = map(lambda x: os.path.splitext(x)[0], correctnames)

    # Import all available missions
    for name in potential_mish_names:
	try:
	    #TODO -- better try-except-else
	    # Do we already have this name?  If so, we can't import it (FIXME)
	    for famdict in missions_dict.itervalues(): #Python 2.2
		for m in famdict.itervalues():
		    if name == m.__name__:
			raise ImportError,\
			"mission filename already in symbol dictionary"
		    
	    # Import and see if this appears to *be* a mission
	    try: 
		mish = __import__(name, globals(), locals(), [])
	    except Exception, message:
		raise ImportError, "mission has broken syntax: %s"%message
	    else:
		if not hasattr(mish, 'BLIP_SET'): 
		    raise ImportError, "mission has no BLIP_SET"
		
		# Add to dict, unless we already had this mission from some
		# other file
		if missions_dict.has_key(mish.FAMILY):
		    if missions_dict[mish.FAMILY].has_key(mish.NUM): 
			raise ImportError,\
			"already had mission #%d for family '%s'"%(mish.NUM, mish.FAMILY)
		    else: missions_dict[mish.FAMILY][mish.NUM] = mish
		else:
		    missions_dict[mish.FAMILY] = {mish.NUM:mish}
	    
	except ImportError, message:
	    print >> DEBUG_STREAM, "Error importing mission:",\
	    message #DEBUG
	    print >> DEBUG_STREAM, " ... (while importing '%s' from directory '%s')"%\
	    (name, os.path.abspath(dirname)) #DEBUG
    
    
####### Go

# missions_available is a nested dictionary of mission.FAMILY:{dict of
# mission.NUM:mission} entries
missions_available = {}

# Walk through the filesystem filling the dictionary
os.path.walk(MISSIONS_DIRECTORY, _process_dir, missions_available)

# Quit if there are no missions.
if not missions_available: 
    raise SystemExit,\
    "Error: No missions could be imported from directory '%s'."%\
    MISSIONS_DIRECTORY

# Tidy
del _process_dir
