# Sulk - objects - board

#*****************************************************************************
#
#                                   Sulk
#
#       Copyright (c) 2002 Toby Woodwark <zirtix@softhome.net>
#
# This program is free software.  See the file COPYING for details.
#
#******************************************************************************

"""
The game board.
"""

##############################################################################
####### Import

import pygame, random

from constants import *
import sprites


##############################################################################
####### Class

class Board:
    """Represents a rectangle covering the whole board layout.
    
    Attributes:
    (instance)
	squares_map	-- dict of coord: Square 
			-- to help find squares using logical coords
	centersquare	-- a Square -- in the middle at game start
	game		-- a SulkGame
	rect		-- a Rect -- covering the board
	group		-- a spritegroup 
			-- containing any thing that is _attached_ to the board
	_comments	-- a spritegroup -- SquareComment instances
	_comments_tally -- int -- total # of comment strings available to make
	"""
    
    def __init__(self, sulkgame):
        """        
        Reads in the layout and creates Squares.
	Uses MAP_RECT and SQ_LENGTH constants.  NB if MAP_RECT is bigger than
	the eventual rect, rect will be MAP_RECT.  This funct also makes all
	the MiniSquare, Door, EntryTriangle and ExitArrow instances, and the
	SectionLines."""
        
	self.game = sulkgame

	squares_map = {}
	
	# Prep sprites.BoardGraphic
	sprites.BoardGraphic.board_container = self.group = pygame.sprite.Group()

	self._comments = pygame.sprite.Group()
	self._comments_tally = 0
	self.rect = pygame.Rect(MAP_RECT)
	
        # Decide where rect should start - 
	# set a class attribute for Square
        sprites.Square.offset = offsetx, offsety = MAP_RECT.left, MAP_RECT.top
	
        # Prepare to find the center square
        centersquare = None
        
        # First we loop through the array getting dimensions for rect
        for sectionl in sulkgame.mission.BOARD:	 
            for squaretuple in sectionl:
        	coord = squaretuple[0]
        	# TODO Do we need to ensure rect will have room for the entrytriangles?
		# (Nothing done on that so far.)
        	s_rect = pygame.Rect(
                (SQ_LENGTH * coord[0]) + offsetx, (SQ_LENGTH * coord[1]) + offsety,
		SQ_LENGTH, SQ_LENGTH)
        	self.rect.union_ip(s_rect)
    	    
        # We now have dimensions, 
	# so we can make minisquares at the same time as squares.
	# To do this we need to set a class attribute for MiniSquare
        sprites.MiniSquare.offset = \
	(
	MINIMAP_RECT.centerx - self.rect.width / (2 * MINI_SCALE),
        MINIMAP_RECT.centery - self.rect.height / (2 * MINI_SCALE)
	)
        
	# perf shortcuts
	sprites_square = sprites.Square
	rotate = pygame.transform.rotate
	frange = sprites.functions.find_range
	ang = sprites.functions.ang
	group = pygame.sprite.Group
	
        # Iterate through the array again 
        for sectionl in sulkgame.mission.BOARD:
	    
	    # Construct a section
            section = group()
	    
	    for squaretuple in sectionl:   
		# We construct a square
		
                # Initial processing
        	coord = squaretuple[0]
		if squaretuple[1:]: squaredict = squaretuple[1]
		else: squaredict = {}
    	    
		# Make square & add to dict
		s = sprites_square(coord, section)
		squares_map[coord] = s
    	    
		# Check for centersquare
		#if centersquare is None and coord == beginplace: centersquare = s
    	    
		# Process dict
		for key, value in squaredict.items():
		    # Objective?
		    if key == O:
			s.objectivenum = value
		    # Marine deploy?
		    elif key == M:
			s.isdeploy = True
			s.deploy_attribute = value
		    # Stealer entry?
		    elif key == ENTRY:
			s.isentry = True
			s.efacing = value
			# Make the triangle thing - whose constructor will make its limbosquares for us
			sprites.EntryTriangle(s)
		    # Marine exit point?
		    elif key == EXIT:
			s.isexit = True
			s.efacing = value
			# Make an exit arrow
			sprites.ExitArrow(s)
		    # Door?
		    elif key == DOOR:
			sprites.Door(s, value)
                    # Bulkhead ?
                    elif key == BULKHEAD:
                        sprites.Bulkhead(s, value)
                    # Rubble ?
                    elif key == RUBBLE:
                        sprites.Rubble(s, value)
                    # Crate ?
                    elif key == CRATE:
                        sprites.Crate(s, value)
                    # Cryo tank?    
                    elif key == CRYOGENIC_TANK:
                        sprites.Cryogenic_Tank(s, value)
		    # Ducting?
		    elif key == DUCTING:
			sprites.Ducting(s, value)
		    # Comment?
		    elif key == COMMENT:
			s.comment = value
			self._comments_tally += 1
			
	    # Now choose an image for each square in each section
	    for s in section.sprites():
		# Random angle to rotate image by
		angle = 90 * random.randrange(0,4)
		if len(section) > 5:
		    # We're dealing with an actual room
		    # We want 2 orthogonally adjacent squares in the same section
		    ort_adjs = 0
		    for sq in section.sprites():
			# Is the square orthogonally adjacent?
			if ang(s, sq) in (F,B,L,R)\
			and frange(s, sq) <= 1:
			    ort_adjs += 1
		    #print >> DEBUG_STREAM, "objects.board.Board --> init, ort_adjs is:",ort_adjs #DEBUG
		    if ort_adjs >= 2:
			# Alternate image
			s.image = rotate(sprites_square.room_image, angle)
			# unrandomized -> s.image = sprites.Square.room_image
			continue
		# Ordinary image
		s.image = rotate(sprites_square.corridor_image, angle)
		# unrandomized -> s.image = sprites.Square.corridor_image
		
	#Add sectionlines
	for s in squares_map.values():
	    for sq in s.get_adjacents():
		a = ang(s, sq)
		if s.section is not sq.section:
		    if a == B:
			sprites.SectionLine(s, UP)
			break
		    if a == L:
			sprites.SectionLine(s, RIGHT)
			break

	# Reposition map
	#if centersquare: self.center_map_on(centersquare)
	if self._comments_tally: 
	    self.build_comments()
	    self.bring_into_map(self._comments)
	self.squares_map = squares_map


    def get_square_from_coord(self, coord): #NOT USED YET
	"""-> Square instance"""

	return self.squares_map[coord]

    
    def get_comments_num(self):
	"""get_commentnum() -> int
	return number of comments currently showing"""

	return len(self._comments)


    def get_comments_avail(self):
	"""get_comments_avail() -> int
	return number of comments that can be displayed"""

	return self._comments_tally
	

    def build_comments(self):
	"""build_comments() -> None
	make comments
	
	(sprites explaining which areas of the board are
	important)."""
	
	if not self.get_comments_num() and self.get_comments_avail():
	    for sqr in self.game.squares.sprites():
		if hasattr(sqr, 'comment'): 
		    self._comments.add(sprites.SquareComment(sqr))

	
    def kill_comments(self):
	"""kill_comments() -> None
	kill all comment sprites"""
	
	for sprite in self._comments.sprites(): sprite.kill()


    def refresh_comments(self):
	"""refresh_comments() -> None
	refresh() all comment sprites"""
	
	for sprite in self._comments.sprites(): sprite.refresh()


    def shift_map(self, offset, warpmouse=0):
	"""shift_map(offset, warpmouse=0) -> None
	move the squares, and hence features and pieces, around
	
	The map 'window' appears to have been moved by -offset."""
	
	moved = []
	if self.rect.move(offset).colliderect(MAP_RECT):
	    for s in self.group.sprites():
		s.rect.move_ip(offset)
	    self.rect.move_ip(offset)
	    if warpmouse: pygame.mouse.set_pos(MAP_RECT.center)


    def center_map_on(self, object):
	"""center_map_on(object) -> None
	take any object that has a rect attribute, and center the map on it
	
	The object.rect must lie well within self.rect."""
	
	self.shift_map((MAP_RECT.centerx - object.rect.centerx, MAP_RECT.centery - object.rect.centery))


    def bring_into_map(self, thing):
	"""bring_into_map(thing) -> None
	move the map to cover something
	
        'thing' must be a spritegroup or a sprite.
	
	Takes a group or sprite (square, feature or piece) and moves the map to cover it.
	Tries to move the boardrect as little as possible to get a member of the spritegroup 
	into the center of the map."""
	
	if not isinstance(thing, (pygame.sprite.Sprite, pygame.sprite.Group)):
	    print >> DEBUG_STREAM, "Board --> bring_into_map, \
	    error: not a sprite or group:", thing #DEBUG
	    return
	
	class DummyClass:
	    rect = MAP_RECT
    
        def get_the_square_of(sprite):
	    """get_the_square_of(sprite) -> Square instance
	    
	    Tries to get a square for a game piece."""

	    if isinstance(s, sprites.Square): return sprite
	    elif hasattr(s, 'square'): return sprite.square
	    else: 
		print >> DEBUG_STREAM, "Board --> bring_into_map --> \
		get_the_square_of, no square for sprite:", sprite #DEBUG

	# Go
	if isinstance(thing, pygame.sprite.Sprite):
	    spritegroupsprites = [thing]
	else:
	    # Assume group
            try:
                spritegroupsprites = thing.sprites()
            except:
                raise SystemExit,\
                "objects.board.Board.bring_into_map given " + thing +\
                " instead of sprite or spritegroup"
	
	# If the map already has the group well inside it, return
	if MAP_RECT.inflate(-4 * SQ_LENGTH, -4 * SQ_LENGTH).\
        collidelistall([s.rect for s in spritegroupsprites]): return

	# Otherwise, find the sprite closest to the center
	mysq = None
	mapdummyclassinstance = DummyClass()
	list = map(get_the_square_of, spritegroupsprites)
	dist = 100 #infinity really
	frange = sprites.functions.find_range #shortcut 
	
	for sq in list:
	    range = frange(sq, mapdummyclassinstance) #hack
	    if range < dist: 
		dist = range
		mysq = sq
	if mysq: self.center_map_on(mysq)
	else: 
	    print >> DEBUG_STREAM, "Board --> bring_into_map --> error: no mysq" #DEBUG
