# Sulk - objects - teams

#*****************************************************************************
#
#                                   Sulk
#
#       Copyright (c) 2002 Toby Woodwark <zirtix@softhome.net>
#
# This program is free software.  See the file COPYING for details.
#
#*****************************************************************************

"""
Team classes.
"""

###### Import

from __future__ import generators
import pygame
import util #for Null!
import sprites #for Blip
from constants import\
DEBUG_STREAM, STEALERTEAM, STEALER_COLOR,\
MARINE_COLOR, MARINETEAM,\
NEITHER_COLOR, UP, DOWN, LEFT, RIGHT

###### Classes

class SulkTeam(pygame.sprite.Group):
    """A group for Piece instances to live in.
    
    Parent class.
        
    Attributes:
    (class)
	type	    -- string -- STEALERTEAM or MARINETEAM
	color	    -- RGB -- characteristic color
    (instance)
	player	    -- module -- supplying UI handlers
	enemy_team  -- a SulkTeam
	secrets_showing -- bool
	status	    -- int -- ACTIVE or INACTIVE
	game	    -- a SulkGame
	casualties  -- int -- deaths so far
	_c	    -- iterator -- returned by _cycle()
	"""
    
    def __init__(self, game):
	
        pygame.sprite.Group.__init__(self)
	self.hide_secrets()
	self.casualties = 0
	self.game = game
	self._c = self._cycle() #will of course be 'empty'


    def refresh(self, *args):
	"""refresh(*args) -> None

	Call refresh(...) on the pieces."""
	
	print >> DEBUG_STREAM, "%s --> refresh, %d pieces"%\
	(self.__class__.__name__, len(self)) #DEBUG
	for piece in self.sprites():
	    piece.refresh(*args)


    def show_secrets(self):
	"""show_secrets() -> None
	
	Lets a player see info only he should be able to know."""
    
	#print >> DEBUG_STREAM, "SulkTeam --> show_secrets --> Secrets showing: ", self.type #DEBUG
	self.secrets_showing = 1


    def hide_secrets(self):
	"""hide_secrets() -> None
	
	Duh."""
    
	#print >> DEBUG_STREAM, "SulkTeam --> hide_secrets --> Secrets hidden: ", self.type #DEBUG
	self.secrets_showing = 0


    def find_a_nice_facing(self):
	"""find_a_nice_facing() -> int
    
	Returns the modal facing of a team's pieces.	
	Good for a laff, and helps Deploy phases not be a pain."""
    
	def is_relevant_piece(piece):
	    # blips and lurkers are not real
	    return (not (isinstance(piece, sprites.Blip) or piece.get_lurking()))
	
	fdict = {UP:0, RIGHT:0, DOWN:0, LEFT:0}
	for s in filter(is_relevant_piece, self.sprites()):
	    fdict[s.facing] += 1
	flist = zip(fdict.values(), fdict.keys()) #hmm
	flist.sort() #on the values() parts first
	return flist[-1][1]


    def _cycle(self):
	"""_cycle() -> generator object
	
	GENERATOR
	next() on the generator gets the next piece out.
	
	Never throws a StopIteration, but will return a Null if
	there are no pieces."""

	#Python 2.2

	while 1:
	    if not self: 
		print >> DEBUG_STREAM, "SulkTeam --> _cycle, no pieces"
		yield util.Null
	    for piece in self.sprites(): 
		yield piece

    
    def get_next(self, *statuses):
	"""get_next(*statuses) -> piece
	
	Gets the next piece which has status one of 'statuses'.  Constant
	cycle, i.e. is *usually* guaranteed to get a piece we haven't seen
	recently.
	
	Returns None if no pieces are available."""

	if not statuses: 
	    # Allow all
	    statuses = (INACTIVE, ACTIVE, USED)
	
	i = 0
	while i < len(self): #hmm
	    i += 1
	    x = self._c.next()
	    # Check piece status
	    if x.status in statuses: 
		return x


class NeutralTeam(SulkTeam): #?
    """Anonymous team for stuff like C.A.T.s.
    """
    
    color = NEITHER_COLOR
    type = None
    enemy_team = None #hope to raise errors etc.

    
class MarineTeam(SulkTeam):
    """The Marine team.
    """
    
    color = MARINE_COLOR
    type = MARINETEAM


class StealerTeam(SulkTeam):
    """The Stealer team.
    """
    
    color = STEALER_COLOR
    type = STEALERTEAM


    def show_secrets(self):
	SulkTeam.show_secrets(self)
	for piece in self.sprites():
	    getattr(piece, 'reveal', util.Null)()


    def hide_secrets(self):
	SulkTeam.hide_secrets(self)
	for piece in self.sprites():
	    getattr(piece, 'unreveal', util.Null)()
