# Sulk - players - ai0 - UIinput

#*****************************************************************************
#
#                                   Sulk
#
#       Copyright (c) 2002 Toby Woodwark <zirtix@softhome.net>
#
# This program is free software.  See the file COPYING for details.
#
#*****************************************************************************

"""
Input classes -- AI0 player.

Currently this agent acts fairly stupidly, without
focused behaviour or mission objectives.
"""

##############################################################################
####### Import / set up

## Modules
import pygame, random

## Sulk modules
from constants import *
import util
import sprites
import phases
from players.local.UIinput import * #and then override...


##############################################################################
####### InputHandlers

# These form a stack in a phase.handlers and handle UI each tick through handle(). 

class InputHandler:
    """Turns pygame events into UI interactions.
    
    Parent class.
    
    The handle() method should be called once per tick.

    Attributes:
    (instance)
	phase	               -- Phase instance
	_currently_hesitating  -- int -- msec of hesitation remaining
	"""
	
    def __init__(self, phase):

	for x in phase.handlers:
	    print >> DEBUG_STREAM, "=AI0", #DEBUG
	print >> DEBUG_STREAM, "=AI0 %s --> __init__"%self.__class__.__name__ #DEBUG
	
	phase.handlers.append(self)
	self.phase = phase
	self._currently_hesitating = 0
    
    
    def handle(self):

	pygame.event.pump()
	if self.phase.game.paused:
	    e = pygame.event.wait()
	else:
	    e = pygame.event.poll()
	if e.type == QUIT or (e.type == KEYDOWN and e.key == K__QUIT):
            self.phase.game.end()
	    return True
	elif e.type == KEYDOWN:
	    if e.key == K__INTERRUPT and self.phase.interruptable: 
		self.phase.game.marines.player.phases.Marines_Interrupt(self.phase) #FIXME?
		return True
	    for b in self.phase.game.globalbuttons:
		if e.key == b.key: 
		    b.hit()
		    return True
	elif e.type == MOUSEBUTTONDOWN:
	    for dec in self.phase.game.buttons.sprites():
		if dec.rect.collidepoint(e.pos):
		    dec.hit()
		    return True
	if self._currently_hesitating > 0:
            # Wait at least 100 msec
            self._currently_hesitating -= min(sprites.Graphic.elapsed, 100)
	    return True
	
	
    def _hesitate(self, num=300):
	"""-> None
        cause a wait for 'num' msec"""
        
	#print >> DEBUG_STREAM, "players.local.ai.UIinput.%s --> _hesitate %d"%(self.__class__.__name__, num) #DEBUG
	self._currently_hesitating = num #msec

    	
    def destroy(self):
	
	if hasattr(self, 'phase'):
	    while self in self.phase.handlers:
		self.phase.handlers.remove(self)
	
	
class Deploy_A_New_Piece(InputHandler):
    
    def __init__(self, phase, piececlass, deployteam=None, squadname=None):
	
	# Massive code copying, yay
	InputHandler.__init__(self, phase)

	self.tris = []
	self._done = False
	
	self.piececlass = piececlass
	self.squadname = squadname
	if deployteam is None: self.deployteam = self.piececlass.team
	else: self.deployteam = deployteam

	self._d_clock = sprites.DeployClock(self)
	_deployable_squares = self._deployable_squares = pygame.sprite.Group()
	
	# See which squares are permissible
	pre_deploy_rule = self.phase.game.mission.pre_deploy_rule #shortcut
	add = _deployable_squares.add #shortcut
	
	if piececlass.team.type == MARINETEAM:
	    #raise SystemExit, "AI0 can't handle Marines yet!"
	    for sqr in self.phase.game.squares.sprites():
		if sqr.piece or not (sqr.isdeploy and sqr.get_passable()) or\
		not pre_deploy_rule(piececlass, sqr, squadname): 
		    continue
		add(sqr)
		
	elif piececlass.team.type == STEALERTEAM:
	    # We are deploying a blip
	    self.value = self.phase.reinflist[0] #first element
	    for tri in self.phase.game.entrytriangles.sprites():
		# NB stealers apply the pre_deploy_rule to _entrypoints_ :
		if not pre_deploy_rule(piececlass, tri.entrypoint): 
		    continue
		havepiecesalready = []
		# May not deploy to entry areas with 3 blips already
		for sqr in tri.limbosquares.sprites():
		    if sqr.piece.sprite.__class__ is piececlass: #hmm, relying on GroupSingle internals
			havepiecesalready.append(sqr)
		if len(havepiecesalready) > 2: 
		    #RULES--> may not deploy to entrytriangles with > 2 blips already
		    continue
		self.tris.append(tri)
		for sqr in tri.limbosquares.sprites():
		    if not sqr.piece:
			add(sqr)
			    
	else: print >> DEBUG_STREAM, "Deploy_A_New_Piece --> init, error: unknown self.piececlass.team.type"#DEBUG
        
	self._hesitate(500)
        
	if not _deployable_squares:
	    print >> DEBUG_STREAM, "Deploy_A_New_Piece --> init, nowhere to deploy!"#DEBUG
            self.phase.reinflist = [] # SE from bug 729687
	    self.destroy()
	else:
	    self.phase.game.board.bring_into_map(_deployable_squares)
	    self.select = None #Will be a Square instance
            
            if piececlass.team.type == MARINETEAM:
                #Hilites?
	        pass
		
	    elif piececlass.team.type == STEALERTEAM:
                #Now order triangles by proximity to marines
                max_search_dist = 40 #arbitrary limit on search depth
                scores ={}
                
                def score(t):
                    # A low score is good
                    # TODO better heuristics?
                    print >> DEBUG_STREAM, "Deploy_A_New_Piece --> scoring entrypoint:", t.entrypoint #DEBUG
                    x = t.entrypoint.find_piece_near(self.piececlass.team.enemy_team, max_search_dist)
                    if x:
                        piece, val = x
                        #HEURISTIC usually very bad if we are forced to lurk, increase val by a large amount
                        if val <= 6:
                            val += 10 #was 10 - val
                        #HEURISTIC flames are _bad_
                        if t.entrypoint.get_flaming():
                            val += 5 #was 4
                        #HEURISTIC things right in the entrance are pretty bad
                        if not t.entrypoint.get_passable():
                            val += 2 #was 1 
                        #HEURISTIC things already deployed and sitting there are _very_bad_
                        for sq in t.limbosquares.sprites():
                            if sq.piece:
                                val = int(1.2 * val) #factor?
                        print  >> DEBUG_STREAM, "                                        ... ", val #DEBUG
                        return val
                    return max_search_dist #'worst score'
                
                for tri in self.tris:
                    scores[tri] = score(tri)

                def cmpfunc(tri1, tri2):
                    return scores[tri1].__cmp__(scores[tri2]) #IntType.__cmp__

                self.tris.sort(cmpfunc)

		
    def handle(self):
	
	if InputHandler.handle(self): return
	if self._done: 
	    self.destroy()
	    return
	if not self.select:
	    # Choose deploy area
            if self.piececlass.team.type == MARINETEAM:
                self.select = random.choice(self._deployable_squares.sprites())
		
	    elif self.piececlass.team.type == STEALERTEAM:
                for tri in self.tris:
                    for s in tri.limbosquares.sprites():
                        if s.piece and isinstance(s.piece.sprite, sprites.Blip): 
                            break
                    else:
                        for s in tri.limbosquares.sprites():
                            if not s.piece:
                                self.select = s
                                break
                        else: continue
                        break
	    if not self.select: raise "b0rk"
	    self.phase.game.board.bring_into_map(self.select)
	    self._hesitate()
	    return
	if self.piececlass is sprites.Blip:  
	    # Make a blip
	    newblip = sprites.Blip(self.select, self.value)
	    self.phase.game.mission.post_deploy_script(newblip)
	    # Show on map
	    self.phase.game.board.bring_into_map(newblip)
	    del self.phase.reinflist[0]
	    self._hesitate()
	    self._done = True
	else:
	    # Get a facing
	    facing = random.choice((UP, DOWN, LEFT, RIGHT))
	    
	    # Make a piece
	    if issubclass(self.piececlass, sprites.Terminator_Marine):
		newpiece = self.piececlass(self.select, facing, self.squadname)
	    else: 
		newpiece = self.piececlass(self.select, facing)	 
	    self.phase.game.mission.post_deploy_script(newpiece)
            
	    self.destroy()
            
	    # New inputhandler
	    self.deployteam.player.UIinput.Face_A_Piece(self.phase, newpiece, self.deployteam)


    def destroy(self):

	InputHandler.destroy(self)
	self._d_clock.kill()
	del self._d_clock  #?
	if hasattr(self, 'value'): del self.value
	self._deployable_squares.empty()


def Select_Next_Inactive_Piece(phase, team, oldpiece=None):
    """Select_Next_Inactive_Piece(phase, team, oldpiece=None) -> None
    """
    
    print >> DEBUG_STREAM, "players.ai0.UIinput.Select_Next_Inactive_Piece" #DEBUG
    
    if team.type != STEALERTEAM:
        raise SystemExit, "players.ai0.UIinput.Select_Next_Inactive_Piece AI0: can only handle STEALERTEAM!"
    
    # Get the piece with nearest enemy
    max_search_dist = 30 #arbitrary limit, further than this and pieces are selected randomly
    
    def piecenear(p):
	return p.square.find_piece_near(team.enemy_team, max_search_dist)
    
    bestdist = int(max_search_dist)
    getpiece = None
    for piece in team.sprites():
	if piece.status == INACTIVE:
	    d = piecenear(piece)
            if d:
                dist = d[1]
            else:
                dist = max_search_dist
            if dist <= bestdist:
		bestdist = dist
		getpiece = piece
		
    if getpiece: 
	# Configure pieces' status properly
	getpiece.set_active(oldpiece)
	
	# Start using that new piece
	Use_A_Piece(phase, getpiece)
    else:
	# No pieces left
	phase._end() #?


Select_A_Piece = Select_Next_Inactive_Piece


class Use_A_Piece(InputHandler):
    """Stealers only -- no marines or blips
    
    Attributes:
    (instance)
	piece   -- a Piece
	"""
	
    def __init__(self, phase, piece):
	
	if isinstance(piece, sprites.Blip):
	    # Blips are boring
	    piece.convert()
	    return
	InputHandler.__init__(self, phase)
	self.piece = piece
	self.phase.select(piece) #hmm
	self.phase.game.board.bring_into_map(piece)
	self._get_target() #sets route
	self._idling = False
        self._hesitate(150)


    def _get_target(self):
	
        if self.piece.aiflags.get('target', util.Null).alive():
            #self.piece.aiflags['route'] = None #NOOOOO
            return
        if not self.piece.team.enemy_team:
	    print >> DEBUG_STREAM,\
            "players.ai0.UIinput.Use_A_Piece --> _get_target, no enemies!" #DEBUG
	    self.piece.status = USED
	    self.destroy()
	    return
        target = self.piece.square.find_piece_near\
        (self.piece.team.enemy_team, 1000)[0] #'infinite' search
	print >> DEBUG_STREAM,\
        "players.ai0.UIinput.Use_A_Piece --> _get_target, result:",target #DEBUG
        if not target:
            raise SystemExit, "players.ai0.UIinput.Use_A_Piece --> _get_target, could not acquire target"
        self.piece.aiflags['target'] = target
        self.piece.aiflags['route'] = None # Yet to be worked out

    
    def _get_route(self):
	"""_get_route() -> None
	acquire a route to the nearest enemy"""

	#TODO Should be cleverer!  Especially about LOS
        
        target = self.piece.aiflags['target']

        assert target.alive()
        
        if self.piece.aiflags.get('route'):
            return
        
        # Get some shortest routes
        routes = []
        
        # (Are we ignoring flames?)
        if not self.piece.square.get_flaming():
            routes = sprites.functions.find_shortest_routes\
            (self.piece.square, target.square, lambda sq: not sq.get_flaming())
        if not routes:
            routes = sprites.functions.find_shortest_routes\
            (self.piece.square, target.square) 
        if not routes:
            raise SystemExit,\
"players.ai0.UIinput.Use_A_Piece._get_route(self), cannot find any routes (the board is not connected)"
        
        newroutes = []
        
	# Try to find make each route end in orthogonal step
        # This way the Stealer can attack at the last step
	for route in routes:
	    if sprites.functions.ang(route[-2], route[-1]) in (F,B,L,R):
                pass
            else:
                # No orthogonally ending routes available,
                # try to insert a kink in a random route to enable this
                for bridgesquare in route[-2].get_adjacents():
                    if bridgesquare in route[-1].get_adjacents():
                        print >> DEBUG_STREAM,\
"players.ai0.UIinput.Use_A_Piece --> _get_route, ROUTE ATTACK TWEAK at", bridgesquare #DEBUG
                        route[-1:-1] = [bridgesquare] 
                        # (TODO this is not guaranteed to be unique, 
                        # could this be important?  Possibly check 
                        # for los to this bridgesquare, or facing 
                        # of target)
                        break
                else:
                    print >> DEBUG_STREAM,\
"players.ai0.UIinput.Use_A_Piece --> _get_route,\
could not create route with orthogonal final step" #DEBUG

            # Now see if we can straighten out any stupid kinks in 
            # the opening part (DOES THIS WORK?)
            fln = self.piece.square.get_flaming()
            start, end = route[:-2], route[-2:]
            for i in range(len(start[:-2])):
                s1, s2, s3 = start[i], start[i+1], start[i+2]
                if not s2.get_passable(fln):
                    for s in s1.get_adjacents():
                        if s.get_passable(fln) and s in s3.get_adjacents():
                            print >> DEBUG_STREAM,\
"players.ai0.UIinput.Use_A_Piece --> _get_route, ROUTE TWEAK at", s #DEBUG
                            start[i+1] = s
                            break
                    
            # Remove self.piece.square from the beginning of route, 
            # since we're already there
            route = route[1:]

            if route not in newroutes: newroutes.append(route)
        
        bestscore = None
        for route in newroutes:
            # Score each route, i.e. see how many APs to traverse it
            # TODO DEATHWING rubble etc
            numpoints = 0
            piecetoggle = False
            for sq in route:
                numpoints += 1
                # HEURISTIC Compensate for pieces in the way 
                #(except for the target)
                if sq is not target.square\
                and not piecetoggle and sq.piece:
                    piecetoggle = True
                    numpoints += 3
                # Compensate for blocking features in the way
                for fe in sq.features.sprites():
                    if fe.blocksmove:
                        numpoints += 1
                        break
            print >> DEBUG_STREAM,\
"players.ai0.UIinput.Use_A_Piece --> _get_route,\
 scored a route:", numpoints #DEBUG
            if not bestscore or numpoints < bestscore:
                bestscore = numpoints
                bestroute = route
        
        #self.route = bestroute
        self.piece.aiflags['route'] = bestroute
	self.piece.aiflags['hide'] = False # Temporarily reset hiding status
        
	if not self.piece.aiflags.get('blocking')\
        and bestscore > self.piece.get_points_avail()\
        and not self.piece.aiflags.get('hid'):
	    # We can't possibly attack this turn, and we aren't in any 
            # other stealer's way, so let's see whether to hide or not
	    if self.piece.get_lurking(): 
                square = self.piece.square.entrytriangle.entrypoint
	    else: 
                square = self.piece.square
            # If a Marine has LOS to our deploysquare or current square, don't hide
   	    for en in self.piece.team.enemy_team.sprites():
		if en.can_see(square):
		    break
	    else:
                # Hide
		self.piece.aiflags['hide'] = True
                print >> DEBUG_STREAM,\
"players.ai0.UIinput.Use_A_Piece --> _get_route, **hiding**" #DEBUG
        
        # Make fellow lurkers have the same route and hiding status
        # (FIXME KLUDGE)
        if self.piece.get_lurking():
            for sq in self.piece.square.entrytriangle.limbosquares.sprites():
                if sq is not self.piece.square and sq.piece\
                and isinstance(sq.piece.sprite, self.piece.__class__):
                    print >> DEBUG_STREAM,\
"players.ai0.UIinput.Use_A_Piece --> _get_route, route TRANSFERRED to fellow lurker:",\
sq.piece.sprite #DEBUG
                    for s in 'hide', 'route', 'target':
                        sq.piece.sprite.aiflags[s] = self.piece.aiflags[s]
        
	#print >> DEBUG_STREAM,\
#"players.ai0.UIinput.Use_A_Piece --> _get_route, result:",bestroute #DEBUG
        
	
    def _move(self):
	"""_move() -> bool
	move piece the next step on the current route
	
	Return True if we managed to move"""
	
        square = self.piece.aiflags['route'][0]
	if self.piece.aiflags.get('hide'):
	    for enemy in self.piece.team.enemy_team.sprites():
		if enemy.can_see(square):
		    return False
	if self.piece.goMove(square):
            if self.piece.alive():
                self.piece.aiflags['route'] = self.piece.aiflags['route'][1:]
                self.piece.aiflags['blocking'] = False
                self.phase.game.board.bring_into_map(self.piece)
            return True
	return False


    def _turn(self):
	"""_turn() -> bool
	turn the piece the right way
	
	Return True if we managed to turn"""

        angle = sprites.functions.ang(self.piece, self.piece.aiflags['route'][0], self.piece.facing)
	if angle in (R, B_R_R, B_R):
	    return self.piece.goTurn(RIGHT)
	if angle in (B_R_B, B, B_L_B):
	    return self.piece.goTurn(DOWN)
	if angle in (B_L, B_L_L, L, F_L_L):
	    return self.piece.goTurn(LEFT)
        return False


    def destroy(self):
	
        if self.piece.alive():
            # Store a flag telling us what behavior we took this turn
            if self.piece.aiflags.get('hide'): self.piece.aiflags['hid'] = True
            else: self.piece.aiflags['hid'] = False
            # TODO when to reset the following?
            self.piece.aiflags['hide'] = False
            self.piece.aiflags['target'] = util.Null
            self.piece.aiflags['route'] = util.Null
            self.phase.unselect()
            self.piece.status = USED #?
        self.phase.unselect()
	del self.piece
	InputHandler.destroy(self)


    def handle(self):
	
	if not self.piece.alive():
	    #print >> DEBUG_STREAM, "players.ai0.UIinput.Use_A_Piece --> handle, self.piece is dead" #DEBUG
            self._hesitate(500)
	    self.destroy()
	    return
	
	if InputHandler.handle(self):
	    return
	
	if isinstance(self.piece, sprites.Terminator_Marine)\
	and isinstance(self.piece.square, sprites.LimboSquare):
	    print >> DEBUG_STREAM, "players.ai0.UIinput.Use_A_Piece --> handle, \
	    self.piece is on wrong kind of square" #DEBUG
	    self.destroy()
	    return
	
	if self.piece.team.status == ACTIVE and self.piece.status != ACTIVE\
	and (self.piece.team.type != MARINETEAM or self.phase.game.cpbutton.status != PRESSED):
	    # e.g. when a stealer is USED for some reason -- we must die
	    print >> DEBUG_STREAM, "players.ai0.UIinput.Use_A_Piece --> handle, \
	    self.piece is not ACTIVE" #DEBUG
	    self.destroy()
	    return
	
	if not self.piece.apremaining and \
	(self.piece.team.type == STEALERTEAM and isinstance(self.piece, sprites.Blip)\
	or (self.piece.team.type == MARINETEAM and self.piece.team.status == ACTIVE\
	and not self.piece.try_mns and self.phase.game.cpbutton.status != PRESSED)): #hmm
	    sprites.ScrollMessage("AI Finished with a piece.")
	    self.piece.status = USED
	    self.destroy()
	    return
	
	if self.piece != self.phase.cpiece():
	    self.phase.select(self.piece)
	
	# 'AI' starts here
        if not self.piece.aiflags.get('target', util.Null).alive():
	    self._get_target()
	    return
	#if not self.route:
        if not self.piece.aiflags.get('route'):
	    self._get_route()
            self._hesitate(100)
            return
        
        assert self.piece.aiflags['target'].square is self.piece.aiflags['route'][-1]

	if self._turn():
	    self._hesitate(300)
	    return
	if self._move():
	    self._hesitate(450)
	    return
	
	# Select ahead square to test against tricky cases
        select = self.piece.aiflags['route'][0]
		
	if select.piece:
            if self.piece.team.enemy_team.has(select.piece):
                # Attacking                
                if self.piece.goCC(select.piece.sprite):
                    self.piece.aiflags['blocking'] = False #?
		    self._hesitate(400)
                else:
                    self._hesitate(400)
                    self.destroy()
	    else:
                # Friendly piece
                print >> DEBUG_STREAM, "Use_A_Piece --> handle, friendly piece in the way" #DEBUG
                # Our route has been blocked at point 0, so self.route[1:] is inaccessible.
                # Try to change the route.
                # Get some 'new small routes' between here and the square after the blockage
                # These routes avoid pieces.
                
                # Routes avoid fire?
                if self.piece.square.get_flaming():
                    f = lambda sq: False
                else:
                    f = lambda sq: sq.get_flaming()
                    
                smallroutes = sprites.functions.find_shortest_routes(\
                self.piece.square, self.piece.aiflags['route'][1], lambda sq:not (f(sq) or sq.piece), maxlen=4)
                if smallroutes:
                    print >> DEBUG_STREAM,"SMALLROUTES:", smallroutes #DEBUG
                    print >> DEBUG_STREAM, "Use_A_Piece --> handle, friendly piece in the way, changing route" #DEBUG
                    # Use the first one we can get (hmm?)
                    print >> DEBUG_STREAM, "OLDROUTE:", self.piece.aiflags['route'] #self.route #DEBUG
                    # Make a new route out of the smallroute and all but the first 2
                    # squares of the old route
                    self.piece.aiflags['route'] = smallroutes[0][1:] + self.piece.aiflags['route'][2:]
                    print >> DEBUG_STREAM, "NEWROUTE:",self.piece.aiflags['route'] #self.route #DEBUG
                else:
                    print >> DEBUG_STREAM, "Use_A_Piece --> handle, friendly piece in the way, setting blocking" #DEBUG
                    select.piece.sprite.aiflags['blocking'] = True
                    self.destroy()
	    return
	    
	if select.features:
	    # Attacking, or using a door...
	    selected_feature = select.features.sprites()[0] 
	    #FIXME there could be more than 1 feature... (how to fix?)
	    if selected_feature.blocksmove and isinstance(selected_feature, sprites.Door): 
                # It's a closed door, should we use it?
                if self.piece.aiflags.get('hide', False):
                    # Test against LOS after door opening
                    selected_feature.blockslos = False
                    for enemy in self.piece.team.enemy_team.sprites():
                        if enemy.can_see(self.piece.square) or enemy.can_see(select):
                            # Don't use this door, it'll backfire
                            selected_feature.blockslos = True
                            self.destroy()
                            return
                # Try to use door
                selected_feature.blockslos = True
                if self.piece.goDoor(selected_feature):
		    self._hesitate(500)
		    return
            self.destroy()
	
	if self._idling:
	    self.piece.status = USED #?
	    self.destroy()
	else:
	    self._idling = True


class Convert_A_Blip(InputHandler):
    """Only called for AI voluntary deploy

    (All blips are converted in the lurkspot anyway)
    """
    
    piececlass = sprites.Genestealer
    
    
    def __init__(self, phase, blip):
	
	InputHandler.__init__(self, phase)
	self.blip = blip
	blip.hide()
	blip.square.piece.empty()
	# We leave the blip in a spritegroup (blip.team)
	# so it is still alive -- this is important KLUDGE
	
	if blip.get_lurking():
	    while blip.value > 0:
		# Place stealers in the entrytriangle limbo area
		# This fails gracefully if we run out of room in the entrytriangle (which should be impossible-ish)
		for square in [s for s in blip.square.entrytriangle.limbosquares.sprites() if not s.piece]:
		    self._put_stealer_on(square)
		    self._hesitate()
		    break
		else:
		    # We are out of room (shouldn't happen)
		    # sprites.ScrollMessage?
		    print >> DEBUG_STREAM, "Convert_A_Blip --> init, no more room in entrytriangle"
		    
	    self.destroy()
	    
	else:
            if blip.convertvoluntary: 
                self.phase.unselect() #hmm
                self.deployteam = blip.team
            else:
                raise SystemExit, "AI0 can't handle involuntary conversion!"
                self.deployteam = blip.team.enemy_team
	    raise SystemExit, "AI0 can't handle main board conversion!"
	   
	   
    def _put_stealer_on(self, square):
	"""_put_stealer_on(square) -> None
	make a stealer on square."""
	
	self.blip.value -= 1
	if isinstance(square, sprites.LimboSquare):
	    newpiece = self.piececlass(square, (square.entrytriangle.entrypoint.efacing + 2) % 4) #face the entrypoint
	    newpiece.may_unlurk_at_will = self.blip.may_unlurk_at_will
	    self.phase.game.mission.post_deploy_script(newpiece)
	else:
	    pass
	    
	
    def destroy(self):
	
	InputHandler.destroy(self)
	
	if hasattr(self, 'clock'):
	    self.clock.kill()
	    del self.clock
	self.blip.kill(NODEATH)
	del self.blip #?


class Face_A_Piece(Face_A_Piece, InputHandler):

    def create(self):

	self.created = True
        self._done = False
	self.phase.game.board.bring_into_map(self.piece)
	self.clock = sprites.FacingClock(self.piece, self.facingteam)
        self._hesitate(250)


    def handle(self):
	
	if not self.created: 
            self.create()
            return True
        if InputHandler.handle(self):
            # We are hesitating
            return True
        if self._done: 
            self.destroy()
            return True
        # Now turn the piece randomly.
        # FIXME for stealer onboard conversion
        self.piece.turn(random.choice((UP, DOWN, LEFT, RIGHT)) - self.piece.facing, True)
        self._hesitate(250)
        self._done = True
        return True


    def destroy(self):
	
	InputHandler.destroy(self)
	
	if self.created:
	    self.clock.kill()
	    del self.clock
	    del self.piece #?


class Ambush_Counter_Dialog(InputHandler):
    #IN PROGRESS
        
    def __init__(self, phase):
       
        # AI0 uses ambush if there are no stealer-controlled pieces 
        # or entrypoints within 10 squares of an enemy, *and* a 
        # random 1/2 chance comes off.

        enemyteam = phase.game.stealers.enemy_team

        if random.randrange(0, 2) > 0:
            for spr in phase.game.stealers.sprites():
                if spr.square.find_piece_near(enemyteam, 10):
                    break
            else:
                # Yes
                phase.ambush_counter = True
                sprites.ScrollMessage("AI0 chooses to use an Ambush Counter.")
                return
        # No
        phase.ambush_counter = False
