# Sulk - players - local - UIinput

#*****************************************************************************
#
#                                   Sulk
#
#       Copyright (c) 2002 Toby Woodwark <zirtix@softhome.net>
#
# This program is free software.  See the file COPYING for details.
#
#*****************************************************************************

"""
Local input classes.
"""

##############################################################################
####### Import

## Modules
import pygame, random, os #(os for abspath)
import pprint #debugging

## Sulk modules
from constants import *
import util
import sprites
import phases
import sound

##############################################################################
## Debugging

debugclasskeys = {
K_1:sprites.Terminator_Marine_with_Storm_Bolter,
K_2:sprites.Terminator_Marine_with_Heavy_Flamer,
K_3:sprites.Terminator_Marine_Sergeant,
K_4:sprites.Terminator_Marine_Sergeant_with_Power_Sword,
K_5:sprites.Terminator_Marine_Captain,
K_6:sprites.Terminator_Marine_Librarian,
K_7:sprites.Terminator_Marine_with_Lightning_Claws,
K_8:sprites.Terminator_Marine_with_Chain_Fist,
K_9:sprites.Terminator_Marine_with_Assault_Cannon,
K_0:sprites.Terminator_Marine_with_Thunder_Hammer,
K_F1:sprites.Genestealer,
K_F2:sprites.Blip,
K_F3:sprites.C_A_T,
K_F4:sprites.Ambush_Counter,
K_F8:sprites.Crate,
K_F9:sprites.Door,
K_F10:sprites.Bulkhead,
K_F11:sprites.Cryogenic_Tank,
K_F12:sprites.Rubble,
}


##############################################################################
####### InputHandlers

# These form a stack in a phase.handlers and handle UI each tick through handle(). 

class InputHandler:
    """Turns pygame events into UI interactions.
    
    Parent class.
    
    The handle() method should be called once per tick.
    
    get() is used internally to poll the event queue and returns None or a tuple of 
    (sprite,button).  It calls further methods:

        Methods called _do_...() return None or a tuple of (sprite,button).  
        Methods called _try_...() return boolean.

    Attributes:
    (instance)
        spritegroups -- tuple of spritegroups -- the target groups
        phase       -- Phase instance
        """
        
    spritegroups = ()


    def __init__(self, phase):

        for x in phase.handlers:
            print >> DEBUG_STREAM, "=lo=", #DEBUG
        print >> DEBUG_STREAM, "=lo= %s --> __init__"%self.__class__.__name__ #DEBUG
        
        phase.handlers.append(self)
        self.phase = phase

        
    def get(self):
        """get() -> tuple or None
        
        Returns a tuple of (thing, mousebutton).

        Returns None if we run out of events, or hit a button instead of
        something relevant."""
        
        if self.phase.game.paused:
            pygame.event.pump() #?
            return self._do_event(pygame.event.wait())
        else:
            return self._do_event(pygame.event.poll()) #or event.get() ?


    def _do_event(self, e):
        """_do_event(e) -> tuple or None

        Handle a generic event."""

        self._get_quit(e)

        # Look at keyboard
        if e.type == KEYDOWN: return self._do_keyb(e)

        # Look for mouse interaction
        elif e.type == MOUSEBUTTONDOWN: return self._do_mouse(e)


    def _get_quit(self, e):
        """_get_quit(e) -> boolean
        
        Look for a quit, return True if we find one"""
        
        if e.type == QUIT or (e.type == KEYDOWN and e.key == K__QUIT):
            self.phase.game.end()
            return True
        return False


#### Keyboard
    def _do_keyb(self, e):
        """_do_keyb(e) -> tuple or None
        
        Handle a keyboard event, return a (sprite, button) tuple if we find one."""
        # Subclasses should override

        # Execute '_try_' methods until one comes out true
        for grp in self.spritegroups:
            x = self._do_keyb_on_spritegroup(grp, e)
            if x: return x
        else:
            (
            self._try_keyb_on_map(e) 
            or self._try_keyb_interrupt(e)
            or self._try_keyb_on_buttons(e) 
            or self._try_keyb_on_cpiece(e)
            )


    def _do_keyb_on_spritegroup(self, spritegroup, e):
        """_do_keyb_on_spritegroup(spritegroup, e) -> tuple or None

        Try to get members of the spritegroups through matching their .key
        attribute (used for ClickBox, FacingDec, etc)."""

        if spritegroup:
            for c in spritegroup.sprites():
                if getattr(c, 'key', None) == e.key: return c, 1 #simulated left-click


    def _try_keyb_on_map(self, e):
        """_try_keyb_on_map(e) -> boolean
        
        Collect and act on keypresses for board navigation."""
        
        board = getattr(self.phase.game, 'board', None)
        e_key = e.key
        if board: #crufty
            if e_key == K__MOVE_MAP_DOWN:           
                board.shift_map((0,             -SQ_LENGTH * 2))
                return True
            elif e_key == K__MOVE_MAP_LEFT:         
                board.shift_map((SQ_LENGTH * 2, 0))
                return True
            elif e_key == K__MOVE_MAP_UP:           
                board.shift_map((0,             SQ_LENGTH * 2))
                return True
            elif e_key == K__MOVE_MAP_RIGHT:    
                board.shift_map((-SQ_LENGTH * 2,        0))
                return True
            elif e_key == K__CHEAT_DEBUG_UNDER_MOUSE:
                # Debugging stuff
                pos = pygame.mouse.get_pos()
                g = self.phase.game
                for spr in g.marines.sprites() + g.stealers.sprites() + g.features.sprites()\
                + g.squares.sprites():
                    if spr.rect.collidepoint(pos):
                        sprites.ScrollMessage("!")
                        print >> DEBUG_STREAM, "SpriteGet --> _try_keyb_on_map, debugging sprite:",spr #DEBUG
                        pprint.pprint(spr.__dict__, DEBUG_STREAM) #DEBUG
                        return True
            elif e_key == K__TOGGLE_FPS:
                self.phase.game.toggle_fpsclock()
            else:
                # Debugging stuff
                pos = pygame.mouse.get_pos()
                if e_key in debugclasskeys.keys():
                    # Spawn a piece
                    for spr in self.phase.game.squares.sprites():
                        if spr.rect.collidepoint(pos) and not spr.piece:
                            aclass = debugclasskeys[e_key]
                            sprites.ScrollMessage("!" + aclass.type)
                            if aclass == sprites.Blip :
                                val = self.phase.game.get_blipval()
                                sprites.ScrollMessage(str(val))
                                aclass(spr, val)
                            elif aclass == sprites.Ambush_Counter :
                                val = self.phase.game.get_ambush_counter_val()
                                sprites.ScrollMessage(str(val))
                                aclass(spr, val)
                            elif aclass == sprites.C_A_T:
                                sprites.C_A_T(spr)
                            else:
                                #print >> DEBUG_STREAM, "SpriteGet --> _try_keyb_on_map, cheat class:",aclass  #DEBUG
                                facing = random.randrange(0,4)
                                if issubclass(aclass, sprites.Terminator_Marine):
                                    aclass(spr, facing, "Nosquad")
                                else: 
                                    aclass(spr, facing)
                            return True
        return False


    def _try_keyb_interrupt(self, e):
        """_try_keyb_interrupt(e) -> boolean 
        
        Collect and act on keypresses for Marine interrupts."""
        
        if e.key == K__INTERRUPT and self.phase.interruptable: 
            phases.Marines_Interrupt(self.phase) #hmm
            return True
        return False


    def _try_keyb_on_buttons(self, e):
        """_try_keyb_on_buttons(e) -> boolean
        
        Collect and act on keypresses to hit buttons."""
        
        for b in self.phase.game.globalbuttons:
            if e.key == b.key: 
                b.hit()
                return True
        return False


    def _try_keyb_on_cpiece(self, e):
        """_try_keyb_on_cpiece(e) -> boolean
        
        Collect and act on keypresses to manipulate or change the current piece"""
        
        if getattr(self.phase, 'cpiece', util.Null)():  #(testing cpiece() for being None/Null also)
            # Debug
            if e.key == K__CHEAT_DEBUG:
                sprites.ScrollMessage("!")
                print >> DEBUG_STREAM, "SpriteGet --> _try_keyb_on_cpiece, cpiece.__dict__ is ..." #DEBUG
                pprint.pprint(self.phase.cpiece().__dict__, DEBUG_STREAM) #DEBUG
            elif e.key == K__CENTER_MAP_ON_CPIECE:
                print >> DEBUG_STREAM, "SpriteGet --> _try_keyb_on_cpiece, centering map" #DEBUG
                self.phase.game.board.center_map_on(self.phase.cpiece())
            # Buttons
            for b in self.phase.cpiece().buttons.sprites():
                if e.key == b.key: 
                    b.hit()
                    return True
            # Other piece-specific keyb handling
            else: return self.phase.cpiece().keyb(e.key)
        return False


#### Mouse
    def _do_mouse(self, e):
        """_do_mouse(e) -> tuple or None
        
        Handle a mouse event, return a sprite-tuple if we find one.
        We check the spritegroup last to minimize occlusion problems."""
        # Subclasses should override

        for grp in self.spritegroups:
            x = self._do_mouse_on_spritegroup(grp, e)
            if x: return x
        else:
            # Execute '_try_' methods until one comes out true
            (
            self._try_mouse_on_minimap(e) 
            or self._try_mouse_on_map(e)
            or self._try_mouse_on_buttons(e)
            )
    

    def _do_mouse_on_spritegroup(self, spritegroup, e):
        """_do_mouse_on_spritegroup(spritegroup, e) -> tuple or None

        Return a tuple of (sprite, mousebutton) if we find one under the mouse."""
        
        if spritegroup and e.button in M__SELECT:
            pos = e.pos
            for thing in spritegroup.sprites():
                # The mouse must fall in the object *and* in the object's clipping rectangle
                if thing.rect.collidepoint(pos) \
                and (thing.render_clip is None or thing.render_clip.collidepoint(pos)):
                    return thing, e.button


    def _try_mouse_on_minimap(self, e):
        """_try_mouse_on_minimap(e) -> boolean
        
        Collect and act on clicks in MINIMAP_RECT, return true if done."""
        
        if e.button in M__MOVE_MINIMAP\
        and MINIMAP_RECT.collidepoint(e.pos)\
        and hasattr(self.phase.game, 'littlemapbox'): #hmm
            offsetx = (self.phase.game.littlemapbox.rect.centerx - e.pos[0]) * MINI_SCALE
            offsety = (self.phase.game.littlemapbox.rect.centery - e.pos[1]) * MINI_SCALE
            self.phase.game.board.shift_map((offsetx, offsety))
            return True
        else: return False


    def _try_mouse_on_map(self, e):
        """_try_mouse_on_map(e) -> boolean
        
        Collect and act on clicks in MAP_RECT, return true if done."""

        # We can move the map here.
        
        if MAP_RECT.collidepoint(e.pos) and e.button in M__MOVE_MAP\
        and hasattr(self.phase.game, 'board'): #hmm
            offsetx = (MAP_RECT.centerx - e.pos[0])
            offsety = (MAP_RECT.centery - e.pos[1])
            self.phase.game.board.shift_map((offsetx, offsety), 1)
            return True
        else: return False


    def _try_mouse_on_buttons(self, e):
        """_try_mouse_on_buttons(e) -> boolean

        Collect and act on clicks on buttons, return true if done."""
        
        for dec in self.phase.game.buttons.sprites():
            if dec.rect.collidepoint(e.pos):
                dec.hit()
                return True
        else: return False
                

#### Handle + destroy
    def handle(self):
        """handle() -> None
        
        Interpret self.get() at will.
        Call destroy() at some point."""
        
        #placeholder
        pass
        
        
    def destroy(self):
        """destroy() -> None
        
        Remove self from the stack of handlers."""
        
        #hmm -- should I automatically destroy 
        #any handlers above me in the stack?  
        #hmm -- how much to help gc out with 
        #del statements (see below)?
        
        if hasattr(self, 'phase'):
            while self in self.phase.handlers:
                self.phase.handlers.remove(self)


class Action_InputHandler(InputHandler):
    """For action phase.  Permits piece-switching.

    Parent class.
    """
    
    def _do_keyb(self, e):

        for grp in self.spritegroups:
            x = self._do_keyb_on_spritegroup(grp, e)
            if x: return x
        else:
            # Execute '_try_' methods until one comes out true
            (
            self._try_keyb_on_map(e)
            or self._try_keyb_on_buttons(e)
            or self._try_keyb_nextpiece(e)   #piece-switching
            or self._try_keyb_interrupt(e) #interrupts
            or self._try_keyb_on_cpiece(e) #piece actions
            )

    
    def _try_keyb_nextpiece(self, e):
        """_try_keyb_nextpiece(e) -> boolean
        
        Collect and act on switching-piece keypresses."""

        if e.key == K__SELECT_NEXT_INACTIVE_PIECE and self.phase.secrets_team\
        and not isinstance(self.phase, InputHandler): #KLUDGE
            self.phase.secrets_team.player.UIinput.Select_Next_Inactive_Piece\
            (self.phase, self.phase.secrets_team, oldpiece=getattr(self, 'piece', None))
            return True
        return False


class Piece_Action_InputHandler(Action_InputHandler):
    """Parent for Use_A_Piece, etc.  get() selects from limbosquares and squares.
    """
    
    def __init__(self, phase, piece):

        Action_InputHandler.__init__(self, phase)
        self.spritegroups = (phase.game.squares, phase.game.limbosquares)
        self.piece = piece #hmm

        
    def _try_mouse_on_map(self, e):
        # Additional behaviour - we can turn/move piece.
        # A right-click moves the piece forwards towards the mouse.

        piece = self.piece
        if e.button in M__TURN \
        and MAP_RECT.collidepoint(e.pos) and not piece.rect.collidepoint(e.pos):
            hdng = sprites.functions.heading(piece.rect.center, e.pos, piece.facing)
            if hdng in (F_R_F, F, F_L_F):
                # Move forward
                pos = piece.square.rect.move(FACEMAP[piece.facing]).center
                sq = sprites.functions.get_square_at(pos)
                if sq is not None:
                    piece.goMove(sq)
            else:
                # Move (if we are using a BLIP)
                # or turn
                hdng_map = \
                {
                (B_L_L, L, F_L_L, F_L):LEFT,
                (F_R, F_R_R, R, B_R_R):RIGHT,
                (B_R_B, B, B_L_B):DOWN
                }
                for headings in hdng_map.keys():
                    if hdng in headings:
                        facing = hdng_map[headings]
                        if piece.type in (BLIP, CAT):
                            pos = piece.square.rect.move(FACEMAP[facing]).center
                            sq = sprites.functions.get_square_at(pos)
                            if sq is not None: 
                                piece.goMove(sq)
                        else: piece.goTurn(facing)
                        break
                else:
                    print >> DEBUG_STREAM, "can't use that heading" #DEBUG
                    pass
                return True
        else: return Action_InputHandler._try_mouse_on_map(self, e)

        
class Handle_A_Dialog(InputHandler): #this is really strict!  TODO restore map movement etc
    """Execute callbacks for a given Dialog instance.
    
    Attributes:
        dialog --  a Dialog instance
        """
        
    def __init__(self, phase, dialog):
        
        InputHandler.__init__(self, phase)
        self.dialog = dialog
        
    
    def _do_event(self, e):
        
        self._get_quit(e)
        if self.dialog.alive():
            self.dialog.handle(e)
        else:
            self.destroy()
        
        
    def handle(self):
        
        if self.dialog.alive():
            return self.get()
        else:
            self.destroy()


    def destroy(self):
        
        InputHandler.destroy(self)
        if self.dialog.alive():
            self.dialog.kill()
        del self.dialog #hmm


class Handle_Main_Menu(Handle_A_Dialog):
    """Provides a specific dialog - the main menu - and its callbacks.
    
    Pauses the game.

    Also provides subdialogs and their callbacks.
    """
    #N.B. can be called when the game is already paused.

    #This class is clunky, take care :]

    def destroy(self):
    
        self.phase.game.unpause()
        Handle_A_Dialog.destroy(self)
        

    def __init__(self, phase):
        
        g = phase.game
        
        # Pause if possible; if not, punt
        if not g.pause(): return

        gm = g.mission
        
        def backtomainmenu():
            #"copy me"
            self.destroy() #?
            self.__class__(phase)

        def configmenu(): #IN PROGRESS
            
            def saveconfig():
                print >> DEBUG_STREAM, "Handle_Main_Menu --> configmenu --> \
                saveconfig, saving config to:", options.SULKRC_FILE_NAME, #DEBUG
                if options.save_opts(options.SULKRC_FILE_NAME):
                    sprites.ScrollMessage("Options saved.")
                    print >> DEBUG_STREAM, "... success" #DEBUG
                else:
                    sprites.ScrollMessage("Error! Options could not be saved.")
                    print >> DEBUG_STREAM, "... failure" #DEBUG

            if os.access(options.SULKRC_FILE_NAME, os.F_OK):
                exist = "This file exists."
            else:
                exist = "This file does not exist."

            strings = ("The current config location is '%s'."%os.path.abspath(options.SULKRC_FILE_NAME),
                        exist,
                        "What do you want to do?")

            d = sprites.NewDialog()
            d.add_widget(sprites.MTextDisplayWidget(
            ("Config menu",strings)))
            d.add_widget(sprites.ButtonWidget("Save config now", "s", saveconfig, True))
            d.add_widget(sprites.HorizRuleWidget())        
            d.add_widget(sprites.ButtonWidget("Back", "b", backtomainmenu, True))
            d.pack()

            Handle_A_Dialog(phase, d)
                

        def soundmenu():
            
            def sound_toggle(boolean): #is this working properly?
                if boolean:
                    sound.start()
                else:
                    sound.stop()

            d = sprites.NewDialog()
            
            # Insert the right options
            if options.SOUND_ENABLED:
                
                #OLD
                #def sound_set(level):
                #    "Callback factory"
                #    # This is crufty and causes a leak if you keep pressing buttons
                #    #TODO slider widget?
                #    def f():
                #        sound.set_vol(level) #int
                #        sound.test()
                #        soundmenu() #'Call me again!'
                #    return f

                #d.add_widget(sprites.MTextDisplayWidget(
                #("Sound menu",("Current volume level: %d%%"%sound.get_vol(),"What do you want to do?",))))
                #l = [20, 40, 60, 80, 100]
                #for x in l:
                #    d.add_widget(sprites.ButtonWidget("%d%%"%x, str(x)[0], sound_set(x), True))

                d.add_widget(sprites.MTextDisplayWidget(
                ("Sound menu",("What do you want to do?",))))

                def upfunc():
                    sound.set_vol(min(100, sound.get_vol() + 5))
                    sound.test()
                    level.set_text(getthetext())
                    d.pack()
                        
                def downfunc():
                    sound.set_vol(max(sound.get_vol() - 5, 0))
                    sound.test()
                    level.set_text(getthetext())
                    d.pack()

                def getthetext():
                    return "Current volume level: %d%%"%sound.get_vol()
            
                level = sprites.CRTextWidget(getthetext())
                d.add_widget(level)
                d.add_widget(sprites.ButtonWidget("<", "<", downfunc, False))
                d.add_widget(sprites.ButtonWidget(">", ">", upfunc, False))
                
            else:
                d.add_widget(sprites.MTextDisplayWidget(
                ("Sound menu",("What do you want to do?",))))
        
            d.add_widget(sprites.CheckboxWidget("Enable sounds", "e", sound_toggle, options.SOUND_ENABLED))
            d.add_widget(sprites.HorizRuleWidget())      
            d.add_widget(sprites.ButtonWidget("Cancel", "c", backtomainmenu, True))
            d.add_widget(sprites.ButtonWidget("OK", "o", d.ok, True))
            d.pack()

            Handle_A_Dialog(phase, d)
                
        def displaymenu():

            def comments_toggle(boolean):
                if boolean:
                    g.board.build_comments()
                else:
                    g.board.kill_comments()

            def fullscreen_toggle(boolean):
                if boolean != options.SCREEN_FULLSCREEN:
                    if pygame.display.toggle_fullscreen():
                        options.SCREEN_FULLSCREEN = (not options.SCREEN_FULLSCREEN)
                    else:
                        sprites.ScrollMessage("Sorry, couldn't toggle fullscreen.")
                        
            d = sprites.NewDialog()
            d.add_widget(sprites.MTextDisplayWidget(
            ("Display menu",("What do you want to do?",))))
            d.add_widget(sprites.CheckboxWidget("Full screen", "f", fullscreen_toggle, options.SCREEN_FULLSCREEN))
                    
            # Insert a comments option if comments are available
            if g.board.get_comments_avail():
                d.add_widget(sprites.CheckboxWidget("Show board comments", "c", comments_toggle, bool(g.board.get_comments_num())))
                
            d.add_widget(sprites.HorizRuleWidget())      
            d.add_widget(sprites.ButtonWidget("Cancel", "c", backtomainmenu, True))
            d.add_widget(sprites.ButtonWidget("OK", "o", d.ok, True))    
            d.pack()
            
            Handle_A_Dialog(phase, d)

        def missionmenu():

            def mish_story():
                #subdialog
                d = sprites.NewDialog()
                d.add_widget(sprites.MTextDisplayWidget(
                ("Mission: '%s'" % gm.NAME,(gm.STORY,))))
                d.add_widget(sprites.HorizRuleWidget())
                d.add_widget(sprites.ButtonWidget("Back", "b", missionmenu, True))
                d.pack()
                Handle_A_Dialog(phase, d)

            def abort_mish():
                #subdialog
                d = sprites.NewDialog()
                d.add_widget(sprites.MTextDisplayWidget(
                ("Abort mission",("Are you sure you want to abort?",))))
                d.add_widget(sprites.HorizRuleWidget())
                d.add_widget(sprites.ButtonWidget("No", "n", missionmenu, True))
                d.add_widget(sprites.ButtonWidget("Yes", "y", g.restart, True))
                d.pack()
                Handle_A_Dialog(phase, d)
        
            # Go
            d = sprites.NewDialog()
            d.add_widget(sprites.MTextDisplayWidget(
            ("Mission: '%s'" % gm.NAME,
            ("Author: %s" % gm.AUTHOR, gm.INFO,"What do you want to do?",))))
            d.add_widget(sprites.ButtonWidget("Mission story...", "s", mish_story, True))
            d.add_widget(sprites.ButtonWidget("Abort mission...", "a", abort_mish, True))
            d.add_widget(sprites.HorizRuleWidget())      
            d.add_widget(sprites.ButtonWidget("Back", "b", backtomainmenu, True))
            d.pack()

            Handle_A_Dialog(phase, d)

        # Make the main dialog
        d = sprites.NewDialog()
        d.add_widget(sprites.MTextDisplayWidget(
        ("%s main menu"%APP_STRING,(
        "The game is paused.", "What do you want to do?"))))
        if options.SULKRC_FILE_NAME:
            d.add_widget(sprites.ButtonWidget("Config...", "c", configmenu, True))
        d.add_widget(sprites.ButtonWidget("Mission...", "m", missionmenu, True))
        d.add_widget(sprites.ButtonWidget("Display...", "d", displaymenu, True))
        d.add_widget(sprites.ButtonWidget("Sound...", "s", soundmenu, True))
        d.add_widget(sprites.HorizRuleWidget())        
        d.add_widget(sprites.ButtonWidget("Resume", "r", util.Null, True))
        d.add_widget(sprites.ButtonWidget("Exit", "E", g.end, True))
        d.pack()

        Handle_A_Dialog.__init__(self, phase, d)

    
class Select_A_Piece(Action_InputHandler):
    """Waits for the player to pick a piece to move.

    Attributes:
        team -- a SulkTeam -- team to select from
        """ 
    
    def __init__(self, phase, team):
        
        self.spritegroups = (self.team,) = (team,)
        Action_InputHandler.__init__(self, phase)
        self.phase.unselect() #hmm
        self.phase.game.board.bring_into_map(self.team)
        #(TODO? perhaps, check which ones are not USED)
        
        
    def handle(self):
        
        if self.phase.game.donebutton.status == PRESSED: 
            self.destroy()
            return
        
        select = self.get()
        
        if not select: return
        
        if select[0].status == USED:
            if self.team.type == MARINETEAM:
                if select[0].get_lurking():
                    sprites.ScrollMessage("Piece is off board.")
                    return
                elif self.phase.game.cpbutton.status != PRESSED:
                    sprites.ScrollMessage("Piece is used.")
                    return
                else:
                    # CPs a possibility, at least we should always pretend they are
                    sprites.ScrollMessage('Piece is used.  CPs only.')
            else:
                # Stealer cannot go back to used pieces; try again
                sprites.ScrollMessage("Piece is used.")
                #TODO -- a noise?
                return
            
        # We now have a piece
        self.phase.select(select[0])
        if select[0].status == INACTIVE and self.phase.game.cpbutton.status != PRESSED: 
            select[0].status = ACTIVE #USED pieces remain USED
        self.destroy()

    
def Select_Next_Inactive_Piece(phase, team, oldpiece=None):
    """Select_Next_Inactive_Piece(phase, team, oldpiece=None) -> None
    
    An InputHandler-like function for use in an action phase.
    Jumps immediately to the 'next' usable piece in team and sets up
    a Use_A_Piece inputhandler.
    """
    
    #TODO -- Check the same squad first, using get_actor().squadname ?

    if team.type == MARINETEAM and phase.game.cpbutton.status == PRESSED:
        piece = team.get_next(INACTIVE, USED) #?
    else:
        piece = team.get_next(INACTIVE)

    if not piece: 
        sprites.ScrollMessage("No more pieces.")
        return

    # Configure pieces' status properly
    piece.set_active(oldpiece)
    
    # Start using that new piece
    team.player.UIinput.Use_A_Piece(phase, piece)
    

class Use_A_Piece(Piece_Action_InputHandler):
    """Polls for the player to pick a square, and turns this into piece actions.

    If click is on an enmy piece, attempt attack.
    If click is on door, try to use it, etcetera.
    
    Attributes:
        piece -- a Piece -- being used
        """
        
    def __init__(self, phase, piece):
        
        Piece_Action_InputHandler.__init__(self, phase, piece)
        self.phase.select(piece) #hmm
        self.phase.game.board.bring_into_map(piece)


    def destroy(self):
        
        self.phase.unselect()
        del self.piece
        Piece_Action_InputHandler.destroy(self)


    def handle(self):
        
        def nextpiece(piece):
            "Go straight to another piece."
            piece.set_active(oldpiece=self.piece)       
            # Now actually switch
            p = self.phase
            self.destroy()
            self.__class__(p, piece) #'clone me'

        def gofriendly(piece):
            "Tries to switch to a piece."
            # We can't let marines switch piece in interrupts:
            if self.piece.team.status == ACTIVE:
                if (self.piece.team.type == STEALERTEAM or self.phase.game.cpbutton.status != PRESSED)\
                and piece.status == USED:
                    sprites.ScrollMessage('Piece is used.')
                else:
                   nextpiece(piece)
        
        # Go -- 
        # First test to see if being the active handler makes sense,
        # and punt if not
        if not self.piece.alive():
            print >> DEBUG_STREAM, "Use_A_Piece --> handle, self.piece is dead" #DEBUG
            self.destroy()
            return

        elif isinstance(self.piece, sprites.Terminator_Marine) and isinstance (self.piece.square, sprites.LimboSquare):
            print >> DEBUG_STREAM, "Use_A_Piece --> handle, self.piece is on wrong kind of square" #DEBUG
            self.destroy()
            return
        
        elif self.piece.team.status == ACTIVE and self.piece.status != ACTIVE\
        and (self.piece.team.type != MARINETEAM or self.phase.game.cpbutton.status != PRESSED):
            # e.g. when a stealer is USED for some reason -- we must die
            print >> DEBUG_STREAM, "Use_A_Piece --> handle, self.piece is not ACTIVE" #DEBUG
            self.destroy()
            return
        
        elif not self.piece.apremaining and \
        (self.piece.team.type == STEALERTEAM and isinstance(self.piece, sprites.Blip) or \
        (self.piece.team.type == MARINETEAM and self.piece.team.status == ACTIVE\
        and not self.piece.try_mns and self.phase.game.cpbutton.status != PRESSED)): #hmm
            sprites.ScrollMessage("Out of APs.")
            self.piece.status = USED
            self.destroy()
            return
        
        elif self.piece != self.phase.cpiece(): 
            self.phase.select(self.piece)
        
        # Now select a square
        select = self.get() #gets a square or limbosquare
        if not select: return
        
        #print >> DEBUG_STREAM, "Use_A_Piece --> handle, select being processed:",select #DEBUG
        
        # We know we have a square; work starts
        selected_square = select[0]
        
        if selected_square.piece:
            selected_piece = selected_square.piece.sprites()[0]
            if selected_piece is self.piece: 
                print >> DEBUG_STREAM, "Use_A_Piece --> handle, hit self.piece, nothing to do" #DEBUG
            elif self.piece.team.has(selected_piece): 
                # Switching
                gofriendly(selected_piece)
            else: 
                # Attacking
                self.piece.goHostile(selected_piece)
            
        elif selected_square.features:
            # Attacking, or using a door...
            selected_feature = selected_square.features.sprites()[0] 
            #FIXME there could be more than 1 feature... (how to fix?)
            if selected_feature.blocksmove:
                if isinstance(selected_feature, sprites.Door): 
                    self.piece.goDoor(selected_feature)
                else: self.piece.goHostile(selected_feature)
            else: self.piece.goSquare(selected_square)
            
        else:
            # Movement of some kind
            #print >> DEBUG_STREAM, "Use_A_Piece --> handle, plain square selected" #DEBUG
            result = self.piece.goSquare(selected_square)  
            if not result and self.piece.get_points_avail(): 
                # Primitive feedback messages 
                #hmm, could perhaps be done in the Piece.go...() methods
                if self.piece.get_lurking(): 
                    sprites.ScrollMessage("Can't unlurk.")
                elif getattr(self.piece, "overwatching", 0):
                    sprites.ScrollMessage("Overwatching.")


class Handle_Piece_Dialog(Handle_A_Dialog):
    """Parent class
    
    Attributes:
    (instance)
        piece -- a Piece
        """

    def __init__(self, phase, piece, dialog):

        self.piece = piece
        Handle_A_Dialog.__init__(self, phase, dialog)


    def handle(self):
        
        if not self.piece.alive(): 
            self.destroy() 
        else:
            Handle_A_Dialog.handle(self)
        
        
    def destroy(self):
        
        Handle_A_Dialog.destroy(self)
        if self.piece.alive(): 
            self.piece.build_visuals()
            self.piece.build_buttons()



class Parry(Handle_Piece_Dialog):
    """Checks a dialog box for whether player is sure he wants to parry.
        """
    
    def __init__(self, phase, piece, tscore):
        
        def yesfunc():
            piece.answer = True
            sprites.ScrollMessage("Parried.")

        def nofunc():
            piece.answer = False
            sprites.ScrollMessage("Parry declined.")
            
        d = sprites.NewDialog()
        d.add_widget(sprites.MTextDisplayWidget(
        ("Do you want to parry %s ?"%str(tscore),(
        "This will force the %s to reroll the best die (%d)"%(piece.team.enemy_team.type, max(tscore)),))))
        d.add_widget(sprites.HorizRuleWidget())
        d.add_widget(sprites.ButtonWidget("Yes", "y", yesfunc, 1))
        d.add_widget(sprites.ButtonWidget("No", "n", nofunc, 1))
        
        d.pack()    
                        
        Handle_Piece_Dialog.__init__(self, phase, piece, d)

            
class Number_of_Psipoints(Handle_Piece_Dialog):
    """Discover how many psipoints a player wants to spend.
        """
    
    def __init__(self, phase, piece):
        
        self.psipoints = 0
        
        def endfunc():
            piece.answer = self.psipoints
            
        def upfunc():
            if piece.psipointsremaining >= self.psipoints: 
                self.psipoints += 1
                self.text.set_text(getthetext())
                d.pack()
                
        def downfunc():
           if self.psipoints >= 1: 
            self.psipoints -= 1
            self.text.set_text(getthetext())
            d.pack()

        def getthetext():
            return "Number of psi points to spend: " + str(self.psipoints)
            
        d = sprites.Dialog()
        self.text = sprites.CRTextWidget(getthetext())
        d.add_widget(self.text)
        d.add_widget(sprites.HorizRuleWidget())
        d.add_widget(sprites.ButtonWidget("<", "<", downfunc, False))
        d.add_widget(sprites.ButtonWidget(">", ">", upfunc, False))
        d.add_widget(sprites.ButtonWidget("Ok", "O", endfunc, True))
        d.pack()    
                        
        Handle_Piece_Dialog.__init__(self, phase, piece, d)


class Move_A_CAT(InputHandler):
    """Use a CAT.
    """

    def __init__(self, phase, cat):
         
        self.cat = cat
        
        InputHandler.__init__(self, phase)
        
        phase.select(self.cat)
        if sprites.DisplayDie().pips >= 4: team = phase.game.marines
        else: team = phase.game.stealers
        sprites.ScrollMessage("The %s move the %s."%(team.type, self.cat.type))
        self.spritegroups = (self.phase.game.squares,)
        

    def _do_keyb(self, e):
        
        x = Action_InputHandler._do_keyb(self, e)
        if x: return x

        self._try_keyb_on_cat(e)


    def _try_keyb_on_cat(self, e):
        """_try_keyb_on_cat(e) -> boolean"""
        
        # Debug
        if e.key == K__CHEAT_DEBUG:
            sprites.ScrollMessage("!")
            pprint.pprint(self.cat.__dict__, DEBUG_STREAM) #DEBUG
        elif e.key == K__CENTER_MAP_ON_CPIECE:
            print >> DEBUG_STREAM, "SpriteGet --> _try_keyb_on_cat, centering map" #DEBUG
            self.phase.game.board.center_map_on(self.cat)
        # cat keyb handling
        else: return self.cat.keyb(e.key)
        
        
    def handle(self):

        if not self.cat.alive():
            print >> DEBUG_STREAM, "Move_CATs --> handle, self.cat is dead" #DEBUG
            self.destroy()

        elif isinstance(self.cat.square, sprites.LimboSquare):
            print >> DEBUG_STREAM, "Move_CATs --> handle, self.cat is on wrong kind of square" #DEBUG
            #SHOULD NOT HAPPEN
            self.destroy()
        
        elif not self.cat.apremaining:
            sprites.ScrollMessage("Out of APs.")
            self.cat.status = USED
            self.destroy()
        
        else:
            # Now select a square
            select = self.get()
            if select: self.cat.goMove(select[0])


    def destroy(self):

        self.phase.unselect()
        InputHandler.destroy(self)
        del self.cat


class Selfdestruct_A_Piece(Handle_Piece_Dialog):
    """Checks a dialog box for whether player is sure he wants to self-destruct his piece.
        """
    
    def __init__(self, phase, piece):
        
        def yesfunc():
            # Try to kill the piece
            if not piece.goSelfdestruct(): 
                sprites.ScrollMessage("Couldn't self-destruct.")
        
        d = sprites.NewDialog()
        d.add_widget(sprites.MTextDisplayWidget(
        ("Really self-destruct?",(
        "This will destroy every piece and door on the board section.",))))
        d.add_widget(sprites.HorizRuleWidget())        
        d.add_widget(sprites.ButtonWidget("No", "n", util.Null, True))
        d.add_widget(sprites.ButtonWidget("Yes", "y", yesfunc, True))
        d.pack()    
                        
        Handle_Piece_Dialog.__init__(self, phase, piece, d)


class Shoot_Something(Action_InputHandler):
    """This is initiated by the shoot button and forces choice of a target.
    
    Attributes:
        piece -- a Piece
        """

    def __init__(self, phase, piece):
        
        Action_InputHandler.__init__(self, phase)
        self.piece = piece


    def destroy(self):
        
        Action_InputHandler.destroy(self)
        #self.controlling_button.unpress() #infinite recursion here
        #del self.piece #?
        del self.controlling_button #?        


    def handle(self):
        
        if not self.piece.alive(): 
            self.destroy()
            return
        
        elif self.piece.weapon is GUN: 
            # Get an enemy piece or a feature
            targetables = pygame.sprite.Group()
            targetables.add(self.piece.team.enemy_team)
            
            for f in self.phase.game.features.sprites():
                if f.sh_kill_scorereq: targetables.add(f)

            self.spritegroups = (targetables, self.piece.team)
            
        elif self.piece.weapon == FLAMEGUN: 
            # Get a square
            self.spritegroups = (self.phase.game.squares, self.piece.team)
            
        select = self.get()
        if select:
            # Now we have something to shoot at
            #if self.piece.goShoot(select[0]): self.destroy()
            #else: sprites.ScrollMessage("Can't shoot that.")
            if not self.piece.goShoot(select[0]):
                sprites.ScrollMessage("Can't shoot that.")


class Fire_Grenade(Action_InputHandler): #IN PROGRESS
    """This is initiated by the grenade button and forces choice of a target.
    
    Attributes:
        piece -- a Piece
        """

    def __init__(self, phase, piece):
        
        Action_InputHandler.__init__(self, phase)
        self.piece = piece


    def destroy(self):
        
        Action_InputHandler.destroy(self)
        #self.controlling_button.unpress() #infinite recursion here
        #del self.piece #?
        del self.controlling_button #?        


    def handle(self):
        
        if not self.piece.alive(): 
            self.destroy()
            return
        self.spritegroups = (self.phase.game.squares,)
            
        select = self.get()
        if select:
            # Now we have something to fire granade at at
            if not self.piece.goFireGrenade(select[0]):
                sprites.ScrollMessage("Can't fire a grenade there.")


class Use_A_Door(Action_InputHandler):
    """This is initiated by the use door button when pressed
    
    Forces choice of a door.
    
    Should be called only if a piece has a choice between 
    many doors - which is rare.
    """
    
    def __init__(self, phase, piece, candidates):
        #candidates is a group of doors
        
        Action_InputHandler.__init__(self, phase)

        self.piece = piece

        for p in candidates.sprites():
            x = sprites.HiLite(p.square, REDHILITE)
            x.tt_text = lambda: "Use this door" #hmm

        self.spritegroups = (candidates,)

        sprites.ScrollMessage("Pick a door.")
    
    
    def handle(self):
        
        if not self.piece.alive(): 
            self.destroy()
            return
        
        select = self.get()
        if select:
            self.piece.goDoor(select[0], hostility=0)
            self.destroy()


    def destroy(self):

        Action_InputHandler.destroy(self)
        
        del self.controlling_button.spawned #ref to self
        self.controlling_button.unpress()#??
        
        for p in self.spritegroups[0].sprites(): 
            p.square.kill_hilite()
            
        self.spritegroups[0].empty()
        del self.phase #?
        del self.controlling_button #?


class Pass_CAT(InputHandler):
    """This is initiated by the CATButton when it is *un*pressed.
    """

    def __init__(self, phase, piece, cat, candidates):
        
        InputHandler.__init__(self, phase)

        self.piece = piece
        self.cat = cat

        for p in candidates.sprites():
            sprites.HiLite(p.square, REDHILITE).tt_text = lambda: "Pass the %s here"%cat.type

        self.spritegroups = (candidates,)
        sprites.ScrollMessage("Pick a Marine to take the %s."%cat.type)
    
    
    def handle(self):
        #CODE COPYING from Use_A_Door
        
        if not self.piece.alive() or not self.cat.alive(): 
            self.destroy()
            return
        
        select = self.get()
        if select:
            self.piece.goPass_cat(self.cat, select[0])
            self.destroy()


    def destroy(self):

        InputHandler.destroy(self)

        del self.controlling_button.spawned #ref to self

        for p in self.spritegroups[0].sprites(): 
            p.square.kill_hilite()
        
        self.spritegroups[0].empty()
        del self.cat, self.piece, self.phase #?
        del self.controlling_button #?


class Deploy_A_New_Piece(InputHandler):
    """Forces choice of a square/limbosquares to deploy a piece on.

    N.B. *A* Piece; we die after that piece goes down.
    
    This is used in Deploy_ Phases and for reinforcements - never in an action phase.
    We call mission.pre_deploy_rule to check that squares are valid to deploy to.
    
    Attributes:
        _deployable_squares -- pygame.sprite.Group
        _d_clock        -- a sprites.DeployClock instance
        deployteam      -- SulkTeam -- the team of the player placing the piece
        piececlass      -- a subclass of Piece to deploy
        value           -- if piececlass is sprites.Blip, the value of the blip
        """
    
    def __init__(self, phase, piececlass, deployteam=None, squadname=None):
        
        InputHandler.__init__(self, phase)
        
        self.piececlass = piececlass
        self.squadname = squadname
        if deployteam is None: self.deployteam = self.piececlass.team
        else: self.deployteam = deployteam

        # Allow secret toggle
        self.team = self.deployteam

        self._d_clock = sprites.DeployClock(self)
        self.spritegroups = (_deployable_squares,) = (pygame.sprite.Group(),)
        
        # See which squares are permissible
        pre_deploy_rule = self.phase.game.mission.pre_deploy_rule #TA
        add = _deployable_squares.add #TA
        
        if self.piececlass.team.type == MARINETEAM:
            for sqr in self.phase.game.squares.sprites():
                if sqr.piece or not (sqr.isdeploy and sqr.get_passable()) or\
                not pre_deploy_rule(piececlass, sqr, squadname): 
                    continue
                add(sqr)

        elif piececlass is sprites.Ambush_Counter:
            # we are deploying an ambush counter
            self.value = self.phase.game.get_ambush_counter_val()
            for sqr in self.phase.game.squares.sprites():
                # can't deploy ambush counters near a marine
                if not sqr.piece and sqr.get_passable() and\
                pre_deploy_rule(piececlass, sqr) and not sqr.get_team_is_near(piececlass.team.enemy_team):
                    for marine in self.team.enemy_team.sprites():
                        # can't deploy a counter in marine LOS
                        if marine.can_see(sqr):
                            break
                    else: add(sqr)
                
        elif piececlass.team.type == STEALERTEAM:
            # We are deploying a blip
            self.value = self.phase.reinflist[0] #first element
            for tri in self.phase.game.entrytriangles.sprites():
                # NB stealers apply the pre_deploy_rule to _entrypoints_ :
                if not pre_deploy_rule(piececlass, tri.entrypoint): 
                    continue
                havepiecesalready = []
                # May not deploy to entry areas with 3 blips already
                for sqr in tri.limbosquares.sprites():
                    if sqr.piece.sprite.__class__ is piececlass: #hmm, relying on GroupSingle internals
                        havepiecesalready.append(sqr)
                if len(havepiecesalready) > 2: 
                    #RULES--> may not deploy to entrytriangles with > 2 blips already
                    continue
                for sqr in tri.limbosquares.sprites():
                    if not sqr.piece:
                        add(sqr)
                            
        else: print >> DEBUG_STREAM, "Deploy_A_New_Piece --> init, error: unknown self.piececlass.team.type"#DEBUG
        
        if not _deployable_squares:
            print >> DEBUG_STREAM, "Deploy_A_New_Piece --> init, nowhere to deploy!"#DEBUG
            self.phase.reinflist = [] # SE from bug 729687            
            self.destroy()
        else:
            hi = sprites.HiLite
            for sqr in _deployable_squares.sprites():
                hi(sqr, GREENHILITE).tt_text = lambda: "Deploy %s here"%self.piececlass.type
            self.phase.game.board.bring_into_map(_deployable_squares)

    
    def handle(self):

        select = self.get()
        
        if select: 
            if self.piececlass is sprites.Blip:  
                # Make a blip
                self.phase.game.mission.post_deploy_script(sprites.Blip(select[0], self.value))
                del self.phase.reinflist[0]
                
            elif self.piececlass is sprites.Ambush_Counter:
                 self.phase.game.mission.post_deploy_script(self.piececlass(select[0], self.value))
                 
            else: 
                # Get a facing
                facing = self.piececlass.team.find_a_nice_facing()
                
                # Adjust the facing so a square is ahead of the new piece
                i = 0
                while not sprites.functions.get_square_at(select[0].rect.move(FACEMAP[facing]).center) and i < 5:
                    i += 1
                    facing = (facing + 1) % 4
                # Make a piece
                if issubclass(self.piececlass, sprites.Terminator_Marine):
                    newpiece = self.piececlass(select[0], facing, self.squadname)
                else: 
                    newpiece = self.piececlass(select[0], facing)        
                self.phase.game.mission.post_deploy_script(newpiece)
                
                # New inputhandler
                self.deployteam.player.UIinput.Face_A_Piece(self.phase, newpiece, self.deployteam)
            
            self.destroy()


    def destroy(self):

        if self.piececlass is sprites.Ambush_Counter:
            print "BLAAARG"
            self.phase.done_deploying_ambush_counter = True
            self.phase.deploying_ambush_counter = False
        InputHandler.destroy(self)
        self._d_clock.kill()
        for sqr in self.spritegroups[0].sprites(): 
            sqr.kill_hilite()
        del self._d_clock  #?
        if hasattr(self, 'value'): del self.value
        self.spritegroups[0].empty()


class Deploy_A_New_CAT(Deploy_A_New_Piece):
    """Forces choice of a square/square-with-marine to deploy a C.A.T. on.

    We call mission.pre_deploy_rule only if the method is ONBOARD.
    
    Attributes:
        _deployable_squares -- pygame.sprite.Group
        _self._d_clock          -- a sprites.DeployClock instance
        deployteam      -- SulkTeam -- the player placing the piece
        piececlass      -- subclass of sprites.Piece -- the class to deploy
        method          -- int -- How we are dealing with deployment, WITHSQUAD or ONBOARD
        """

    piececlass = sprites.C_A_T

    #NB only destroy() is inherited really
    
    def __init__(self, phase, deployteam, squadname=None, method=WITHSQUAD):
        
        assert (squadname or method != WITHSQUAD)
        
        InputHandler.__init__(self, phase)
        
        self.method = method
        self.squadname = squadname
        if deployteam is None: self.deployteam = self.phase.game.marines #?
        else: self.deployteam = deployteam

        # Allow secret toggle
        self.team = self.deployteam

        # Display deploy info
        self._d_clock = sprites.DeployClock(self)

        # Find deployable squares
        self.spritegroups = (_deployable_squares,) = (pygame.sprite.Group(),)

        add = _deployable_squares.add #TA
        pre_deploy_rule = self.phase.game.mission.pre_deploy_rule #TA
        
        if self.method == WITHSQUAD:
            # We must choose squares with marines on
            
            #RULES can marines carry any number of C.A.T.s/possessions?
            
            #hmm, what should happen if *any* marine is permitted to carry the cat
            #(squad irrelevant)?
            for sqr in self.phase.game.squares.sprites():
                for piece in sqr.piece.sprites():
                    # We need a marine with the right squadname
                    if isinstance(piece, sprites.Terminator_Marine)\
                    and getattr(piece, 'squadname', None) == squadname:
                        add(sqr)
                
        elif self.method == ONBOARD:
            # We must choose generic squares
            for sqr in self.phase.game.squares.sprites():
                if (sqr.isdeploy and sqr.get_passable()) \
                and pre_deploy_rule(sprites.C_A_T, sqr, squadname): 
                    add(sqr)
                    
        else: print >> DEBUG_STREAM, "Deploy_A_New_CAT --> init, error: unknown method %d" % method#DEBUG
        
        if not _deployable_squares:
            print >> DEBUG_STREAM, "Deploy_A_New_CAT --> init, nowhere to deploy!"#DEBUG
            self.destroy()
        else:
            hi = sprites.HiLite
            for sqr in _deployable_squares.sprites():
                hi(sqr, GREENHILITE).tt_text = lambda: "Deploy %s here"%self.piececlass.type
            self.phase.game.board.bring_into_map(_deployable_squares)

    
    def handle(self):

        select = self.get()
        
        if not select: return
        
        newpiece = self.piececlass(select[0])
        
        if self.method == WITHSQUAD:  
            newpiece.set_piece(select[0].piece.sprites()[0])
        
        self.phase.game.mission.post_deploy_script(newpiece)
        self.destroy()


class Ambush_Counter_Dialog(Handle_A_Dialog):
    """Checks a dialog box for whether player wants to place an ambush counter
        """
        
    def __init__(self, phase):
       
        def yesfunc():
            self.phase.ambush_counter = True

        def nofunc():
            self.phase.ambush_counter = False
            
        self.phase = phase
        d = sprites.NewDialog()
        d.add_widget(sprites.MTextDisplayWidget(
        ("Ambush?",("Stealer player: you may place an ambush counter at the end of this turn.","Do you want to do this?"))))
        d.add_widget(sprites.HorizRuleWidget())        
        d.add_widget(sprites.ButtonWidget("No", "n", nofunc, 1))
        d.add_widget(sprites.ButtonWidget("Yes", "y", yesfunc, 1))
        d.pack()      
        
        Handle_A_Dialog.__init__(self, phase, d)


class Convert_A_Blip(InputHandler):
    """Forces choice of the square to deploy a piece on.  
    
    Called by blip.convert().
    In Stealer_Action_Phase (i.e. voluntary, can be lurking)
    or in Marine_Action_Phase when a marine sees a blip (involuntary)
    
    Attributes:
    (instance)
        blip        -- the Blip
        deployteam  -- team responsible for placing stealers
        _deployable_squares -- spritegroup
        clock       -- DeployClock instance
        """
    
    piececlass = sprites.Genestealer
    
    
    def __init__(self, phase, blip):
        
        InputHandler.__init__(self, phase)
        
        phase.interruptable = False
        
        self.created = False
        self.blip = blip
        self.newpieces = []
        self.spritegroups = (pygame.sprite.Group(),)


    def create(self):
        """create() -> None

        Make UI"""

        self.created = True
        
        blip = self.blip
        blip.hide()
        blip.square.piece.empty()
        # We leave the blip in a spritegroup (blip.team)
        # so it is still alive -- this is important
        
        if blip.convertvoluntary: 
            self.phase.unselect() #hmm
            self.deployteam = blip.team
        else: 
            self.deployteam = blip.team.enemy_team
            self.clock = sprites.DeployClock(self)

        if blip.get_lurking():
            while blip.value > 0:
                # Place stealers in the entrytriangle limbo area
                # This fails gracefully if we run out of room in the entrytriangle (which should be impossible-ish)
                for square in [s for s in blip.square.entrytriangle.limbosquares.sprites() if not s.piece]:
                    self._put_stealer_on(square)
                    break
                else:
                    # We are out of room (shouldn't happen)
                    # sprites.ScrollMessage?
                    print >> DEBUG_STREAM, "Convert_A_Blip --> init, no more room in entrytriangle"
                    
            self.destroy()
            
        else:
            # Place first stealer
            self._put_stealer_on(blip.square)
           
           
    def _prep_stealer(self):
        """_prep_stealer() -> int
        
        Prepare to place a stealer.
        Return number of choosable squares."""
        
        blip = self.blip
        _deployable_squares = self.spritegroups[0]
        # Center map
        self.phase.game.board.bring_into_map(blip.square)
        # Now look for other squares to deploy to
        if blip.value <= 0: return
        self.clock = sprites.DeployClock(self)
        # Prep shortcuts
        text = "Deploy %s here"%self.piececlass.type
        enemy_team = blip.team.enemy_team.sprites()
        voluntary = blip.convertvoluntary
        flaming = blip.square.get_flaming()
        for sqr in blip.square.get_adjacents():
            if sqr.get_passable(flaming):
                if voluntary: #marine los relevant:
                    for marine in enemy_team:
                        if marine.can_see(sqr): break   
                    else: #no marines can see:
                        _deployable_squares.add(sqr)
                        sprites.HiLite(sqr, GREENHILITE).tt_text = lambda: text
                else: #los irrelevant:
                    _deployable_squares.add(sqr)  
                    sprites.HiLite(sqr, GREENHILITE).tt_text = lambda: text
        
        return len(_deployable_squares)

        
    def _put_stealer_on(self, square):
        """_put_stealer_on(square) -> None
        
        Make a stealer on the square."""
        
        self.blip.value -= 1
        if hasattr(self, 'clock'): 
            self.clock.kill()
            del self.clock
        if isinstance(square, sprites.LimboSquare):
            newpiece = self.piececlass(square, (square.entrytriangle.entrypoint.efacing + 2) % 4) #face the entrypoint
            newpiece.may_unlurk_at_will = self.blip.may_unlurk_at_will
            self.phase.game.mission.post_deploy_script(newpiece)
        else:
            self.spritegroups[0].remove(square)
            square.kill_hilite()
            newpiece = self.piececlass(square, self.blip.team.find_a_nice_facing())
            if self.blip.convertvoluntary or self.blip.ap == self.blip.apremaining: 
                newpiece.status = INACTIVE
            elif self.blip.type == AMBUSH_COUNTER:
                if self.blip.status == USED:
                    newpiece.status = USED #RULES?
                else:
                    newpiece.status = self.blip.status
                    newpiece.apremaining = self.blip.apremaining
            else: 
                newpiece.status = USED
            self.phase.game.mission.post_deploy_script(newpiece)
        self.newpieces.append(newpiece)
        for sqr in self.spritegroups[0].sprites():
            sqr.kill_hilite()
        self.spritegroups[0].empty()
            
        
    def handle(self):
        
        if not self.created: 
            self.create()
            return #needed for limbo cases
        if self.blip.value <= 0:
            self.destroy()
        elif not self.spritegroups[0]:
            # Find the available squares
            num_squares = self._prep_stealer()
            if not num_squares:
                if self.blip.value == 1:
                    mesg = "A %s is lost."%self.piececlass.type
                else:
                    mesg = "%d %ss are lost."%(self.blip.value, self.piececlass.type)
                sprites.ScrollMessage(mesg)
                self.destroy()
            elif num_squares == self.blip.value:
                # No choice is possible, just slap one on
                self._put_stealer_on(self.spritegroups[0].sprites()[0])
        else: 
            self.blip._markers.sprite.update() #KLUDGE
            # Get the required square
            select = self.get()
            if select: self._put_stealer_on(select[0])


    def destroy(self):
        
        if self.blip.type == AMBUSH_COUNTER:
            for newpiece in self.newpieces: 
                newpiece.update_sees_me()
        
        if not self.blip.get_lurking():
            # The stealer team always chooses facing
            for p in self.newpieces:
                Face_A_Piece(self.phase, p)

        del self.newpieces
        
        self.phase.interruptable = self.phase.__class__.interruptable #?
        
        InputHandler.destroy(self)
        
        for sqr in self.spritegroups[0].sprites():
            sqr.kill_hilite()
        self.spritegroups[0].empty()
        
        if hasattr(self, 'clock'):
            self.clock.kill()
            del self.clock
        self.blip.kill(NODEATH)
        del self.blip #?


class Face_A_Piece(InputHandler):
    """Forces choice of a facing for a piece without one.
    
    Attributes:
    (instance)
        created     -- boolean
        facinggroup -- a spritegroup -- the facing-choosing sprites, plus piece
        piece       -- a Piece -- the piece to be faced
        clock       -- FacingClock instance
        """
    
    def __init__(self, phase, piece, facingteam=None):
        
        InputHandler.__init__(self, phase)
        if facingteam is None: self.facingteam = piece.team
        else: self.facingteam = facingteam
        self.piece = piece
        self.created = False


    def create(self):
        """create() -> None

        Make UI"""

        self.created = True
        self.phase.game.board.bring_into_map(self.piece)
        self.facinggroup = pygame.sprite.Group()
        self.facinggroup.add(
        [sprites.FacingDec(self.piece, UP),
        sprites.FacingDec(self.piece, RIGHT),
        sprites.FacingDec(self.piece, DOWN),
        sprites.FacingDec(self.piece, LEFT), 
        self.piece])
        self.spritegroups = (self.facinggroup,)
        self.clock = sprites.FacingClock(self.piece, self.facingteam)


    def handle(self):
        
        if not self.created: self.create()
        select = self.get() #RULES --> whose secrets?
        if select: 
            # Now turn the piece. (If select[0] is self.piece, the original
            # (find_a_nice_facing) is automatically kept.)
            # We pass True to Piece.turn() to stop piece getting shot in overwatch.
            self.piece.turn(select[0].facing - self.piece.facing, True)
            self.destroy()


    def destroy(self):
        
        InputHandler.destroy(self)
        
        if self.created:
            self.clock.kill()
            del self.clock
            self.facinggroup.remove(self.piece)
            del self.piece #?
            for arrow in self.facinggroup.sprites(): 
                arrow.kill()
            del self.facinggroup
