# Sulk - players - local - phases

#*****************************************************************************
#
#                                   Sulk
#
#       Copyright (c) 2002 Toby Woodwark <zirtix@softhome.net>
#
# This program is free software.  See the file COPYING for details.
#
#*****************************************************************************

"""
Sulk rules objects.
"""

##############################################################################
####### Import / set up

## Modules
import pygame, random
import pprint #debug

## Sulk modules
from constants import *
import util
import sprites
import missions
import UIinput

# We need a mapping of strings in mission module onto sprite classes
class_deploy_map = {\
CAT:                                    sprites.C_A_T,
TERMINATOR_MARINE_WITH_STORM_BOLTER:    sprites.Terminator_Marine_with_Storm_Bolter,
TERMINATOR_MARINE_WITH_HEAVY_FLAMER:    sprites.Terminator_Marine_with_Heavy_Flamer,
TERMINATOR_MARINE_WITH_CHAIN_FIST:      sprites.Terminator_Marine_with_Chain_Fist,
TERMINATOR_MARINE_WITH_ASSAULT_CANNON:  sprites.Terminator_Marine_with_Assault_Cannon,
TERMINATOR_MARINE_WITH_LIGHTNING_CLAWS: sprites.Terminator_Marine_with_Lightning_Claws,
TERMINATOR_MARINE_WITH_THUNDER_HAMMER:  sprites.Terminator_Marine_with_Thunder_Hammer,
TERMINATOR_MARINE_SERGEANT:             sprites.Terminator_Marine_Sergeant,
TERMINATOR_MARINE_SERGEANT_WITH_SWORD:  sprites.Terminator_Marine_Sergeant_with_Power_Sword,
TERMINATOR_MARINE_CAPTAIN:              sprites.Terminator_Marine_Captain,
TERMINATOR_MARINE_LIBRARIAN:            sprites.Terminator_Marine_Librarian,
}


##############################################################################
####### Game Phase (UI handlers level 2)

# A Phase supervises the selection of UI handlers and checks for control 
# passing between players.

class Phase:
    """A container for InputHandlers.

    A Phase instance has loop() called each tick by the game loop.
    Phases can create other Phases.  Only one Phase exists at once.
    
    Parent class.
    
    Attributes:
    (class)
	name	        -- string
	interruptable   -- bool -- if true, Marines_Interrupt is permitted 
	pausable        -- bool -- if true, SulkGame.pause() is permitted
	_secrets_team_type -- int or None -- the team type using secrets
    (instance)
	secrets_team    -- SulkTeam or None
	handlers        -- a list of InputHandlers
	game	        -- a SulkGame
	_select_grp     -- GroupSingle, containing the 'current' Piece (if any)
	"""
	
    pausable = True
    interruptable = False
    name = ""
    _secrets_team_type = None

    
    class _SelectGroup(pygame.sprite.GroupSingle):
        """A group for Piece instances to live in.  Handles UI decorations.
        
            """
        
        def add_internal(self, sprite):
            
            assert isinstance(sprite, sprites.Piece)
            if self.sprite is not 0:
                self.sprite.remove_internal(self)
                self.remove_internal(self.sprite) #?
            self.sprite = sprite
            sprite.build_visuals()
            sprite.build_buttons()
            
            
        def remove_internal(self, sprite):
            
            assert isinstance(sprite, sprites.Piece)
            self.sprite = 0
            sprite.kill_visuals()
            sprite.kill_buttons()


        def empty(self):
            
            if self.sprite is not 0:
                self.sprite.remove_internal(self)
                self.remove_internal(self.sprite)
    
    
    def __init__(self, sulkgame):
	
	#print >> DEBUG_STREAM, "Phase --> init , class:", self.__class__.__name__ #DEBUG
	
	self.game = sulkgame
	self.handlers = []
	sulkgame.phase = self
	self._select_grp = self._SelectGroup()
	self.secrets_team = sulkgame.teams.get(self._secrets_team_type) #team or None
        for team in (sulkgame.stealers, sulkgame.marines):
            if hasattr(team.enemy_team,'player')\
            and team.enemy_team.player.type is not LOCAL_PLAYER\
            and team is self.secrets_team:
	        team.show_secrets()
            else:
                team.hide_secrets()


    def select(self, piece):
	"""select(piece) -> None

	Make a Piece instance the _current_ piece."""
	
	if self._select_grp.sprite is piece:
	    print >> DEBUG_STREAM, "Phase --> select --> piece is already selected" #DEBUG
	elif not piece.alive(): 
	    print >> DEBUG_STREAM, "Phase --> select --> piece is dead" #DEBUG
	else:
	    self._select_grp.add(piece)
	
	
    def unselect(self):
	"""unselect() -> None
    
	Make no piece current."""
	
        self._select_grp.empty()


    def cpiece(self):
        """-> Null or current piece"""

        return self._select_grp.sprite or util.Null
	

    def loop(self):
	"""loop() -> None
	process 1 game loop of interaction"""
        #overload
	
	if not self.handlers: self._end()
	else: self.handlers[-1].handle()


    def _end(self):
	"""_end() -> None
	
	End of this phase"""
	
	#print >> DEBUG_STREAM, "Phase --> _end , class:", self.__class__.__name__ #DEBUG
	#print >> DEBUG_STREAM, "Phase --> _end --> handlers:", self.handlers #DEBUG
	while self.handlers:
	    self.handlers.pop(-1).destroy()
	del self.handlers
	del self.secrets_team
	del self.game.phase #?
   

class Marines_Interrupt(UIinput.InputHandler, Phase): #hmm, *must* we inherit from InputHandler?
    """This starts when marine player presses space during the stealer action phase.

    One marine who saw the last stealer move occur is allowed one action.
    We need to inherit from Phase as well because really this is like a small version of
    a Marine_Action_Phase.

    In the first instance this should be treated like an InputHandler; i.e. pass the
    parent *phase* to the constructor.  However, don't handle() this object; loop() it.

    Don't worry, none of the important methods are directly inherited.
    
    Attributes:
    (class)
	isinstance -- bool
    (instance)
	stealer -- Piece -- the target
	_usablemarines - spritegroup
	"""

    name = "Marines Interrupt"
    _secrets_team_type = MARINETEAM
    isinterrupt = True #marker for donebutton


    class GetInterruptMarine(UIinput.InputHandler):
	"Pick the interrupting marine, in the case when there are 2 or more candidates."
	
	def __init__(self, interphase):
	    UIinput.InputHandler.__init__(self, interphase)
	    sprites.ScrollMessage("Pick a marine.")
	    self.spritegroups = (self.phase._usablemarines,)
	    
	def handle(self):
	    select = self.get()
	    if select:
		self.phase.select(select[0])
		self.destroy()
		
	def destroy(self):
	    self.phase._killchoice()
	    UIinput.InputHandler.destroy(self)


    def __init__(self, phase):
	
	assert phase.interruptable
	
	UIinput.InputHandler.__init__(self, phase)
	self._usablemarines = pygame.sprite.Group()
	self.stealer = self.phase.game.get_actor() #could be None
	self.phase.unselect()
	self._select_grp = self._SelectGroup()
	self.game = self.phase.game
	
	# Eliminate pointless cases
	if not (self.phase.game.cpclock.cpremaining and \
		self.stealer.__class__ is sprites.Genestealer and \
		self.stealer.alive()): 
	    self.destroy()
	    return
	
	# Find the interrupting marine
	Phase.__init__(self, self.game)
	self.game.cpbutton.status = PRESSED #graphical effect only
	for m in self.game.marines.sprites():
	    if m.can_see(self.stealer.square):
		self._usablemarines.add(m)
		sprites.HiLite(m.square, REDHILITE).tt_text = lambda: "Use this Marine to interrupt"
		
	if not self._usablemarines: #no-one saw:
	    self._end()
	    return
	elif len(self._usablemarines) == 1:
	    self.select(self._usablemarines.sprites()[0])
	    self._killchoice()
	else:
	    self.unselect()
	    self.piece = None
	    self.GetInterruptMarine(self)
	sprites.ScrollMessage("Marines INTERRUPT!")


    def _killchoice(self):
	"Take off the marine-choosing decorations."
	
	for m in self._usablemarines.sprites():
	    m.square.kill_hilite()
	self._usablemarines.empty()
    
    
    def loop(self):
	
        p = self.cpiece()
        # these tests used to go status,handlers,not p, p.alive - TW
	if not p: 
	    print >> DEBUG_STREAM, "Marines_Interrupt --> loop, no handlers, no cpiece! BUG!" #DEBUG
	    # We'll just end for the moment.  TODO: work out why/when this happens.
	    self._end()
	elif not p.alive():
	    print >> DEBUG_STREAM, "Marines_Interrupt --> loop, cpiece is dead" #DEBUG
	    self._end() #this can be rolled up using Null - TW
        elif self.game.donebutton.status == PRESSED or (self.game.get_actor() is p): # and p is not util.Null): 
	    # Pressing the donebutton is how a marine player cancels an interrupt
	    self._end()
	elif self.handlers: 
            self.handlers[-1].handle()
	else: 
            UIinput.Use_A_Piece(self, p)


    def destroy(self):
	
	self.game.cpbutton.status = DISABLED
	self.game.donebutton.status = ENABLED
	self.unselect()
	if self.stealer and self.stealer.alive(): 
            self.phase.select(self.stealer) #hmm, this is crass
	self._killchoice()
	del self._usablemarines
	del self.stealer
	del self._select_grp
	UIinput.InputHandler.destroy(self)
	
	
    def _end(self):
	
	Phase._end(self)
	self.game.phase = self.phase
	self.destroy()
	

class Choose_Players(Phase):
    """Put up a players dialog.
    
    Sets up player modules.
    Not a 'true phase'.
    
    The aim is to assign a player to each team.
	"""

    pausable = False
    _secrets_team_type = None
    

    def __init__(self, sulkgame):
	
        import players  #EVIL KLUDGE
                        # importing the parent module, how evil is that? - TW
                        
        playersdict = players.players_available
        
	print >> DEBUG_STREAM, "Choose_Players --> init --> players available:" #DEBUG
	pprint.pprint(playersdict, DEBUG_STREAM) #DEBUG

	# Tidy screen
	sulkgame.clear_screen()
	pygame.display.update()
	
	Phase.__init__(self, sulkgame)
        
        # Prepare widgets
        def twidget(teamtype):
            "Make and return a player choice Widget assigning player to team."

            choices = playersdict[teamtype]
            theteam = self.game.teams[teamtype]
            l = len(choices)
            if l < 1:
                raise UserWarning, "Smooshy teamtype error" #DEBUG
            elif l == 1:
                # No choice
                select = choices[0]
                theteam.player = select
                return sprites.CRTextWidget(theteam.type + ": must be used by " + select.name + ".")
            else:
                # Choice
                choicemap = {}
                def okfunc(choice):
                    def f():
                        theteam.player = choice
                    return f
                for c in choices:
                    choicemap[(theteam.type + ": " + c.name, c.mnemonic)] = okfunc(c)
                return sprites.RadioButtonWidget(choicemap)

        d = sprites.Dialog()
        
        d.add_widget(sprites.MTextDisplayWidget(
        (APP_AND_VERSION_STRING,(COPYRIGHT_STRING,"Who is playing?"))))
        for k in playersdict.keys(): d.add_widget(twidget(k))
        d.add_widget(sprites.HorizRuleWidget())
        d.add_widget(sprites.ButtonWidget("Exit", "E", self.game.end, True))
        d.add_widget(sprites.ButtonWidget("OK", "O", d.ok, True)) #stereotype
        
        d.pack()

        # Get an InputHandler
        UIinput.Handle_A_Dialog(self, d)


    def _end(self):

        Phase._end(self)
        Choose_The_Mission(self.game)
	
        
class Choose_The_Mission(Phase):
    """Put up a mission dialog.
    
    Sets up self.game.mission module object.
    Not a 'true phase'.

    Attributes:
	mission -- a module -- the chosen mission
	"""

    pausable = False
    _secrets_team_type = None
    

    class MissionDialog(UIinput.Handle_A_Dialog):
	"""Makes a dialog for choosing between missions."""
	
	def __init__(self, phase, familydict, familyname):
            
	    assert familydict
	    
	    def f(j):
		"Make and return a mission choice callback for a ClickBox."
		def func():
		    phase.mission = familydict[j]
		return func

            d = sprites.NewDialog()
            d.add_widget(sprites.MTextDisplayWidget(
            ("Mission family: '%s'."%familyname,("Which mission would you like to play?",))))
	    for i in familydict.keys(): 
		d.add_widget(sprites.ButtonWidget(str(i), str(i), f(i), True))
            d.add_widget(sprites.HorizRuleWidget())        
            d.add_widget(sprites.ButtonWidget("Back", "b", phase._familydialog, True))
            #d.add_widget(sprites.ButtonWidget("Exit", "e", phase.game.end, True))		
            d.pack()
            
	    UIinput.Handle_A_Dialog.__init__(self, phase, d)
				
				
    #loop() is inherited
    
    def __init__(self, sulkgame):
	
	print >> DEBUG_STREAM, "Choose_The_Mission --> init --> missions.missions_available:" #DEBUG
	pprint.pprint(missions.missions_available, DEBUG_STREAM) #DEBUG

	self.mission = None

	#Tidy screen
	sulkgame.clear_screen()
	pygame.display.update()
	
	Phase.__init__(self, sulkgame)
	if len(missions.missions_available) > 1: 
	    self._familydialog()
	else: 
	    self.MissionDialog(self, missions.missions_available[missions.missions_available.keys()[0]],\
			       missions.missions_available.keys()[0])
	
    def _familydialog(self):
	
	def f(j):
	    "Make and return a family choice callback for a ClickBox."
	    def func():
		self.MissionDialog(self, missions.missions_available[j], j)
	    return func
	
        familynames = missions.missions_available.keys()
        
        d = sprites.NewDialog()
        
        d.add_widget(sprites.MTextDisplayWidget(
        ("Which mission family?",())))
        
	mnemonic_strings_used = ["e","b"]
	for n in familynames:
	    for letter in n:
		if letter.lower() not in mnemonic_strings_used:
		    mnemonic_strings_used.append(letter.lower())
		    mnem = letter
		    break
	    else: key = None
	    d.add_widget(sprites.ButtonWidget(n, mnem, f(n), True))
        d.add_widget(sprites.HorizRuleWidget())
        def goback():
            self._end()
            Choose_Players(self.game)
        d.add_widget(sprites.ButtonWidget("Back", "b", goback, True))
        #d.add_widget(sprites.ButtonWidget("Exit", "e", self.game.end, True))
        
        d.pack()

        # Get an InputHandler
        UIinput.Handle_A_Dialog(self, d)


    def _end(self):
	
	self.game.mission = self.mission
	del self.mission
	Phase._end(self)
	#NB SulkGame.__init__ makes the Pregame phase.
	
       	
class Pregame_Phase(Phase):
    """Present the mission information.
    
    Not a true phase.  Uses superclass loop() method.
    Starts the phase cycle.
    """

    pausable = False
    _secrets_team_type = None
    
    
    def __init__(self, sulkgame):
	
	Phase.__init__(self, sulkgame)

	gm = sulkgame.mission
	
	# Make comments 
	sulkgame.board.build_comments()
	
	def mish_story():
	    #subdialog
            d = sprites.NewDialog()
            d.add_widget(sprites.MTextDisplayWidget(
            ("Mission: '%s'" % gm.NAME,(gm.STORY,))))
            d.add_widget(sprites.HorizRuleWidget())        
            d.add_widget(sprites.ButtonWidget("Back", "b", start_menu, True))
            d.pack()
            
	    # N.B. use ` %s' to fix the kerning for Heidelberg font
	    UIinput.Handle_A_Dialog(self, d)
	   
	def start_menu():
            d = sprites.NewDialog()
            d.add_widget(sprites.MTextDisplayWidget(
            ("Mission: '%s'" % gm.NAME,(
            "Author: %s" % gm.AUTHOR, gm.INFO, 
            "What do you want to do?",))))
            d.add_widget(sprites.HorizRuleWidget())        
            d.add_widget(sprites.ButtonWidget("Back", "b", sulkgame.restart, True))
            d.add_widget(sprites.ButtonWidget("Mission story", "s", mish_story, True))
            d.add_widget(sprites.ButtonWidget("Go", "g", self._end, False))
            d.add_widget(sprites.ButtonWidget("Exit", "e", sulkgame.end, True))
            d.pack()
            
	    UIinput.Handle_A_Dialog(self, d)
	
	# Add a handler
	start_menu()


    def _end(self):
	
	# Kill the comment sprites
	self.game.board.kill_comments()
	    
	# First turn begins
	self.game.turnnumber = 1

	Phase._end(self)
	Marine_Deploy_Phase(self.game)

 
class Marine_Deploy_Phase(Phase):
    """Marines deploy at the beginning of the mission.
    
    Not a true phase.
    
    Attributes:
	reinflist -- list -- a copy of a mission instance's MARINES list attribute
	"""

    name = "Marine Deploy Phase"
    _secrets_team_type = None #No marine secrets here!


    def __init__(self, sulkgame):
	
	Phase.__init__(self, sulkgame)
	self.reinflist = list(self.game.mission.MARINES)
	self.squadlist = []
	print >> DEBUG_STREAM, "Marine_Deploy_Phase --> init, self.reinflist:", self.reinflist #DEBUG
	
	
    def loop(self):
	
	# We iterate through the mission.MARINES array 'depth first' making marines
	
	if self.handlers: self.handlers[-1].handle()
	
	elif self.squadlist: #(a while loop really)
	    # Get the next piece class
	    atuple = self.squadlist.pop()
            astring = atuple[0]
            ateamstring = atuple[1]
            try:
                deploying = class_deploy_map[astring]
	    except KeyError: 
		raise SystemExit, "Bug! Marine_Deploy_Phase --> loop, unknown marine type", astring #DEBUG
            
	    # Which team shall deploy the piece?
	    if ateamstring == MARINETEAM: 
                deployteam = self.game.marines
	    elif ateamstring == STEALERTEAM: 
                deployteam = self.game.stealers
	    else:
		raise SystemExit, "Bug! Marine_Deploy_Phase --> loop, unknown deployteam type", ateamstring #DEBUG

	    # Call the level 2 handler
	    #print >> DEBUG_STREAM, "Marine_Deploy_Phase --> loop, new piece, self.squadname:", self.squadname #DEBUG
	    if deploying is sprites.C_A_T:
                catmethod = atuple[2]
		deployteam.player.UIinput.Deploy_A_New_CAT(self, deployteam, squadname=self.squadname, method=catmethod)
	    else:
		deployteam.player.UIinput.Deploy_A_New_Piece(self, deploying, deployteam, squadname=self.squadname)

	elif self.reinflist:
	    squad = self.reinflist.pop()
	    self.squadname = squad[0]
	    self.squadlist = list(squad[1])

	else: self._end()


    def _end(self):
	
	Phase._end(self)
	del self.reinflist
	self.game.stealers.player.phases.Stealer_Deploy_Phase(self.game)


class Stealer_Deploy_Phase(Phase):
    """Stealers deploy at beginning of mission.
    
    Not a true phase.
    
    Attributes:
	reinfnum -- int
	"""

    name = "Stealer Deploy Phase"
    _secrets_team_type = STEALERTEAM    


    def __init__(self, sulkgame):
	
	Phase.__init__(self, sulkgame)
	
	reinflist = []
	for i in range(self.game.mission.BLIPS[0]):
	    reinflist.append(self.game.get_blipval())
	    
	if not reinflist:
	    sprites.ScrollMessage("No Stealers to deploy.")
	    self._end()
	
	self.reinflist = reinflist


    def loop(self):
	
	if self.handlers: self.handlers[-1].handle()
		
	elif self.reinflist:
	    # New handler, taking the first element
	    self.game.stealers.player.UIinput.Deploy_A_New_Piece(self, sprites.Blip, None)
	    
   	else: self._end()


    def _end(self):
	
	Phase._end(self)
	if hasattr(self, 'reinflist'): del self.reinflist

	sprites.ScrollMessage("Deploying over.")
	
	# The first turn is about to begin
	Clock_and_CP_Phase(self.game)


class Clock_and_CP_Phase(Phase):
    """Prepares for Marine clock and CP counter.

    A true phase, but with no loop().
    This is the first phase in the turn phase cycle (clock_cp, maction, reinf, saction, end)
    """

    pausable = False
    _secrets_team_type = None
    

    def __init__(self, sulkgame):
	
	Phase.__init__(self, sulkgame) #?

    	# Get the clock/cp bonuses
	timer_bonus = cp_bonus = 0
	if options.MARINE_BEGINNER: timer_bonus += MARINE_BEGINNER_TURNTIMER_BONUS
	for piece in sulkgame.marines.sprites():
	    timer_bonus += getattr(piece, 'timer_bonus', 0)
	    cp_bonus += getattr(piece, 'cp_bonus', 0)
	
	# That's it
	Phase._end(self) #?
	sulkgame.marines.player.phases.Marine_Action_Phase(sulkgame, timer_bonus, cp_bonus) #NEW API

       
class Marine_Action_Phase(Phase):
    """While the button is not pressed and time remains,
    or if he is doing something involved (like deploying stealers), sprites.Marine player uses pieces.
    
    Attributes:
	_turntimer -- sprites.TurnTimer instance
	"""

    _secrets_team_type = MARINETEAM
    

    def __init__(self, sulkgame, timer_bonus, cp_bonus):
	
	Phase.__init__(self, sulkgame)
        self._turntimer = sprites.TurnTimer(TURNTIMER_BASE_TIME + timer_bonus)
	sprites.ScrollMessage("Timer starts.") #TODO -- add sound
	self.game.cpclock = sprites.CPClock(self.game.marines, random.randrange(1,7) + cp_bonus)
	self.game.donebutton.status = ENABLED
	self.game.cpbutton.status = ENABLED
	self.unselect()
	self.game.marines.status = ACTIVE
	sprites.ScrollMessage("Now Marines move.")
	# Catch blips revealed by the dispersal of unpersistent features e.g. sprites.Flames instances
	sprites.functions.discover_blips(self.game.marines.sprites())
   
   
    def loop(self):
        
	p = self.cpiece()
        
	#TODO: pause turn clock if we are in a mode under stealer control (i.e. facing stealers.)	
	if self.game.donebutton.status == PRESSED or self._turntimer.expired or not self.game.marines: 
	    #TODO -- add an 'out of time!' sound
	    self._end()
	elif self.handlers: 
            self.handlers[-1].handle()	
	elif not (p and p.alive()): 
	    # Try to go back to previous cpiece -- this is important behaviour
	    for m in self.game.marines.sprites():
		if m.status == ACTIVE: #N.B. up to one of these
		    self.game.marines.player.UIinput.Use_A_Piece(self, m)
		    break
	    else:
		# Ensure that we keep selecting pieces
		self.game.marines.player.UIinput.Select_A_Piece(self, self.game.marines)
	else: 
	    if (p.status != ACTIVE and self.game.cpbutton.status != PRESSED)\
	    or p.get_lurking():
		# The CPs have been turned off, or cpiece has started lurking
		print >> DEBUG_STREAM, "Marine_Action_Phase --> loop, unselecting" #DEBUG
		self.unselect() #hmm
		return
	    else:
		print >> DEBUG_STREAM, "Marine_Action_Phase --> loop, no handlers, cpiece exists" #DEBUG
		self.game.marines.player.UIinput.Use_A_Piece(self, p)
    
    
    def _end(self):
	
	self.unselect() #?
	
	if self._turntimer.expired: sprites.ScrollMessage('Out of time!')
	else: sprites.ScrollMessage('Marines done moving.')
	self.game.marines.status = INACTIVE
	self._turntimer.kill()
	del self._turntimer
	
	self.game.donebutton.status = DISABLED
	self.game.cpbutton.status = DISABLED #so the stealer player can't press it
	
	# Execute special mission events
	msg = self.game.mission.maction_end_script(self.game)
	if msg: sprites.ScrollMessage(msg)

        # Quick victory check
	gameover = self.game.game_over_check()
	
	# Update comments
	self.game.board.refresh_comments()

	if gameover: return
    
	# Prep next phase
	for marine in self.game.marines.sprites(): 
	    marine.status = USED #shouldn't be necessary
	    # Reset the fire bonus
	    marine._fire_bonus = 0
	    # Unset move-and-shoot
	    marine.try_mns = 0
	    marine.may_mns_shoot = 0
	    
	Phase._end(self) 
	self.game.stealers.player.phases.Stealer_Reinforce_Phase(self.game)


class Stealer_Reinforce_Phase(Stealer_Deploy_Phase):
    """While there are reinforcements remaining, Stealer player deploys blips.
    """

    _secrets_team_type = STEALERTEAM
    

    def __init__(self, sulkgame):
	
	self._started = False
        self.ambush_counter = False
        Phase.__init__(self, sulkgame)
        if self.game.mission.BLIPS[1] and (USE_AMBUSH_COUNTERS in self.game.mission.FLAGS):
            # Ambush counters allowed,
            # but only if there are not two already.
            num = 0
            for piece in self.game.stealers.sprites():
                if piece.type is AMBUSH_COUNTER:
                   num += 1
                   if num == 2:
                       self._start()
                       break
            else:
                self._started = False
                self.game.stealers.player.UIinput.Ambush_Counter_Dialog(self)
            return
        self._start()

                
    def loop(self):
	
	if self.handlers: 
            self.handlers[-1].handle()

        elif not self._started:
            self._start()
		
	elif self.reinflist:
	    # New handler, taking the first element
	    self.game.stealers.player.UIinput.Deploy_A_New_Piece(self, sprites.Blip, None)
	    
   	else: self._end()
	
        
    def _start(self):

        self._started = True
        reinflist = []
        num = int(self.game.mission.BLIPS[1])
        if self.ambush_counter:
            num -= 1
	for i in range(num):
	    reinflist.append(self.game.get_blipval())    
	if not reinflist:
	    sprites.ScrollMessage("No Stealers to reinforce.")
	    self._end()
	    return
	
	self.reinflist = reinflist
        sprites.ScrollMessage('Now Stealers reinforce.')
       
       
    def _end(self):
	
	Phase._end(self)
	if hasattr(self, 'reinflist'): del self.reinflist
	sprites.ScrollMessage("Reinforcement over - now Stealers move.")
	self.game.stealers.player.phases.Stealer_Action_Phase(self.game, self.ambush_counter)


class Stealer_Action_Phase(Phase):
    """While the button is not pressed, Stealer player uses pieces.
    """
    _secrets_team_type = STEALERTEAM
    interruptable = True
    name = "Stealer Action Phase"
    
    def __init__(self, sulkgame, ambush_counter):
        
	self.ambush_counter = ambush_counter
	Phase.__init__(self, sulkgame)
	self.game.stealers.status = ACTIVE
	self.game.donebutton.status = ENABLED
        self.done_deploying_ambush_counter = False
        self.deploying_ambush_counter = False


    def loop(self):
        
        if self.game.donebutton.status == PRESSED or\
        self.done_deploying_ambush_counter or\
        (not self.deploying_ambush_counter and not self.game.stealers): 
            self._end()
        elif self.handlers: 
            self.handlers[-1].handle()
        elif self.cpiece().alive() and self.cpiece().team.status == ACTIVE:
            print >> DEBUG_STREAM, "Stealer_Action_Phase --> loop, no handlers, cpiece exists and team active" #DEBUG
            self.game.stealers.player.UIinput.Use_A_Piece(self, self.cpiece())
        else:
            self.game.stealers.player.UIinput.Select_A_Piece(self, self.game.stealers)


    def _end(self):
	
        self.game.stealers.status = INACTIVE
        self.unselect()
        
        if self.done_deploying_ambush_counter:
            self.ambush_counter = False
            
	if self.ambush_counter:
            UIinput.Deploy_A_New_Piece(self, sprites.Ambush_Counter)
            #sprites.ScrollMessage('Stealers deploy an Ambush Counter.')
            self.done_deploying_ambush_counter = False
            self.deploying_ambush_counter = True
            self.game.donebutton.status = ENABLED #?
            return
        
	self.game.donebutton.status = DISABLED
	Phase._end(self)
        sprites.ScrollMessage('Stealers done moving.')    
        End_Phase(self.game)


class End_Phase(Phase):
    """Cleanup, check for victory.

    Only persistent if there are no special FLAGs.
    Features which are nonpersistent are removed, marines come off overwatch etc.
    """
    
    #RULES--> is it better to reveal the CPs, or to leave an element of mystery?
    
    pausable = False
    _secrets_team_type = None
    
    
    def __init__(self, sulkgame):
	
	Phase.__init__(self, sulkgame)
	
	# Stop interrupts
	self.game.set_actor()
	
	if CAT in self.game.mission.FLAGS:
	    
	    self._cats = []
	    for f in self.game.features.sprites():
		if isinstance(f, sprites.C_A_T) and f.may_wander():
		    self._cats.append(f)

	    if self._cats: 
		self.game.donebutton.status = ENABLED
		return
	    
	self._end()

    
    def loop(self):
	
	if self.game.donebutton.status == PRESSED: self._end()
	elif self.handlers: self.handlers[-1].handle()
	elif self._cats:
	    #TODO port to player module API
	    # Add a handler, pop a cat
	    UIinput.Move_A_CAT(self, self._cats.pop())
	else: self._end()
	
 
    def _end(self):

	game = self.game
	
	# Mission-scripted events
	msg = game.mission.end_script(game)
	if msg: sprites.ScrollMessage(msg)

	# Check that game is still going.  If not, punt
	# (SulkGame will set up PostGame_Phase for us)
	if game.game_over_check(): return
        
	# Tidying
	game.cpclock.kill()
	game.cpclock = None
	
	# Stop interrupts
	game.set_actor()
	
	# Increment turn
	sprites.ScrollMessage("Turn %d ends." % game.turnnumber)
	game.turnnumber += 1
    
	# Refresh pieces
	game.marines.refresh()
	game.stealers.refresh()
	game.neutrals.refresh()
	game.board.refresh_comments()
	
	# Kill features
	for feature in game.features.sprites():
	    if not feature.persist: feature.kill(NODEATH)
	    
	# Phase over
	Phase._end(self)
	
	# Restart the turn loop
	Clock_and_CP_Phase(game)


class Postgame_Phase(Phase):
    """Display winner.
    
    Not a true phase.
    """
    
    pausable = False
    _secrets_team_type = None
    
    
    def __init__(self, sulkgame, victorystring):

	Phase.__init__(self, sulkgame)

	sprites.ScrollMessage(victorystring)
	
        d = sprites.NewDialog()
        d.add_widget(sprites.MTextDisplayWidget(
        ("Game over",(
        victorystring, "Would you like to play again?"))))
        d.add_widget(sprites.HorizRuleWidget())        
        d.add_widget(sprites.ButtonWidget("No", "n", self.game.end, True))
        d.add_widget(sprites.ButtonWidget("Yes", "y", self.game.restart, True))
        d.pack()
        
        UIinput.Handle_A_Dialog(self, d)
