# Sulk - sprites - exotic

#*****************************************************************************
#
#                                   Sulk
#
#       Copyright (c) 2002 Toby Woodwark <zirtix@softhome.net>
#
# This program is free software.  See the file COPYING for details.
#
#*****************************************************************************

"""
Sulk optional graphic classes (not used in all missions).
"""

##############################################################################
####### Import / set up

## Modules
import pygame

## My modules
import util #for Null
from constants import *  
import pieces
import misc
import textual


##############################################################################

class C_A_T(misc.Feature, pieces.Piece):
    """Represents the C.A.T., a "cyber-altered task unit".
    
    Can sometimes be moved about Piecelike (autonomously).  Counts as a Feature
    for most game purposes.  When there is a piece holding me, I follow that 
    piece as one of its possessions.

    Is a member *either* of a Piece's 'possessions' group, *or* a Square's 'features'
    group.

    Attributes:
	damaged -- bool -- if true, any more damage will destroy me
	piece	-- a Piece or None -- the Marine carrying me, if any
	"""
	
    #work in progress

    #TODO -- think about UI and graphics more

    render_layer = 3
    update_auto = False #I am updated by Piece.possessions.update()

    tt_text = lambda self: self.type #?

    # Feature attributes..
    blocksmove = False
    persist = True
    blockslos = False
    fl_kill_scorereq = 1 #'killed automatically' (hmm, this will still show a die)
    sh_kill_scorereq = None #may not be shot
    
    # Piece attributes...
    type = CAT
    #NB 'team' will come from sulk.py (used to be None)
    ap = 3
    cc = None #may not be attacked directly
    _movemap = { F_L:1, F:1, F_R:1, L:1, R:1, B_L:1, B:1, B_R:1 }
    _turnmap = { } #always face UP
    _keybturndict = { }
    _button_class_list = ( )
    usedoor_cost = None
    _cc_cost = None

    kill = pieces.Piece.kill

    can_turn = util.Null	#can never turn (RULES--> turns for free(!))
    _post_action = util.Null


    def __init__(self, square):

	misc.Feature.__init__(self, square, UP)
	#N.B. Piece init not called!
	
	# Add to neutral team
	self.add(self.team)

	# Special attributes
	self.damaged = False
	self.piece = None
	
	# Piece attributes...
	self.refresh()

	# Create spritegroups for attachables
	self._markers	= pygame.sprite.GroupSingle()
	self.buttons	= pygame.sprite.Group() #not used
	self._sel	= pygame.sprite.GroupSingle()
	self._clocks	= pygame.sprite.Group() #?

    
    def update(self):

	if self.square.get_flaming(): self.kill()
	elif self.piece: 
	    self.square.features.remove(self) #overkill...
	    self.square = self.piece.square
	    self.rect = self.square.rect
	else: 
	    self.square.features.add(self) #overkill...
	    self.rect = self.square.rect
	self._clocks.update()
	if self._markers: self._markers.sprite.update()

    
    def kill(self, style=SILENT):
	
	if self.alive():
	    self.kill_visuals() #?
	    
	    #TODO implement sounds
	    for m in self._markers.sprites():
		m.kill()

	    misc.Feature.kill(self, style)

	    
    def get_fumbled(self):
	"""get_fumbled() -> bool

	Randomly damage or release cat from a marine.
	Returns 1 if the cat is dropped."""
	
	d = textual.DisplayDie().pips
	if d == 2: 
	    self.drop()
	    textual.ScrollMessage("C.A.T. is fumbled!") #?
	    return 1
	else:
	    if d == 1:
		self.damage()
	    return 0

	
    def drop(self):
	"""drop() -> None
	
	Hit the floor."""

	if self.piece: self.square = self.piece.square
	self.set_piece()
	self.update()


    def dead_drop(self):
	"""drop() -> None
	
	Hit the floor when my piece dies."""

	self.drop()
	
	d = textual.DisplayDie().pips
	if d == 1:
	    self.kill()
	elif d in (2,3):
	    self.damage()
	
	
    def set_piece(self, piece=None):
	"""set_piece(piece=None) -> None

	Attach to a piece.
	If None is passed, detach from self.piece (if any)."""
	
	print >> DEBUG_STREAM, "C_A_T --> set_piece, piece is:",piece #DEBUG
	
	if self.piece: 
	    self.piece.possessions.remove(self)
	self.piece = piece
	if piece: 
	    piece.possessions.add(self)
	    self.square.features.remove(self)

    
    def may_wander(self):
	"""may_wander() -> bool
	
	Returns True if I can wander autonomously (in endphase)"""

	if self.piece is None and not self.damaged and self.alive(): 
	    return True
	else: 
	    return False
	
    
    def damage(self, style=SKEWERED):
	"""damage(style=SKEWERED) -> None

	Place a marker or kill self."""
	
	print >> DEBUG_STREAM, "C_A_T --> damage, self.damaged:",self.damaged #DEBUG
	
	if self.damaged: 
	    self.kill(style)
	else:
	    self._markers.add(misc.DamageMarker(self))
	    self.damaged = True
	    
	
    def can_move(self, square):
	
	# (catmove argument)
        if self.square.get_routable_to(square, catmove=True): 
            return True


    def move(self, square):
	#code copying from Piece... kind of
	
	#self.movesound.play() #TODO
        self.square.features.remove(self)
        self.square = square
        self.update()
