# Sulk - sprites - graphic

#*****************************************************************************
#
#                                   Sulk
#
#       Copyright (c) 2002 Toby Woodwark <zirtix@softhome.net>
#
# This program is free software.  See the file COPYING for details.
#
#*****************************************************************************

"""
Sulk basic graphic class.
"""

#######################################################################################
####### Import

import pygame

from constants import DEBUG_STREAM, MAP_RECT


#######################################################################################
####### Base graphics classes

class Graphic(pygame.sprite.Sprite):
    """Parent class.
    
    Attributes:
    (class)
	render_container -- spritegroup -- used by rendering routines
	render_layer	-- int -- controls render order
	render_clip	-- Rect or None -- where the graphic is drawn (None if can be drawn anywhere)
	update_auto	-- bool	-- whether to update automatically (i.e. every tick) in game loop
	fps		-- float -- current game fps
	elapsed		-- int -- the length in msec of the last game tick
	speed_ratio	-- float -- how slow the game is running, from 1.0 up (1.0 = normal,
			   2.0 = half speed, etc) 
    (instance/class)
	tt_text		-- *function* or None -- called by ToolTip if available, returns string

    Render layers info:
	0=squares, background, buttons, turnclock, scrollmessages
	1=basic features
	2=flames
	3=C_A_Ts
	4=doors, pieces
	5=Selectdec, facing arrows, hilites
	6=markers
	    <gap>
	8=HUD (Clocks, dice)
	    <gap for future stuff>
	10=dialogs
	11=tooltips
	"""
	
    # Render group should be worked out 'dynamically' from render_clip and
    # render_layer attribute (this is done in objects.rendering)
    
    render_layer = 0	# 
    render_clip = None	# defaults
    update_auto = False	#
    tt_text = None	# not tooltip-able
    
	
    def __init__(self, *groups):
	
	pygame.sprite.Sprite.__init__(self)
	
	self.show()
	
	# Add self to any additional groups
	self.add(groups)

    
    def hide(self):
	"""hide() -> None
	
	Remove self from render group"""

	self.remove(self.render_container)


    def show(self):
	"""show() -> None
	
	Add self to render group"""
	
	self.add(self.render_container)

	
class BoardGraphic(Graphic):
    """Parent class for game objects and anything that attaches
    to the game board.

    This does not include certain things that attach to Squares (for example
    Pieces) because their 'rect' attribute is identical with that of their
    Square.
    
    Attributes:
    (class)
	board_container -- a large spritegroup -- I automatically add myself to this
	"""

    render_clip = MAP_RECT


    def __init__(self, *groups):

	Graphic.__init__(self, self.board_container, *groups)
