# Sulk - sprites - textual

#*****************************************************************************
#
#                                   Sulk
#
#       Copyright (c) 2002 Toby Woodwark <zirtix@softhome.net>
#
# This program is free software.  See the file COPYING for details.
#
#*****************************************************************************

"""
Sulk graphics involving text.
"""

##############################################################################
####### Import

## Modules
import pygame, random

## My modules
import util 
from constants import *  
import graphic


##############################################################################
####### Timer

class TurnTimer(graphic.Graphic):
    """Times out Marine turns.

    Up to one exists at once.
    
    Attributes:
    (class)
	elapsed_group	-- pygame.sprite.Group -- group (supplied outside this module)
    (instance)
	expired		-- bool
	time		-- int -- milliseconds until expiry
	value		-- tuple -- minutes, seconds left
	lastvalue	-- tuple
	"""

    render_layer = 0
    render_clip = None
    update_auto = True
    
    tt_text = None
    
    def __init__(self, seconds):
	
        graphic.Graphic.__init__(self)
	
        self.time = seconds * 1000
        self.expired = 0
        self.lastvalue = (0,0)
        self.update()
	self.rect = self.image.get_rect()
	self.rect.center = TURNTIMER_RECT.center


    def update(self):
	
        # We assume that we have been given self.elapsed
	# during the tick
	
	if self.expired: return #stick at 0:00
        self.time -= self.elapsed
        if self.time < 0: 
	    self.expired = 1
	    self.time = 0
        self.minutes = int(self.time  / 60000)
        self.seconds = int((self.time - (self.minutes * 60000)) / 1000) #hmm
        self.value = (self.minutes, self.seconds)
        if self.value != self.lastvalue:
            self.lastvalue = self.value
	    if self.seconds < 10: text = "%d:0%d"%(self.minutes,self.seconds)
	    else: text = "%d:%d"%(self.minutes,self.seconds)
            self.image = TURNTIMER_FONT.render(text, 1, TURNTIMER_TEXT_COLOR)
	    

##############################################################################
####### Clocks

class Clock(graphic.Graphic):
    """Displays info in the HUD"""

    render_layer = 8
    render_clip = MAP_RECT
    update_auto = True #FIXME -- use SulkGame.set_actor() or something, this is brutal

    
    
class APClock(Clock):
    """Counts down all APs.

    Only one exists at once.
    
    Attributes:
    (instance)
	piece	    -- a Piece
	lastaps	    -- int
	"""

    #TODO -- perhaps a border that changes color according to piece status 
    #(USED/INACTIVE/ACTIVE)
    
    def __init__(self, piece):
	
	graphic.Graphic.__init__(self)
	self.piece = piece
	self.lastaps = None
	
	
    def update(self):
	
	if self.piece.alive(): 
	    if self.piece.apremaining != self.lastaps:
		self.lastaps = self.piece.apremaining
		text = " AP: %d "%self.piece.apremaining
		self.tt_text = lambda: "Piece has %d action points left"%self.piece.apremaining
		self.image = APCLOCK_FONT.render(text, 1, self.piece.team.color, APCLOCK_BACK_COLOR)
		self.image.set_alpha(CLOCKS_ALPHA)
		self.rect = self.image.get_rect()
		self.rect.topright = APCLOCK_TOPRIGHT
	else: self.kill()


class AmmoClock(Clock):
    """Counts down ammo.
    
    Attributes:
    (instance)
	piece		-- Piece
	lastammo	-- int
	"""
	
    def __init__(self, piece):
	
	graphic.Graphic.__init__(self)
	self.piece = piece
	self.lastammo = None
	
	
    def update(self):
	
	if not self.piece.alive(): 
	    self.kill()
	elif self.piece.ammoremaining != self.lastammo:
	    self.lastammo = self.piece.ammoremaining
	    text = " Ammo: %d " % self.piece.ammoremaining
	    self.image = AMMOCLOCK_FONT.render(text, 1, self.piece.team.color, AMMOCLOCK_BACK_COLOR)
	    self.image.set_alpha(CLOCKS_ALPHA)
	    self.rect = self.image.get_rect()
	    self.rect.topleft = AMMOCLOCK_TOPLEFT

            
class PsipointsClock(Clock):
    """Counts down psipoints.
    
    Attributes:
    (instance)
	piece		-- Piece
	lastpsipoints	-- int
	"""
	
    def __init__(self, piece):
	
	graphic.Graphic.__init__(self)
	self.piece = piece
	self.lastpsipoints = None
	
	
    def update(self):
	
	if not self.piece.alive(): 
	    self.kill()
	elif self.piece.psipointsremaining != self.lastpsipoints:
	    self.lastpsipoints = self.piece.psipointsremaining
	    text = " Psi points: %d " % self.piece.psipointsremaining
	    self.image = AMMOCLOCK_FONT.render(text, 1, self.piece.team.color, AMMOCLOCK_BACK_COLOR)
	    self.image.set_alpha(CLOCKS_ALPHA)
	    self.rect = self.image.get_rect()
	    self.rect.topleft = AMMOCLOCK_TOPLEFT
            

class DeployClock(Clock):
    """Shows which piece is currently to be deployed.
    
    Cannot be displayed with an AmmmoClock instance about.
    
    Attributes:
    (instance)
	_deployer	-- a Deploy_A_New_Piece or Convert_A_Blip instance 
	_last_text	-- string
	"""

    #TODO: subclasses instead of incredible lameness.  
    #(Check out Deploy_A_New_Piece)

    
    def __init__(self, deployer):
	
	graphic.Graphic.__init__(self)
	self._deployer = deployer
	self._last_text = None
	
	
    def update(self):
	
	dp = self._deployer
	piececlass = dp.piececlass
	
	if piececlass.type == BLIP:
	    # (stealer case)
	    if dp.deployteam.secrets_showing:
		text = " %ss: %s left \n Current: %d "%\
		(piececlass.type,
		str(dp.phase.reinflist), 
		dp.value)
	    else:
		text = " %ss: %d left \n Current: *?* "%\
		(piececlass.type,
		len(dp.phase.reinflist))
                
	elif piececlass.type == AMBUSH_COUNTER: #PM
            text = " Deploy an Ambush counter"
            
	elif piececlass.type == CAT:
	    # (cat case)
	    if dp.method == WITHSQUAD:
		text = " The %s deploy a %s \n To squad '%s' "%\
		(dp.deployteam.type, 
		piececlass.type,
		dp.squadname)

	    elif dp.method == ONBOARD:
		text = " The %s deploy a %s "%\
		(dp.deployteam.type,
		piececlass.type)

	elif piececlass.type == GENESTEALER:
	    # (stealer case) -- we are dealing with Conversion
	    if dp.blip.value == 1:
		text = " The %s deploy a %s \n (final one) "%\
		(dp.deployteam.type, piececlass.type)
	    else:
		text = " The %s deploy a %s \n (%d more remaining) "%\
		(dp.deployteam.type, piececlass.type,
		dp.blip.value - 1)

	else:
	    # (marine case)
	    # How many marines remain?
	    num = 1
	    for tuple in dp.phase.squadlist:
		if tuple[0] != CAT:
		    num += 1
	    for squadtuple in dp.phase.reinflist:
		for tuple in squadtuple[1]:
		    if tuple[0] != CAT:
			num += 1
	    text = " Marines: %d left \n The %s deploy a %s "%\
	    (num, 
	    dp.deployteam.type, 
	    piececlass.type)
	    #hmm, squadname?
	
	# Now find updated image (TODO -- stop checking this every fucking tick!)
	if text != self._last_text:
	    self._last_text = text
	    
	    self.image = util.render.para_with_CR(
		DEPLOYCLOCK_FONT, 
		text, 
		0, 
		1, 
		dp.deployteam.color, 
		DEPLOYCLOCK_BACK_COLOR
	    )
	    self.image.set_alpha(CLOCKS_ALPHA)
	    self.rect = self.image.get_rect()
	    self.rect.topleft = AMMOCLOCK_TOPLEFT


class FacingClock(Clock):
    """Show that a piece must be faced by a particular team.
    """
    
    def __init__(self, piece, team):
	
	graphic.Graphic.__init__(self)
	text = " The %s choose a facing \n for this %s "%(team.type, piece.type)
	self.image = util.render.para_with_CR(
	DEPLOYCLOCK_FONT, 
	text, 
	0, 
	1, 
	team.color, 
	DEPLOYCLOCK_BACK_COLOR
	)
	self.image.set_alpha(CLOCKS_ALPHA)
	self.rect = self.image.get_rect()
	self.rect.topleft = AMMOCLOCK_TOPLEFT

    
class CPClock(Clock):
    """Counts down Marine CPs.

    Up to one exists at once, called gs.cpclock.
    Inherits secrecy from self.team.
    
    Attributes:
    (class)
	_color_map		    -- dict of bool:RGBA -- false for normal color, true for secrets revealed
	_back_color_map		    -- dict of bool:RGBA -- as above
    (instance)
	team			    -- SulkTeam -- always a MarineTeam instance
	_last_cpremaining	    -- int
	cpremaining		    -- int
	_last_secrets_showing	    -- bool -- to compare with the current secrecy value
	"""
	
    _color_map = {
    0:CPCLOCK_TEXT_COLOR_NORMAL,
    1:CPCLOCK_TEXT_COLOR_REVEALED}
    _back_color_map = {
    0:CPCLOCK_BACK_COLOR_NORMAL,
    1:CPCLOCK_BACK_COLOR_REVEALED}
    
    
    def __init__(self, team, cps):
	
        graphic.Graphic.__init__(self)
	self.team = team
        self.cps = self.cpremaining = cps
        self._last_cpremaining = None
	
	
    def update(self):
	
        if self.cpremaining != self._last_cpremaining\
	or self.team.secrets_showing != self._last_secrets_showing:
            self._last_secrets_showing = self.team.secrets_showing 
	    self._last_cpremaining = self.cpremaining
	    if self.team.secrets_showing: 
		text = "!cp: %d "%self.cpremaining
		self.tt_text = lambda : "The %s have %d command points remaining"%(self.team.type, self.cpremaining)
	    else:
		used = self.cps - self.cpremaining
		text = " CP: %d "%used
		self.tt_text = lambda : "The %s have used %d command points"%(self.team.type, used)
	    self.image = CPCLOCK_FONT.render(
	    text, 
	    1, 
	    self._color_map[self.team.secrets_showing], 
	    self._back_color_map[self.team.secrets_showing]
	    )
	    self.image.set_alpha(CLOCKS_ALPHA)
	    self.rect = self.image.get_rect()
	    self.rect.topright = CPCLOCK_TOPRIGHT  
  

class DisplayDie(Clock):
    """A decoration that represents a dieroll.
    
    This sprite slowly fades from view and then kills itself (Aaah).
    Many may exist, though hopefully not more than about six on small displays.
    
    Attributes:
    (class)
	min_fade_delta	-- int
	_initial_alpha	-- int
	_kill_alpha	-- int
	most_recent	-- a die or None
    (instance)
	pips		-- int -- the dieroll value (1-6)
	team		-- SulkTeam or None -- to decide color
	"""
    
    _initial_alpha = 255
    _kill_alpha = 35
    min_fade_delta = 2

    most_recent = None
    
    
    def __init__(self, team=None):
	
        graphic.Graphic.__init__(self)
	
	# Get a random dieroll value
        self.pips = random.randrange(1,7)
	
	# Get color
	if team is None: color = NEITHER_COLOR
	else: color = team.color
	
	# Render
        self.image = DISPLAYDIE_FONT.render(" %d " % self.pips, 1, color, DISPLAYDIE_BACK_COLOR)
	self.rect = self.image.get_rect()
	#(border)
	pygame.draw.rect(self.image, color, self.rect, 3) #hmm, aesthetics
	#(alpha)
	self.image.set_alpha(self._initial_alpha)
	#(TODO? prerender or gimp these dice' images, there are only 6 * 3 of them max)
	
	#Position
	if not self.most_recent or not self.most_recent.alive():
	    # 1st visible roll (for a while)
	    self.rect.bottomright = DICE_BOTTOMRIGHT
	else:
	    # There are other dice about
	    self.rect.bottomright = \
	    self.most_recent.rect.bottomleft
	    self.rect.move_ip(-PAD, 0)
	    if not self.render_clip.contains(self.rect):
		# Too many dice -- we're offscreen, correct this
		self.rect.bottomright = DICE_BOTTOMRIGHT
	self.__class__.most_recent = self


    def update(self):
	
	alpha = self.image.get_alpha()
        if alpha < self._kill_alpha: 
	    self.kill()
	else:
	    self.image.set_alpha(alpha - int(self.min_fade_delta * self.speed_ratio))


class FPSClock(Clock):
    """Show FPS at bottom left of screen.
    
    Attributes:
    (instance)
	lastfps -- int
	"""
    	
    def __init__(self):
	
	Clock.__init__(self)
	self.lastfps = None
	
	
    def update(self):
	
	if self.fps != self.lastfps:
	    self.lastfps = self.fps
	    text = str(round(self.fps, 2))
	    self.image = AMMOCLOCK_FONT.render(text, 1, WHITE, GREY1)
	    self.image.set_alpha(CLOCKS_ALPHA)
	    self.rect = self.image.get_rect()
	    self.rect.bottomleft = FPSCLOCK_BOTTOMLEFT


##############################################################################
####### Messages

class SquareComment(graphic.BoardGraphic):
    """Commentary on a board square.
    
    Attributes:
    (class)
	max_text_width	-- int -- pixels
    (instance)
	square		-- a Square
	"""
	
    render_layer = 5
    render_clip = MAP_RECT
    update_auto = False
    
    max_text_width = 4 * SQ_LENGTH
    
    
    def __init__(self, square):

	graphic.BoardGraphic.__init__(self)
	
	self.text = square.comment
	self.square = square
	self.render()


    def render(self):
	"->None"
	
	textimage = util.render.mtext(COMMENT_FONT, 
				    (self.text, ()), 
				    self.max_text_width, 
				    1, 
				    COMMENT_TEXT_COLOR)
	textrect = textimage.get_rect()
	self.image = pygame.Surface(textrect.inflate(2 * HALF, 2 * HALF).size)
	self.rect = self.image.get_rect()
	textrect.center = self.rect.center
	self.image.fill(COMMENT_BACK_COLOR)
	pygame.draw.rect(self.image, COMMENT_EDGE_COLOR, self.rect, 3)
	self.image.blit(textimage, textrect)
	self.image.set_alpha(COMMENT_ALPHA)
	self.rect.center = self.square.rect.center


    def refresh(self):
	"->None"

	if self.text != self.square.comment:
	    self.text = self.square.comment
	    self.render()


class ScrollMessage(graphic.Graphic):
    """A sprite that scrolls leftwards across SCROLLMESSAGE_RECT and kills itself 
    after going offscreen.

    The most recent message hangs around until hit by another.
    
    Attributes:
    (class)
	pad	    -- int -- sideways clearance between messages
	min_deltax  -- int -- optimal sideways movement per tick (at MAX_FRAME_RATE)
	most_recent -- GroupSingle -- my most recent fellow instance 
    (instance)
	text	    -- string
	prev	    -- instance -- the previous instance
	hit	    -- bool -- if true we may not freeze at left of area
	"""

    render_layer = 0
    render_clip = SCROLLMESSAGE_RECT
    update_auto = True

    tt_text = None
    
    pad = 3 * HALF
    min_deltax = - float(SCROLLMESSAGE_FONT.size("M")[0] * 10) / MAX_TICK_RATE

    most_recent = pygame.sprite.GroupSingle()
    
    def __init__(self, text):
	
	# Die straight off if we have the same text as the previous, not-yet-visible message.
	mostrec = self.__class__.most_recent.sprite
	if mostrec\
	and mostrec.text == text and mostrec.rect.right > SCROLLMESSAGE_RECT.right:
            print >> DEBUG_STREAM, "ScrollMessage.__init__, redundant text: '%s'"%text #DEBUG
	    return

	graphic.Graphic.__init__(self)

	# I am now the most recent
	self.prev = mostrec
	self.__class__.most_recent.add(self)
	self.text = text

	self.hit = 0

        self.image = SCROLLMESSAGE_FONT.render(text, 1, SCROLLMESSAGE_TEXT_COLOR)
        self.rect = self.image.get_rect()
	if self.prev:
	    self.rect.left = SCROLLMESSAGE_RECT.union(self.prev.rect).right + self.pad
	else:
	    self.rect.left = SCROLLMESSAGE_RECT.right + self.pad
        self.rect.centery = SCROLLMESSAGE_RECT.centery


    def update(self):
	
	deltax = self.min_deltax * self.speed_ratio
	if not self.hit\
	and self.rect.left + deltax <= SCROLLMESSAGE_RECT.left + PAD\
	and self.rect.right <= SCROLLMESSAGE_RECT.right:
	    self.rect.left = SCROLLMESSAGE_RECT.left + PAD
	else:
	    self.rect.move_ip(deltax, 0)
	    if self.prev and self.rect.left - self.prev.rect.right <= self.pad:
		self.prev.hit = 1
	    if self.rect.right < SCROLLMESSAGE_RECT.left: 
		self.kill()


    def kill(self):
	
	del self.prev
	graphic.Graphic.kill(self)


##############################################################################
####### Tooltip

class ToolTip(graphic.BoardGraphic):
    """Displays info about many classes of Graphic instances.
    
    The target sprite must have a nontrivial tt_text() attribute.
    One at once.
    
    Attributes:
    (class)
	_initial_alpha
	_max_alpha
	_min_fade_rate
    (instance)
	_is_fading_in -- bool
	"""

    render_clip = None
    render_layer = 11
    update_auto = True

    tt_text = None
    
    _initial_alpha = 0
    _max_alpha = TOOLTIP_ALPHA
    _min_fade_rate = 20 #hmm
    
    
    def __init__(self, sprite):
	
	assert sprite.alive()
	assert sprite.tt_text is not None #?

	text = sprite.tt_text()
	
	assert text
	
	# Interesting init kludges to keep ttips in right places
	if isinstance(sprite, Clock)\
	or sprite.render_clip is not MAP_RECT:
	    graphic.Graphic.__init__(self)
	else:
	    graphic.BoardGraphic.__init__(self)
	
	# Render
	textimage = util.render.para_with_CR(
	TOOLTIP_FONT, 
	text, 
	0, #not centered
	1, #AA
	TOOLTIP_TEXT_COLOR)
	textrect = textimage.get_rect()
	image = pygame.Surface(textrect.inflate(HALF, HALF).size)
	rect = image.get_rect()
	image.fill(TOOLTIP_BACK_COLOR)
	
        # border
	pygame.draw.rect(
	image,
	TOOLTIP_EDGE_COLOR,
	rect,
	1)
	
        # top left corner triangle
	pygame.draw.polygon(
	image, 
	TOOLTIP_EDGE_COLOR, 
	((rect.left, rect.top+5), rect.topleft, (rect.left+5, rect.top)), 
	0)
	
	textrect.center = rect.center
	image.blit(textimage, textrect)
	image.set_alpha(self._initial_alpha)

	# Position
	size, hotspot, xormasks, andmasks = pygame.mouse.get_cursor()
	posx, posy = pygame.mouse.get_pos()
	rect.topleft = posx + size[0] - hotspot[0], posy + size[1] + HALF - hotspot[1] 

	# Keep inside the screen
	rect.clamp_ip(SCREEN_RECT)

	# Nail attrs
	self.image = image
	self.rect = rect

	# Start fadein
	self._is_fading_in = True


    def update(self):

	if self._is_fading_in:
	    a = self.image.get_alpha() + int(self.speed_ratio * self._min_fade_rate)
	    if a >= self._max_alpha:
		self.image.set_alpha(self._max_alpha)
		self._is_fading_in = False
	    else:
		self.image.set_alpha(a)
