# Sulk - util - fileutil

#*****************************************************************************
#
#                                   Sulk
#
#       Copyright (c) 2002 Toby Woodwark <zirtix@softhome.net>
#
# This program is free software.  See the file COPYING for details.
#
#*****************************************************************************

"""
Sulk file functions.
"""

##############################################################################
####### Import

import os, pygame 

from constants import \
DEFAULT_THEME_DIRECTORY, PERPIXEL_ALPHA, TOPLEFT_COLORKEY, DEBUG_STREAM,\
options

from null import Null


##############################################################################
####### Classes
    
class SoundWrapper:
    """Simply a wrapper that checks first before playing
    the sound."""
    
    def __init__(self, pysound):

	self._sound = pysound


    def play(self, *args, **kwargs):
	"""play([loops, [maxtime]]) -> Channel"""
	
	if (pygame.mixer.get_init() and options.SOUND_ENABLED):
	    return self._sound.play(*args, **kwargs)

    
    def fadeout(self, millisec):
	"""fadeout(millisec) -> None"""
	
	return self._sound.fadeout(millisec)

    
    def get_num_channels(self):
	"""get_num_channels() -> int"""

	return self._sound.get_num_channels()

 
    def	get_volume(self):
	"""get_volume() -> val"""
	
	return self._sound.get_volume()


    def set_volume(self, val):
	"""set_volume(val) -> None"""
	
	return self._sound.set_volume(val)
	 
    def stop(self):
	"""Sound.stop() -> None"""
	
	return self._sound.stop()
    

##############################################################################
####### Functions

def theme_image(name, colorkey=None):
    """util.load.theme_image(name, colorkey=None -> Surface instance
    load a themed image from the THEME_DIRECTORY and return it, including format conversion
    
    Cascades to the DEFAULT_THEME_DIRECTORY on failure."""

    #TODO nicer themeing
    
    try: img = _loadimg(os.path.join(options.THEME_DIRECTORY, name), colorkey)
    except Exception, message: 
	print >> DEBUG_STREAM, "util.load_theme_image, could not load themed image" #DEBUG
	print >> DEBUG_STREAM, message #DEBUG
	try: img = _loadimg(os.path.join(DEFAULT_THEME_DIRECTORY, name), colorkey)
	except Exception, message: 
	    print >> DEBUG_STREAM, "util.load_theme_image, punt" #DEBUG
	    raise SystemExit, "Themed image error:", message #DEBUG
	else: return img
    else: return img
    

def image(name, colorkey=None):
    """util.load.image(name, colorkey=None) -> Surface instance
    load an unthemed image and return it, including format conversion"""
    
    return _loadimg(os.path.join(options.IMAGE_DIRECTORY, name), colorkey)


def _loadimg(fullname, colorkey=None):
    """_loadimg(fullname, colorkey=None) -> Surface instance
    loads an image and return it, including format conversion
    
    (pygame.display must be initialized.)
    If passed TOPLEFT_COLORKEY, gets the colorkey fom image topleft.
    If passed PERPIXEL_ALPHA, uses perpixel alpha."""

    try:
        image = pygame.image.load(fullname)
    except pygame.error, message:
        print >> DEBUG_STREAM, 'Cannot load image:', fullname #DEBUG
        raise SystemExit, message
    
    if colorkey is not None:
	if colorkey == PERPIXEL_ALPHA:
	    return image.convert_alpha()
        elif colorkey == TOPLEFT_COLORKEY:
            colorkey = image.get_at((0,0))
	image.set_colorkey(colorkey, pygame.RLEACCEL)
    return image.convert()
          

def sound(name):
    """util.load.sound(name) -> SoundWrapper instance
    loads and return a sound if available
    
    (pygame.mixer must be initialized.)
    Returns a useful dummy object if the file is unavailable."""
    
    fullname = os.path.join(options.SOUND_DIRECTORY, name)
    try:
        sound = pygame.mixer.Sound(fullname)
    except pygame.error, message:
	print >> DEBUG_STREAM, "util.load.sound, Cannot load sound '%s':"%fullname, message #DEBUG
        return Null
    return SoundWrapper(sound)
